package br.com.centralit.api.dao.impl;

import java.util.Collection;
import java.util.LinkedList;

import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.CredencialProcessoDao;
import br.com.centralit.api.model.CredencialProcesso;
import br.com.centralit.api.viewHelper.GrupoUsuarioVH;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;
import br.com.centralit.framework.model.Grupo;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 08/03/2016 - 16:45:42
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
@Repository("credencialProcessoDao")
public class CredencialProcessoDaoHibernate extends CitGenericDAOImpl implements CredencialProcessoDao {

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 */
	public CredencialProcessoDaoHibernate() {

		super(CredencialProcesso.class);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por listar grupos e usuarios do sistema
	 * 
	 * @author rogerio.costa
	 * 
	 * @return Collection<GrupoUsuarioVH>
	 */
	public Collection<GrupoUsuarioVH> findGrupoUsuarioByName(String nome) {

		Collection<GrupoUsuarioVH> grupoUsuarioVHList = new LinkedList<GrupoUsuarioVH>();

		SearchSeven searchSevenGrupo = new SearchSeven(Grupo.class);

		searchSevenGrupo.addFilterILike("nome", "%" + nome + "%");

		searchSevenGrupo.setMaxResults(10);

		Collection<Grupo> grupoList = this.search(searchSevenGrupo, Grupo.class);

		SearchSeven searchSevenUsuario = new SearchSeven(Usuario.class);

		searchSevenUsuario.addFilterILike("username", "%" + nome + "%");

		searchSevenUsuario.setMaxResults(10);

		Collection<Usuario> usuarioList = this.search(searchSevenUsuario, Usuario.class);

		for (Grupo grupo : grupoList) {

			grupoUsuarioVHList.add(new GrupoUsuarioVH(grupo.getId(), grupo.getNome(), "-", 1L));

		}

		for (Usuario usuario : usuarioList) {

			grupoUsuarioVHList.add(new GrupoUsuarioVH(usuario.getId(), usuario.getUsername(), UtilObjeto.isReferencia(usuario.getUnidade()) ? usuario.getUnidade().getNome() : "-", 2L));

		}

		return grupoUsuarioVHList;

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter através do id do usuario e processo selecionado
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idProcesso
	 * @param idUsuario
	 * @return CredencialProcesso
	 */
	public CredencialProcesso obterPorUsuarioEProcesso(Long idProcesso, Long idUsuario) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterEqual("processo.id", idProcesso);

		searchSeven.addFilterEqual("usuario.id", idUsuario);

		searchSeven.setMaxResults(1);

		return this.searchUnique(searchSeven, persistentClass);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter através do id do grupo e processo selecionado
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idProcesso
	 * @param idUsuario
	 * @return CredencialProcesso
	 */
	public CredencialProcesso obterPorGrupoEProcesso(Long idProcesso, Long idGrupo) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterEqual("processo.id", idProcesso);

		searchSeven.addFilterEqual("grupo.id", idGrupo);

		searchSeven.setMaxResults(1);

		return this.searchUnique(searchSeven, persistentClass);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar se o usuario contem credencial
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idUsuario
	 * @param idProcesso
	 * @return Boolean
	 */
	public Boolean contemCredencial(Long idUsuario, Long idProcesso) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterEqual("processo.id", idProcesso);

		searchSeven.addFilterEqual("usuario.id", idUsuario);

		return UtilObjeto.isReferencia(this.searchUnique(searchSeven));
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por listar através do id do processo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idProcesso
	 * @return Collection<CredencialProcesso>
	 */
	public Collection<CredencialProcesso> findByIdProcesso(Long idProcesso) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterEqual("processo.id", idProcesso);

		return this.search(searchSeven);

	}

}
