package br.com.centralit.api.dao.impl;

import java.util.Calendar;
import java.util.Collection;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.DocumentoGedDao;
import br.com.centralit.api.model.DocumentoGed;
import br.com.centralit.api.model.SigiloPrivilegio;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.util.UtilObjeto;

import com.googlecode.genericdao.search.Filter;
import com.googlecode.genericdao.search.Sort;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 23/04/2016 - 16:52:47
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
@Repository("documentoGedDao")
@SuppressWarnings("unchecked")
public class DocumentoGedDaoHibernate extends CitGenericDAOImpl implements DocumentoGedDao {

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 */
	public DocumentoGedDaoHibernate() {

		super(DocumentoGed.class);
	}

	@Override
	public Collection<DocumentoGed> findByProcesso(Long idProcesso) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterEqual("processo.id", idProcesso);

		return this.search(searchSeven);
	}

	@Override
	public Collection<DocumentoGed> findByIdProcessoUltimaVersao(Long id) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterEmpty("versaoPosterior");

		searchSeven.addFilterEqual("processo.id", id);

		return this.search(searchSeven);
	}

	@Override
	public Collection<DocumentoGed> findByIdDocumentoTodasVersoes(Long id) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterOr(Filter.equal("id", id), Filter.equal("idRaiz", id));
		searchSeven.addSort(Sort.asc("versaoDocumento"));

		return this.search(searchSeven);
	}

	@Override
	public Collection<DocumentoGed> findByIdDocumentoTodasVersoesAtivas(Long id) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterOr(Filter.equal("id", id), Filter.equal("idRaiz", id));

		searchSeven.addSort(Sort.asc("versaoDocumento"));

		searchSeven.addFilterNotEmpty("dataInativo");

		return this.search(searchSeven);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter documento através do processo, tipoDocumento e numero
	 * 
	 * @author rogerio.costa
	 * 
	 * @param documentoGed
	 * 
	 * @return DocumentoGed
	 */
	public DocumentoGed obterPorProcessoTipoDocumentoENumero(DocumentoGed documentoGed) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterEqual("processo.id", documentoGed.getProcesso().getId());

		searchSeven.addFilterEqual("tipoDocumento.id", documentoGed.getTipoDocumento().getId());

		searchSeven.addFilterEqual("numero", documentoGed.getNumero());

		searchSeven.setMaxResults(1);

		return this.searchUnique(searchSeven);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter o ultimo numero do processo e tipoProcesso
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idTipoDocumento
	 * @param idProcesso
	 * @return
	 */
	public Long obterQuantidadePorTipoDocumentoEDataCriacao(Long idTipoDocumento, Calendar dataInicial, Calendar dataFinal) {

		Session session = (Session) this.em().getDelegate();

		Criteria criteria = session.createCriteria(persistentClass);

		criteria.setProjection(Projections.rowCount());

		criteria.add(Restrictions.eq("tipoDocumento.id", idTipoDocumento));

		criteria.add(Restrictions.between("dataCriacao", dataInicial, dataFinal));

		criteria.add(Restrictions.isNull("versaoAnterior"));

		return (Long) criteria.uniqueResult();
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter através do id do PlanoClassificacao
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idPlanoClassificacao
	 * 
	 * @return Collection<Processo>
	 */
	public Collection<DocumentoGed> findPorPlanoClassificacao(Long idPlanoClassificacao) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterEqual("planoClassificacao.id", idPlanoClassificacao);

		return this.search(searchSeven);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar se o documento é sigiloso através do id
	 * 
	 * @author rogerio.costa
	 * 
	 * @param id
	 * 
	 * @return Sigilo
	 */
	public Boolean isSigiloso(Long id) {

		Criteria criteria = this.newCriteria();

		criteria.setProjection(Projections.id());

		criteria.add(Restrictions.eq("id", id));

		criteria.add(Restrictions.isNotNull("sigilo"));

		criteria.setMaxResults(1);

		return UtilObjeto.isReferencia(criteria.uniqueResult());

	}

	@Override
	public Collection<DocumentoGed> findByProcessoSigilo(Long idProcesso) {

		Long idUsuario = ( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId();

		Criteria criteriaPrivilegio = this.newCriteria(SigiloPrivilegio.class);

		criteriaPrivilegio.setProjection(Projections.property("sigilo.id"));

		criteriaPrivilegio.createAlias("privilegio", "privilegio");

		criteriaPrivilegio.createAlias("privilegio.grupoPrivilegios", "grupoPrivilegio");

		criteriaPrivilegio.add(Restrictions.isNull("grupoPrivilegio.dataInativo"));

		criteriaPrivilegio.createCriteria("grupoPrivilegio.grupo").createAlias("grupoUsuarios", "grupoUsuario");

		criteriaPrivilegio.add(Restrictions.eq("grupoUsuario.usuario.id", idUsuario));

		Criteria criteria = this.newCriteria();

		criteria.add(Restrictions.eq("processo.id", idProcesso));

		criteria.add(Restrictions.isNull("dataInativo"));

		criteria.createAlias("status", "status");

		criteria.add(Restrictions.eq("status.codigo", 0L));

		criteria.add(Restrictions.in("sigilo.id", criteriaPrivilegio.list()));

		return criteria.list();
	}

}
