package br.com.centralit.api.dao.impl;

import java.util.Calendar;
import java.util.Collection;

import javax.persistence.Query;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.PlanoClassificacaoDao;
import br.com.centralit.api.model.PlanoClassificacao;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;
import br.com.centralit.framework.util.UtilDataBase;
import br.com.centralit.framework.util.UtilDate;

import com.googlecode.genericdao.search.Filter;
import com.googlecode.genericdao.search.Search;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 21/10/2015 - 16:02:49
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
@Repository("planoClassificacaoDao")
@SuppressWarnings("unchecked")
public class PlanoClassificacaoDaoHibernate extends CitGenericDAOImpl implements PlanoClassificacaoDao {

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 */
	public PlanoClassificacaoDaoHibernate() {

		super(PlanoClassificacao.class);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por listar a entidade<code>PlanoClassificacao</code> que não contem parent, ou sejá o primeiro nó da arvore
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idOrganizacao
	 * 
	 * @return Collection<PlanoClassificacao>
	 */
	public Collection<PlanoClassificacao> findParent(Long idOrganizacao) {

		SearchSeven search = new SearchSeven(this.persistentClass);

		search.addFilterEqual("organizacao.id", idOrganizacao);

		search.addFilterEmpty("planoClassificacaoParent.id");

		return this.search(search);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por buscar PlanoClassificacao filhas através do id do parent
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idPlanoClassificacao
	 * 
	 * @return Collection<PlanoClassificacao>
	 */
	public Collection<PlanoClassificacao> findChildrens(Long idPlanoClassificacao) {

		SearchSeven search = new SearchSeven();

		search.addFilterEqual("planoClassificacaoParent.id", idPlanoClassificacao);

		return this.search(search);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por retornar a quantidade de filhos por determinado plano
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idPlanoClassificacao
	 * 
	 * @return Integer
	 */
	public Integer getCountChildrens(Long idPlanoClassificacao) {

		Search search = new Search(this.persistentClass);

		search.addFilterEqual("planoClassificacaoParent.id", idPlanoClassificacao);

		return this._count(search);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por listar ultimo nivel ou que possua a opção "permitirUsarComoClassificacao"
	 * 
	 * @author rogerio.costa
	 * 
	 * @param codigoNome
	 * 
	 * @return Collection<PlanoClassificacao>
	 */
	public Collection<PlanoClassificacao> findAssuntoAutoComplete(String codigoNome, Long idOrganizacao) {

		Search search = new SearchSeven(this.persistentClass);

		search.addFilterEqual("organizacao.id", idOrganizacao);

		search.addFilterOr(Filter.ilike("nome", "%" + codigoNome + "%"), Filter.ilike("codigo", "%" + codigoNome + "%"));

		search.addFilterOr(Filter.isEmpty("planoClassificacaoFilhasMapeadas"), Filter.equal("permitirUsarComoClassificacao", Boolean.TRUE));

		return this.search(search);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por listar recursivo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param nome
	 * @param idOrganizacao
	 * @return Collection<PlanoClassificacao>
	 */
	public Collection<PlanoClassificacao> findTree(String nome, Long idOrganizacao) {

		StringBuilder strQuery = new StringBuilder();
		Query query;

		switch (UtilDataBase.getDataBaseName(em())) {

		// ////////////////////////////////////// POSTGRESQL //////////////////////////////////////////
			case POSTGRESQL:

				strQuery.append("WITH RECURSIVE rel_tree AS (SELECT id, nome, codigo, organizacao_id, planoClassificacaoParent_id, observacao, faseCorrente, faseIntermediaria, tipoDestinacao_id, permitirUsarComoClassificacao, dataBloqueio,  datacriacao, dataedicao, datainativo, autor_id, editor_id, version, inativador_id, 1 AS level, array[id] AS path_info ");
				strQuery.append("FROM planoClassificacao ");
				strQuery.append("WHERE organizacao_id = " + idOrganizacao + " AND LOWER(nome) LIKE LOWER('%" + nome + "%')");
				strQuery.append("UNION ALL ");
				strQuery.append("SELECT c.id, c.nome, c.codigo, c.organizacao_id, c.planoClassificacaoParent_id, c.observacao, c.faseCorrente, c.faseIntermediaria, c.tipoDestinacao_id, c.permitirUsarComoClassificacao, c.dataBloqueio,  c.datacriacao, c.dataedicao, c.datainativo, c.autor_id, c.editor_id, c.version, c.inativador_id, p.level + 1, p.path_info||c.id ");
				strQuery.append("FROM planoClassificacao c ");
				strQuery.append("JOIN rel_tree p ON p.planoclassificacaoparent_id = c.id ) ");
				strQuery.append("SELECT DISTINCT ON (id) id, nome, codigo, organizacao_id, planoClassificacaoParent_id, observacao, faseCorrente, faseIntermediaria, tipoDestinacao_id, permitirUsarComoClassificacao, dataBloqueio,  datacriacao, dataedicao, datainativo, autor_id, editor_id, version, inativador_id ");
				strQuery.append("FROM rel_tree WHERE organizacao_id = " + idOrganizacao + " ORDER BY id, level ASC");

				query = em().createNativeQuery(strQuery.toString(), PlanoClassificacao.class);

				return query.getResultList();

				// ////////////////////////////////////SQL SERVER //////////////////////////////////////////
			case SQL_SERVER:

				/**
				 * Implementar sql nativa para os outros bancos
				 */
				// ////////////////////////////////////ORACLE//////////////////////////////////////////
			case ORACLE:
				throw new IllegalArgumentException("SQL inexistente para o SGBD Oracle");

				// ////////////////////////////////////MySQL//////////////////////////////////////////
			case MYSQL:
				throw new IllegalArgumentException("SQL inexistente para o SGBD MySQL");

				// ////////////////////////////////////DB2//////////////////////////////////////////
			case DB2:
				throw new IllegalArgumentException("SQL inexistente para o SGBD MySQL");

			default:
				throw new IllegalArgumentException("SQL inexistente para o SGBD");

		}

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por listar atrés através da data de edicao, atual
	 * 
	 * @author rogerio.costa
	 * 
	 * @return Collection<PlanoClassificacao>
	 */
	@Override
	public Collection<PlanoClassificacao> findPorDataEdicaoAtual() {
		
		Calendar c = Calendar.getInstance();
		c.add(Calendar.DATE, -1);
		
		Session session = (Session) this.em().getDelegate();

		Criteria criteria = session.createCriteria(persistentClass);
		
		criteria.add(Restrictions.between("dataEdicao", c, UtilDate.getCalendarDaDataAtual()));
		
		criteria.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY);

		return criteria.list();
	}
}
