package br.com.centralit.api.dao.impl;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.List;

import javax.persistence.FetchType;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.hibernate.criterion.Subqueries;
import org.hibernate.sql.JoinType;
import org.hibernate.transform.ResultTransformer;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.ProcessoDao;
import br.com.centralit.api.model.CredencialProcesso;
import br.com.centralit.api.model.Processo;
import br.com.centralit.api.model.SigiloPrivilegio;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.model.arquitetura.PersistentObject;
import br.com.centralit.framework.util.UtilObjeto;

import com.googlecode.genericdao.search.Filter;
import com.googlecode.genericdao.search.ISearch;
import com.googlecode.genericdao.search.Search;
import com.googlecode.genericdao.search.SearchResult;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 28/01/2016 - 17:29:59
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
@Repository("processoDao")
@SuppressWarnings("unchecked")
public class ProcessoDaoHibernate extends CitGenericDAOImpl implements ProcessoDao {

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 */
	public ProcessoDaoHibernate() {

		super(Processo.class);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter através do id do PlanoClassificacao
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idPlanoClassificacao
	 * 
	 * @return Collection<Processo>
	 */
	public Collection<Processo> findPorPlanoClassificacao(Long idPlanoClassificacao) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterEqual("assunto.id", idPlanoClassificacao);

		return this.search(searchSeven);

	}

	/**
	 * 
	 */
	@Override
	public Boolean isProcessoAbertoEmOutrasUnidades(Processo processo) {

		Criteria criteria = this.newCriteria();

		criteria.setProjection(Projections.rowCount());

		criteria.add(Restrictions.eq("id", processo.getId()));
		criteria.add(Restrictions.ne("unidade.id", processo.getUnidade().getId()));

		return (Long) criteria.uniqueResult() > 0;

	}

	/**
	 * 
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author andre.silva
	 * 
	 * @param idProcesso
	 * @param data
	 * @return
	 */
	@Override
	public Long obterQuantidadePorProcessoEDataCriacao(Calendar dataInicial, Calendar dataFinal) {

		Session session = (Session) this.em().getDelegate();

		Criteria criteria = session.createCriteria(persistentClass);

		criteria.setProjection(Projections.rowCount());

		criteria.add(Restrictions.between("dataCriacao", dataInicial, dataFinal));

		return (Long) criteria.uniqueResult();
	}

	/**
	 *
	 */
	@Override
	public Processo getProcessoByIdProcessInstance(Long idProcessInstance) {

		SearchSeven search = new SearchSeven(persistentClass);

		search.addFilterEqual("idProcessInstance", idProcessInstance);

		search(search, Processo.class);

		return searchUnique(search);
	}

	/**
	 *
	 */
	@Override
	public Processo getProcessoByProtocolo(String nup) {

		SearchSeven search = new SearchSeven(persistentClass);

		search.addFilterEqual("nup", nup);

		search(search, Processo.class);

		return searchUnique(search);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por listar através da permissão que o usuario tem
	 * 
	 * @author rogerio.costa
	 * 
	 * @param search
	 * 
	 * @return <RT> SearchResult<RT>
	 */
	public <RT> SearchResult<RT> searchAndCountPorSigilo(ISearch search) {

		Criteria criteria = this.newCriteria();

		DetachedCriteria detachedPrivilegio = getDetachedPorPermissaoEmSigilo();

		// Joins
		criteria.createAlias("documentos", "documentos", JoinType.LEFT_OUTER_JOIN);

		criteria.createAlias("documentos.status", "statusDocumento", JoinType.LEFT_OUTER_JOIN);
		// Filtros
		criteria.add(Restrictions.or(Restrictions.eq("statusDocumento.codigo", 0L), Restrictions.isNull("validado"), Restrictions.eq("validado", false)));

		criteria.add(Restrictions.or(Subqueries.propertyIn("sigilo.id", detachedPrivilegio), Subqueries.propertyIn("documentos.sigilo.id", detachedPrivilegio)));

		criteria.setProjection(Projections.distinct(Projections.property("id")));

		search.getFilters().add(Filter.in("id", criteria.list()));

		return super.searchAndCount(search);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por filtrar o sigilo que o usuario logado tem permissão através do(s) grupo(s) que o mesmo pertence
	 * 
	 * @author rogerio.costa
	 * 
	 * @return DetachedCriteria/ ids dos sigilos
	 */
	private DetachedCriteria getDetachedPorPermissaoEmSigilo() {

		Long idUsuario = ( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId();

		DetachedCriteria detachedCriteriaPrivilegio = DetachedCriteria.forClass(SigiloPrivilegio.class);

		detachedCriteriaPrivilegio.setProjection(Projections.property("sigilo.id"));

		detachedCriteriaPrivilegio.createAlias("privilegio", "privilegio");

		detachedCriteriaPrivilegio.createAlias("privilegio.grupoPrivilegios", "grupoPrivilegio");

		detachedCriteriaPrivilegio.add(Restrictions.isNull("grupoPrivilegio.dataInativo"));

		detachedCriteriaPrivilegio.createCriteria("grupoPrivilegio.grupo").createAlias("grupoUsuarios", "grupoUsuario");

		detachedCriteriaPrivilegio.add(Restrictions.isNull("grupoUsuario.dataInativo"));

		detachedCriteriaPrivilegio.add(Restrictions.eq("grupoUsuario.usuario.id", idUsuario));

		return detachedCriteriaPrivilegio;
	}

	@Override
	public boolean isSigiloso(Long id) {

		Criteria criteria = this.newCriteria();

		criteria.setProjection(Projections.id());

		criteria.add(Restrictions.eq("id", id));

		criteria.add(Restrictions.isNotNull("sigilo"));

		criteria.setMaxResults(1);

		return UtilObjeto.isReferencia(criteria.uniqueResult());
	}

	public Collection<Processo> autoCompleteProcessoByNupAndStatus(String nup, Dominio dominio) {

		Search search = new SearchSeven(this.persistentClass);

		search.addFilterOr(Filter.ilike("nup", "%" + nup + "%"), Filter.ilike("nup", "%" + nup + "%"));

		search.addFilterEqual("status.id", dominio.getId());

		search.addSortDesc("id");

		search.setMaxResults(10);

		return this.search(search);

	}
	
	
	
	@Override
	public Collection<Processo> autoCompleteProcessoConcluidoByNup(String nup) {

		Criteria criteriaProcesso = this.newCriteria(Processo.class);

		criteriaProcesso.add(Restrictions.ilike("nup", "%" + nup + "%"));

		criteriaProcesso.createAlias("status", "status", JoinType.LEFT_OUTER_JOIN);

		criteriaProcesso.add(Restrictions.eq("status.codigo", 1L));

		this.filterProcessoPermissaoUsuario(criteriaProcesso);

		criteriaProcesso.addOrder(Order.desc("id"));

		criteriaProcesso.setMaxResults(10);

		return criteriaProcesso.list();
	}

	@Override
	public Collection<Processo> autoCompleteProcessoEmAndamentoByNup(String nup) {

		Criteria criteriaProcesso = this.newCriteria(Processo.class);

		criteriaProcesso.add(Restrictions.ilike("nup", "%" + nup + "%"));

		criteriaProcesso.createAlias("status", "status", JoinType.LEFT_OUTER_JOIN);

		criteriaProcesso.add(Restrictions.not(Restrictions.eq("status.codigo", 1L)));

		criteriaProcesso.add(Restrictions.not(Restrictions.eq("status.codigo", 5L)));

		criteriaProcesso.add(Restrictions.not(Restrictions.eq("status.codigo", 3L)));

		this.filterProcessoPermissaoUsuario(criteriaProcesso);

		criteriaProcesso.addOrder(Order.desc("id"));

		criteriaProcesso.setMaxResults(10);

		return criteriaProcesso.list();
	}

	@Override
	public List<PersistentObject> findAutoComplete(String chave, String valor) {

		Criteria criteriaProcesso = this.newCriteria(Processo.class);

		this.filterProcessoPermissaoUsuario(criteriaProcesso);

		criteriaProcesso.add(Restrictions.ilike(chave, "%" + valor + "%"));

		return criteriaProcesso.list();
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por realizar os filtros em que o usuario logado tem permissão.
	 * 
	 * @author rogerio.costa
	 * 
	 * @param criteriaProcesso
	 */
	private void filterProcessoPermissaoUsuario(Criteria criteriaProcesso) {

		Usuario usuario = ( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() );

		// Verifica se o usuario está lotado na unidade
		if (UtilObjeto.isReferencia(usuario.getUnidade())) {

			// Filtro de permissões herdadas através de grupos
			DetachedCriteria detachedCriteriaPrivilegio = findSigiloUsuarioContemPermissaoAtravesDosGrupos(usuario);

			// Filtro da credencial do processo
			DetachedCriteria detachedCredencialProcesso = findProcessoSigilosoPorCredencial(usuario);

			criteriaProcesso.add(Restrictions.eq("unidade.id", usuario.getUnidade().getId()));

			criteriaProcesso.add(Restrictions.or(Subqueries.propertyIn("sigilo.id", detachedCriteriaPrivilegio), Subqueries.propertyIn("id", detachedCredencialProcesso), Restrictions.or(Restrictions.eq("autor.id", usuario.getId()), Restrictions.isNull("sigilo"))));

			criteriaProcesso.add(Restrictions.isNotNull("idProcessInstance"));

		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por retornar um detachedCriteria com os ids dos sigilos que o usuário tem permissão. O mesmo é consultado através dos grupos que o usuário pertence.
	 * 
	 * @author rogerio.costa
	 * 
	 * @param usuario
	 * @return DetachedCriteria/Sigilo.id
	 */
	private DetachedCriteria findSigiloUsuarioContemPermissaoAtravesDosGrupos(Usuario usuario) {

		DetachedCriteria detachedCriteriaPrivilegio = DetachedCriteria.forClass(SigiloPrivilegio.class);

		detachedCriteriaPrivilegio.setProjection(Projections.property("sigilo.id"));

		detachedCriteriaPrivilegio.createAlias("privilegio", "privilegio");

		detachedCriteriaPrivilegio.createAlias("privilegio.grupoPrivilegios", "grupoPrivilegio");

		detachedCriteriaPrivilegio.add(Restrictions.isNull("grupoPrivilegio.dataInativo"));

		detachedCriteriaPrivilegio.createCriteria("grupoPrivilegio.grupo").createAlias("grupoUsuarios", "grupoUsuario");

		detachedCriteriaPrivilegio.add(Restrictions.isNull("grupoUsuario.dataInativo"));

		detachedCriteriaPrivilegio.add(Restrictions.eq("grupoUsuario.usuario.id", usuario.getId()));

		return detachedCriteriaPrivilegio;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por retornar os ids dos processo sigilosos que o usuário tem permissão, adicionado diretamente para o usuário e grupo que o mesmo está vinculado.
	 * 
	 * @author rogerio.costa
	 * 
	 * @param usuario
	 * @return DetachedCriteria/ Processo.id
	 */
	private DetachedCriteria findProcessoSigilosoPorCredencial(Usuario usuario) {

		DetachedCriteria detachedCredencialProcesso = DetachedCriteria.forClass(CredencialProcesso.class);

		detachedCredencialProcesso.setProjection(Projections.property("processo.id"));

		detachedCredencialProcesso.createAlias("grupo", "grupo", JoinType.LEFT_OUTER_JOIN);

		detachedCredencialProcesso.createAlias("grupo.grupoUsuarios", "grupoUsuario", JoinType.LEFT_OUTER_JOIN);

		detachedCredencialProcesso.add(Restrictions.isNull("grupoUsuario.dataInativo"));

		detachedCredencialProcesso.add(Restrictions.or(Restrictions.eq("grupoUsuario.usuario.id", usuario.getId()), Restrictions.eq("usuario.id", usuario.getId())));

		detachedCredencialProcesso.add(Restrictions.isNull("dataInativo"));

		return detachedCredencialProcesso;
	}
}
