package br.com.centralit.api.dao.impl;

import java.util.Collection;
import java.util.List;

import javax.persistence.Query;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.TipoDocumentoDao;
import br.com.centralit.api.model.TipoDocumento;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;
import br.com.centralit.framework.model.arquitetura.PersistentObject;

import com.googlecode.genericdao.search.Filter;

@Repository("tipoDocumentoDao")
public class TipoDocumentoDaoHibernate extends CitGenericDAOImpl implements TipoDocumentoDao {

	public TipoDocumentoDaoHibernate() {

		super(TipoDocumento.class);
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	@Override
	public Collection<TipoDocumento> getTipoDocumentoByNome(String nome) {

		Query query = em().createQuery("select tipoDocumento from TipoDocumento tipoDocumento where tipoDocumento.nome = :nome or UPPER(tipoDocumento.nome) = UPPER(:nome)");
		query.setParameter("nome", nome);

		List documentos = query.getResultList();

		return documentos;
	}

	@Override
	public List<PersistentObject> findAutoComplete(String chave, String valor) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterILike("nome", "%" + valor + "%");

		searchSeven.addFilterOr(Filter.equal("formaCriacao.codigo", Long.parseLong(chave)), Filter.equal("formaCriacao.codigo", 3L));

		searchSeven.setMaxResults(10);

		return this.search(searchSeven);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter através do nome
	 * 
	 * @author rogerio.costa
	 * 
	 * @param nome
	 * 
	 * @return DocumentoGed
	 */
	public TipoDocumento obterPorNome(String nome) {

		Session session = (Session) this.em().getDelegate();

		Criteria criteria = session.createCriteria(persistentClass);

		criteria.add(Restrictions.eq("nome", nome).ignoreCase());

		criteria.setMaxResults(1);

		return (TipoDocumento) criteria.uniqueResult();
	}

}
