package br.com.centralit.api.dao.impl;

import java.util.Collection;

import org.hibernate.Criteria;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.hibernate.criterion.Subqueries;
import org.hibernate.sql.JoinType;
import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.UnidadeProcessoDao;
import br.com.centralit.api.model.CredencialProcesso;
import br.com.centralit.api.model.Processo;
import br.com.centralit.api.model.SigiloPrivilegio;
import br.com.centralit.api.model.UnidadeProcesso;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilObjeto;

import com.googlecode.genericdao.search.Filter;
import com.googlecode.genericdao.search.ISearch;
import com.googlecode.genericdao.search.SearchResult;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 05/01/2016 - 14:49:57
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
@Repository("unidadeProcessoDao")
public class UnidadeProcessoDaoHibernate extends CitGenericDAOImpl implements UnidadeProcessoDao {

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 */
	public UnidadeProcessoDaoHibernate() {

		super(UnidadeProcesso.class);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idProcesso
	 * @return
	 */
	public Collection<UnidadeProcesso> findByIdProcesso(Long idProcesso) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterEqual("processo.id", idProcesso);

		return this.search(searchSeven);

	}

	/**
	 * 
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author andre.silva
	 * 
	 * @param processo
	 * @return
	 */
	public Collection<UnidadeProcesso> findProcesso(Processo processo) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterEqual("processo.id", processo.getId());
		searchSeven.addFilterNotEqual("id", processo.getUnidade().getId());

		return this.search(searchSeven);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar se o processo foi concluido em todas as unidades
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idProcesso
	 * 
	 * @return Boolean
	 */
	public Boolean isProcessoConcluidoEmTodasUnidades(Long idProcesso) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterEqual("processo.id", idProcesso);

		searchSeven.addFilterEmpty("concluido");

		return UtilColecao.isVazio(this.search(searchSeven));
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar se o processo foi enviado para outra unidade
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idProcesso
	 * 
	 * @return Boolean
	 */
	public Boolean isProcessoEnviado(Long idProcesso) {

		Criteria criteria = this.newCriteria();

		criteria.setProjection(Projections.id());
		criteria.add(Restrictions.eq("processo.id", idProcesso));
		criteria.add(Restrictions.isNull("dataInativo"));

		return criteria.list().size() > 1;

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter através da unidade e processo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idProcesso
	 * @param idUnidade
	 * @return UnidadeProcesso
	 */
	public UnidadeProcesso obterPorUnidadeEProcesso(Long idProcesso, Long idUnidade) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterEqual("processo.id", idProcesso);

		searchSeven.addFilterEqual("unidade.id", idUnidade);

		searchSeven.setMaxResults(1);

		return this.searchUnique(searchSeven);

	}

	/**
	 * Método responsável por listar processos através da unidade do usuário
	 * 
	 * @author rogerio.cassimiro
	 * @param search
	 * @param usuario
	 * @param idStatus
	 * @return <RT> SearchResult<RT>
	 */
	@Override
	public <RT> SearchResult<RT> searchAndCountPorUnidade(ISearch search, Usuario usuario, Long idStatus) {

		// Verifica se o usuario está lotado na unidade
		if (UtilObjeto.isReferencia(usuario.getUnidade())) {

			// Filtro de permissões herdadas através de grupos
			DetachedCriteria detachedCriteriaPrivilegio = findSigiloUsuarioContemPermissaoAtravesDosGrupos(usuario);

			// Filtro da credencial do processo
			DetachedCriteria detachedCredencialProcesso = findProcessoSigilosoPorCredencial(usuario);

			// Filtro da unidade, para processos públicos e restritos.
			search.getFilters().add(Filter.or(Filter.not(Filter.equal("processo.status.id", idStatus)), Filter.isEmpty("processo.status.id")));

			search.getFilters().add(Filter.notEqual("processo.status.codigo", 5L));

			search.getFilters().add(Filter.equal("unidade.id", usuario.getUnidade().getId()));

			search.getFilters().add(Filter.or(Filter.equal("concluido", false), Filter.isNull("concluido")));

			Criteria criteriaProcessoSigiloso = this.newCriteria(Processo.class);

			criteriaProcessoSigiloso.setProjection(Projections.property("id"));

			criteriaProcessoSigiloso.add(Restrictions.or(Subqueries.propertyIn("sigilo.id", detachedCriteriaPrivilegio), Subqueries.propertyIn("id", detachedCredencialProcesso)));

			search.getFilters().add(Filter.isNotNull("processo.idProcessInstance"));

			search.getFilters().add(Filter.or(Filter.equal("processo.autor.id", usuario.getId()), Filter.isNull("processo.sigilo"), Filter.in("processo.id", criteriaProcessoSigiloso.list())));

			return super.searchAndCount(search);
		}

		return new SearchResult<RT>();
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por retornar os ids dos processo sigilosos que o usuário tem permissão, adicionado diretamente para o usuário e grupo que o mesmo está vinculado.
	 * 
	 * @author rogerio.costa
	 * 
	 * @param usuario
	 * @return DetachedCriteria/ Processo.id
	 */
	private DetachedCriteria findProcessoSigilosoPorCredencial(Usuario usuario) {

		DetachedCriteria detachedCredencialProcesso = DetachedCriteria.forClass(CredencialProcesso.class);

		detachedCredencialProcesso.setProjection(Projections.property("processo.id"));

		detachedCredencialProcesso.createAlias("grupo", "grupo", JoinType.LEFT_OUTER_JOIN);

		detachedCredencialProcesso.createAlias("grupo.grupoUsuarios", "grupoUsuario", JoinType.LEFT_OUTER_JOIN);

		detachedCredencialProcesso.add(Restrictions.isNull("grupoUsuario.dataInativo"));

		detachedCredencialProcesso.add(Restrictions.or(Restrictions.eq("grupoUsuario.usuario.id", usuario.getId()), Restrictions.eq("usuario.id", usuario.getId())));

		detachedCredencialProcesso.add(Restrictions.isNull("dataInativo"));

		return detachedCredencialProcesso;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por retornar um detachedCriteria com os ids dos sigilos que o usuário tem permissão. O mesmo é consultado através dos grupos que o usuário pertence.
	 * 
	 * @author rogerio.costa
	 * 
	 * @param usuario
	 * @return DetachedCriteria/Sigilo.id
	 */
	private DetachedCriteria findSigiloUsuarioContemPermissaoAtravesDosGrupos(Usuario usuario) {

		DetachedCriteria detachedCriteriaPrivilegio = DetachedCriteria.forClass(SigiloPrivilegio.class);

		detachedCriteriaPrivilegio.setProjection(Projections.property("sigilo.id"));

		detachedCriteriaPrivilegio.createAlias("privilegio", "privilegio");

		detachedCriteriaPrivilegio.createAlias("privilegio.grupoPrivilegios", "grupoPrivilegio");

		detachedCriteriaPrivilegio.add(Restrictions.isNull("grupoPrivilegio.dataInativo"));

		detachedCriteriaPrivilegio.createCriteria("grupoPrivilegio.grupo").createAlias("grupoUsuarios", "grupoUsuario");

		detachedCriteriaPrivilegio.add(Restrictions.isNull("grupoUsuario.dataInativo"));

		detachedCriteriaPrivilegio.add(Restrictions.eq("grupoUsuario.usuario.id", usuario.getId()));

		return detachedCriteriaPrivilegio;
	}

	/**
	 *
	 */
	public Collection<UnidadeProcesso> findByPorIdProcessoEmExecucao(Long idProcesso) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterEqual("processo.id", idProcesso);

		searchSeven.addFilterOr(Filter.equal("concluido", false), Filter.isNull("concluido"));

		return this.search(searchSeven);

	}

	public UnidadeProcesso obterPorUnidadeEProcessoEmExecucao(Long idProcesso, Long idUnidade) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterEqual("processo.id", idProcesso);

		searchSeven.addFilterEqual("unidade.id", idUnidade);

		searchSeven.addFilterOr(Filter.isNull("concluido"));

		searchSeven.setMaxResults(1);

		return this.searchUnique(searchSeven);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar se o processo pertence a unidade
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idProcesso
	 * @param idUnidade
	 * @return boolean
	 */
	public boolean isProcessoNaUnidadeUsuarioLogado(Long idProcesso, Long idUnidade) {

		Criteria criteria = this.newCriteria();

		criteria.setProjection(Projections.id());
		criteria.add(Restrictions.eq("processo.id", idProcesso));
		criteria.add(Restrictions.eq("unidade.id", idUnidade));
		criteria.add(Restrictions.isNull("dataInativo"));
		criteria.setMaxResults(1);

		return UtilObjeto.isReferencia(criteria.uniqueResult());

	}
}
