package br.com.centralit.api.model;

import java.util.Calendar;
import java.util.Collection;
import java.util.Locale;

import javax.persistence.Basic;
import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.annotations.Type;

import br.com.centralit.api.framework.json.ViewsEcm;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.Dominio;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>Documento
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 20/10/2015 - 15:22:15
 * 
 * @version 1.0.0
 * 
 * 
 * @author rogerio.costa
 * 
 */
@Entity
@JsonIgnoreProperties({ "$checked", "nivelAcessoList" })
public class DocumentoGed extends PersistentObjectUnidade implements Cloneable {

	/** Atributo serialVersionUID. */
	private static final long serialVersionUID = 8364258343126147394L;

	/** Atributo id. */
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@JsonView({ Views.GenericView.class })
	private Long id;

	/** Atributo assuntoComplementar. */
	@JsonView({ ViewsEcm.DocumentoGedEdit.class, Views.ProcessoEdit.class, ViewsEcm.ClassificacaoDocumentoGedView.class, Views.ProcessoVHView.class})
	private String assuntoComplementar;

	/** Atributo numero. */
	@JsonView({ ViewsEcm.DocumentoGedEdit.class, Views.ProcessoEdit.class, ViewsEcm.ClassificacaoDocumentoGedView.class, Views.ProcessoVHView.class})
	private String numero;

	/** Atributo idRaiz. */
	@JsonView({ Views.DocumentoGedEdit.class, Views.ProcessoEdit.class })
	private Long idRaiz;

	/** Atributo novaVersao. */
	@Transient
	private Boolean novaVersao;

	/** Atributo versaoDocumento. */
	@JsonView({ Views.DocumentoGedEdit.class, Views.ProcessoEdit.class, Views.ProcessoVHView.class})
	private Double versaoDocumento;

	/** Atributo cancelado. */
	@JsonView({ Views.ProcessoEdit.class, Views.ProcessoVHView.class})
	private Boolean cancelado;

	@JsonView({ Views.DocumentoGedEdit.class })
	private String localizacao;

	/** Atributo conteudo. */
	@JsonView({ ViewsEcm.DocumentoGedEdit.class, Views.ProcessoVHView.class})
	@Transient
	private String conteudo;

	/** Atributo tipoAssunto. */
	@JsonView({ Views.DocumentoGedEdit.class, Views.ProcessoVHView.class })
	private String tipoAssunto;

	/** Atributo conteudoCriptografado. */
	@Lob
	@Basic(fetch = FetchType.LAZY)
	@Type(type = "org.hibernate.type.BinaryType")
	private byte[] conteudoCriptografado;

	/** Atributo conteudoAssinado. */
	@Lob
	@Basic(fetch = FetchType.LAZY)
	@Type(type = "org.hibernate.type.BinaryType")
	private byte[] conteudoAssinado;

	/** Atributo icon. */
	@JsonView({ ViewsEcm.DocumentoGedEdit.class, Views.ProcessoEdit.class, ViewsEcm.ClassificacaoDocumentoGedView.class, Views.ProcessoVHView.class})
	private String icon;

	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsEcm.DocumentoGedEdit.class })
	private Calendar dataReferencia;

	/** Atributo prazoSigilo. */
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsEcm.DocumentoGedList.class })
	private Calendar prazoSigilo;

	/** Atributo aprovado. */
	@JsonView({ ViewsEcm.DocumentoGedEdit.class })
	private Boolean aprovado;

	/** Atributo formaCriacao. Online/Anexo */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ ViewsEcm.DocumentoGedEdit.class, Views.ProcessoEdit.class, Views.ProcessoVHView.class})
	private Dominio formaCriacao;

	/** Atributo status. */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ ViewsEcm.DocumentoGedEdit.class, Views.ProcessoEdit.class })
	private Dominio status;

	/** Atributo estado. Minuta/Copia/Original */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ ViewsEcm.DocumentoGedEdit.class, Views.ProcessoEdit.class })
	private Dominio estado;

	/** Atributo tramitarDocumento. */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ ViewsEcm.DocumentoGedEdit.class, Views.ProcessoEdit.class })
	private TramitarDocumento tramitarDocumento;

	/** Atributo processo. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	@JsonView({ ViewsEcm.DocumentoGedEdit.class })
	private Processo processo;

	/** Atributo processo. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	@JsonView({ ViewsEcm.DocumentoGedEdit.class })
	private Processo origem;

	/** Atributo processo. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	private Processo processoRemocao;

	/** Atributo tipoDocumento. */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ ViewsEcm.DocumentoGedEdit.class, Views.ProcessoEdit.class, Views.ProcessoVHView.class})
	private TipoDocumento tipoDocumento;

	/** Atributo nivelAcesso. */
	@ManyToOne(fetch = FetchType.EAGER)
	@JsonView({ ViewsEcm.DocumentoGedEdit.class, Views.ProcessoEdit.class, ViewsEcm.ClassificacaoDocumentoGedView.class, Views.ProcessoVHView.class })
	private NivelAcessoTipoDocumento nivelAcesso;

	@ManyToOne(fetch = FetchType.EAGER)
	@JsonView({ ViewsEcm.DocumentoGedEdit.class, Views.ProcessoEdit.class, ViewsEcm.ClassificacaoDocumentoGedView.class })
	private NivelAcessoTipoDocumento nivelAcessoOriginal;

	/** Atributo sigilo. */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ ViewsEcm.DocumentoGedEdit.class, Views.ProcessoEdit.class, ViewsEcm.ClassificacaoDocumentoGedView.class, Views.ProcessoVHView.class})
	private Sigilo sigilo;

	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ ViewsEcm.DocumentoGedEdit.class, Views.ProcessoEdit.class, ViewsEcm.ClassificacaoDocumentoGedView.class })
	private Sigilo sigiloOriginal;

	/** Atributo tipoSuporteDocumento. */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ ViewsEcm.DocumentoGedEdit.class, ViewsEcm.ClassificacaoDocumentoGedView.class })
	private TipoSuporteDocumento tipoSuporteDocumento;

	/** Atributo planoClassificacao. */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ ViewsEcm.DocumentoGedEdit.class, Views.ProcessoEdit.class })
	private PlanoClassificacao planoClassificacao;

	/** Atributo destinatario. */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ ViewsEcm.DocumentoGedEdit.class, Views.ProcessoEdit.class })
	private Pessoa destinatario;

	/** Atributo hipoteseLegal. */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ Views.ProcessoEdit.class, ViewsEcm.DocumentoGedEdit.class })
	private HipoteseLegal hipoteseLegal;

	/** Atributo anexo. */
	@OneToOne(mappedBy = "documentoGed", fetch = FetchType.EAGER)
	@JsonView({ ViewsEcm.DocumentoGedEdit.class, Views.ProcessoEdit.class, ViewsEcm.ClassificacaoDocumentoGedView.class, Views.ProcessoVHView.class})
	private AnexoGed anexo;

	/** Atributo versaoAnterior. */
	@ManyToOne(fetch = FetchType.LAZY)
	private DocumentoGed versaoAnterior;

	@ManyToOne(fetch = FetchType.LAZY)
	private DocumentoGed versaoPosterior;

	/** Atributo cancelamentoDocumentoGed. */
	@OneToOne(mappedBy = "documentoGed", fetch = FetchType.EAGER)
	@JsonView({ Views.ProcessoEdit.class })
	private CancelamentoDocumentoGed cancelamentoDocumentoGed;

	/** Atributo Interessados. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "documentoGed", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsEcm.DocumentoGedEdit.class, Views.ProcessoEdit.class })
	private Collection<InteressadoDocumento> interessados;

	@OneToMany(fetch = FetchType.LAZY, mappedBy = "documentoGed")
	@JsonView({ Views.DocumentoGedEdit.class,  Views.ProcessoEdit.class })
	private Collection<Assinatura> assinaturas;

	@OneToOne(cascade = CascadeType.ALL, fetch = FetchType.EAGER)
	@JsonView({ Views.DocumentoGedEdit.class, ViewsEcm.AssinaturaEdit.class })
	private ConteudoTarja conteudoTarja;

	@Transient
	@JsonView({ Views.ProcessoEdit.class, Views.DocumentoGedEdit.class })
	private String tresAssinaturas;

	/**
	 * @return the novaVersao
	 */
	public Boolean getNovaVersao() {

		return novaVersao;
	}

	/**
	 * @param novaVersao
	 *            the novaVersao to set
	 */
	public void setNovaVersao(Boolean novaVersao) {

		this.novaVersao = novaVersao;
	}

	/**
	 * Retorna o valor do atributo <code>id</code>
	 * 
	 * @return <code>Long</code>
	 */
	public Long getId() {

		return id;
	}

	/**
	 * @return the versaoAnterior
	 */
	public DocumentoGed getVersaoAnterior() {

		return versaoAnterior;
	}

	/**
	 * @param versaoAnterior
	 *            the versaoAnterior to set
	 */
	public void setVersaoAnterior(DocumentoGed versaoAnterior) {

		this.versaoAnterior = versaoAnterior;
	}

	/**
	 * @return the versaoPosterior
	 */
	public DocumentoGed getVersaoPosterior() {

		return versaoPosterior;
	}

	/**
	 * @param versaoPosterior
	 *            the versaoPosterior to set
	 */
	public void setVersaoPosterior(DocumentoGed versaoPosterior) {

		this.versaoPosterior = versaoPosterior;
	}

	/**
	 * Define o valor do atributo <code>id</code>.
	 * 
	 * @param id
	 */
	public void setId(Long id) {

		this.id = id;
	}

	/**
	 * Retorna o valor do atributo <code>assuntoComplementar</code>
	 * 
	 * @return <code>String</code>
	 */
	public String getAssuntoComplementar() {

		return assuntoComplementar;
	}

	/**
	 * Define o valor do atributo <code>assuntoComplementar</code>.
	 * 
	 * @param assuntoComplementar
	 */
	public void setAssuntoComplementar(String assuntoComplementar) {

		this.assuntoComplementar = assuntoComplementar;
	}

	/**
	 * Retorna o valor do atributo <code>numero</code>
	 * 
	 * @return <code>String</code>
	 */
	public String getNumero() {

		return numero;
	}

	/**
	 * Define o valor do atributo <code>numero</code>.
	 * 
	 * @param numero
	 */
	public void setNumero(String numero) {

		this.numero = numero;
	}

	/**
	 * Retorna o valor do atributo <code>status</code>
	 * 
	 * @return <code>Dominio</code>
	 */
	public Dominio getStatus() {

		return status;
	}

	/**
	 * Define o valor do atributo <code>status</code>.
	 * 
	 * @param status
	 */
	public void setStatus(Dominio status) {

		this.status = status;
	}

	/**
	 * Retorna o valor do atributo <code>tramitarDocumento</code>
	 * 
	 * @return <code>TramitarDocumento</code>
	 */
	public TramitarDocumento getTramitarDocumento() {

		return tramitarDocumento;
	}

	/**
	 * Define o valor do atributo <code>tramitarDocumento</code>.
	 * 
	 * @param tramitarDocumento
	 */
	public void setTramitarDocumento(TramitarDocumento tramitarDocumento) {

		this.tramitarDocumento = tramitarDocumento;
	}

	/**
	 * Retorna o valor do atributo <code>processo</code>
	 * 
	 * @return <code>Processo</code>
	 */
	public Processo getProcesso() {

		return processo;
	}

	/**
	 * Define o valor do atributo <code>processo</code>.
	 * 
	 * @param processo
	 */
	public void setProcesso(Processo processo) {

		this.processo = processo;
	}

	/**
	 * Retorna o valor do atributo <code>tipoDocumento</code>
	 * 
	 * @return <code>TipoDocumento</code>
	 */
	public TipoDocumento getTipoDocumento() {

		return tipoDocumento;
	}

	/**
	 * Define o valor do atributo <code>tipoDocumento</code>.
	 * 
	 * @param tipoDocumento
	 */
	public void setTipoDocumento(TipoDocumento tipoDocumento) {

		this.tipoDocumento = tipoDocumento;
	}

	/**
	 * Retorna o valor do atributo <code>nivelAcesso</code>
	 * 
	 * @return <code>NivelAcessoTipoDocumento</code>
	 */
	public NivelAcessoTipoDocumento getNivelAcesso() {

		return nivelAcesso;
	}

	/**
	 * Define o valor do atributo <code>nivelAcesso</code>.
	 * 
	 * @param nivelAcesso
	 */
	public void setNivelAcesso(NivelAcessoTipoDocumento nivelAcesso) {

		this.nivelAcesso = nivelAcesso;
	}

	/**
	 * Retorna o valor do atributo <code>anexo</code>
	 * 
	 * @return <code>AnexoGed</code>
	 */
	public AnexoGed getAnexo() {

		return anexo;
	}

	/**
	 * Define o valor do atributo <code>anexo</code>.
	 * 
	 * @param anexo
	 */
	public void setAnexo(AnexoGed anexo) {

		this.anexo = anexo;
	}

	/**
	 * Retorna o valor do atributo <code>planoClassificacao</code>
	 * 
	 * @return <code>PlanoClassificacao</code>
	 */
	public PlanoClassificacao getPlanoClassificacao() {

		return planoClassificacao;
	}

	/**
	 * Define o valor do atributo <code>planoClassificacao</code>.
	 * 
	 * @param planoClassificacao
	 */
	public void setPlanoClassificacao(PlanoClassificacao planoClassificacao) {

		this.planoClassificacao = planoClassificacao;
	}

	/**
	 * Retorna o valor do atributo <code>destinatario</code>
	 * 
	 * @return <code>Pessoa</code>
	 */
	public Pessoa getDestinatario() {

		return destinatario;
	}

	/**
	 * Define o valor do atributo <code>destinatario</code>.
	 * 
	 * @param destinatario
	 */
	public void setDestinatario(Pessoa destinatario) {

		this.destinatario = destinatario;
	}

	public Collection<InteressadoDocumento> getInteressados() {

		return interessados;
	}

	public void setInteressados(Collection<InteressadoDocumento> interessados) {

		this.interessados = interessados;
	}

	/**
	 * Retorna o valor do atributo <code>hipoteseLegal</code>
	 * 
	 * @return <code>HipoteseLegal</code>
	 */
	public HipoteseLegal getHipoteseLegal() {

		return hipoteseLegal;
	}

	/**
	 * Define o valor do atributo <code>hipoteseLegal</code>.
	 * 
	 * @param hipoteseLegal
	 */
	public void setHipoteseLegal(HipoteseLegal hipoteseLegal) {

		this.hipoteseLegal = hipoteseLegal;
	}

	/**
	 * Retorna o valor do atributo <code>processoRemocao</code>
	 * 
	 * @return <code>Processo</code>
	 */
	public Processo getProcessoRemocao() {

		return processoRemocao;
	}

	/**
	 * Define o valor do atributo <code>processoRemocao</code>.
	 * 
	 * @param processoRemocao
	 */
	public void setProcessoRemocao(Processo processoRemocao) {

		this.processoRemocao = processoRemocao;
	}

	/**
	 * @return the versaoDocumento
	 */
	public Double getVersaoDocumento() {

		return versaoDocumento;
	}

	/**
	 * @param versaoDocumento
	 *            the versaoDocumento to set
	 */
	public void setVersaoDocumento(Double versaoDocumento) {

		this.versaoDocumento = versaoDocumento;
	}

	/**
	 * Retorna o valor do atributo <code>idRaiz</code>
	 * 
	 * @return <code>Long</code>
	 */
	public Long getIdRaiz() {

		return idRaiz;
	}

	/**
	 * Define o valor do atributo <code>idRaiz</code>.
	 * 
	 * @param idRaiz
	 */
	public void setIdRaiz(Long idRaiz) {

		this.idRaiz = idRaiz;
	}

	/**
	 * Retorna o valor do atributo <code>username</code>
	 * 
	 */
	@JsonView({ Views.DocumentoGedEdit.class })
	public String getAutorUserName() {

		return this.autor.getUsername();
	}

	/**
	 * Retorna o valor do atributo <code>username</code>
	 * 
	 */
	@JsonView({ Views.DocumentoGedEdit.class })
	public String getUnidadeNome() {

		return this.unidade == null ? null : this.unidade.getNome();
	}

	/**
	 * Retorna o valor do atributo <code></code>
	 * 
	 * @author andre.silva
	 * 
	 * @return String
	 */
	@JsonView({ ViewsEcm.DocumentoGedList.class })
	public String getDocumentoVersao() {

		Double formatado = null;
		String value = null;
		if (this.versaoDocumento != null) {
			formatado = Double.valueOf(String.format(Locale.US, "%.2f", this.versaoDocumento));
			value = String.valueOf(formatado).replace(',', '.');
		}

		return value;
	}

	/**
	 * Retorna o valor do atributo <code>cancelado</code>
	 * 
	 * @return <code>Boolean</code>
	 */
	public Boolean getCancelado() {

		return cancelado;
	}

	/**
	 * Define o valor do atributo <code>cancelado</code>.
	 * 
	 * @param cancelado
	 */
	public void setCancelado(Boolean cancelado) {

		this.cancelado = cancelado;
	}

	/**
	 * Retorna o valor do atributo <code>cancelamentoDocumentoGed</code>
	 * 
	 * @return <code>CancelamentoDocumentoGed</code>
	 */
	public CancelamentoDocumentoGed getCancelamentoDocumentoGed() {

		return cancelamentoDocumentoGed;
	}

	/**
	 * Define o valor do atributo <code>cancelamentoDocumentoGed</code>.
	 * 
	 * @param cancelamentoDocumentoGed
	 */
	public void setCancelamentoDocumentoGed(CancelamentoDocumentoGed cancelamentoDocumentoGed) {

		this.cancelamentoDocumentoGed = cancelamentoDocumentoGed;
	}

	/**
	 * Retorna o valor do atributo <code>sigilo</code>
	 * 
	 * @return <code>Sigilo</code>
	 */
	public Sigilo getSigilo() {

		return sigilo;
	}

	/**
	 * Define o valor do atributo <code>sigilo</code>.
	 * 
	 * @param sigilo
	 */
	public void setSigilo(Sigilo sigilo) {

		this.sigilo = sigilo;
	}

	/**
	 * Retorna o valor do atributo <code>localizacao</code>
	 * 
	 * @return <code>String</code>
	 */
	public String getLocalizacao() {

		return localizacao;
	}

	/**
	 * Define o valor do atributo <code>localizacao</code>.
	 * 
	 * @param localizacao
	 */
	public void setLocalizacao(String localizacao) {

		this.localizacao = localizacao;
	}

	/**
	 * Retorna o valor do atributo <code>conteudo</code>
	 * 
	 * @return <code>String</code>
	 */
	public String getConteudo() {

		return conteudo;
	}

	/**
	 * Define o valor do atributo <code>conteudo</code>.
	 * 
	 * @param conteudo
	 */
	public void setConteudo(String conteudo) {

		this.conteudo = conteudo;
	}

	/**
	 * Retorna o valor do atributo <code>icon</code>
	 * 
	 * @return <code>String</code>
	 */
	public String getIcon() {

		return icon;
	}

	/**
	 * Define o valor do atributo <code>icon</code>.
	 * 
	 * @param icon
	 */
	public void setIcon(String icon) {

		this.icon = icon;
	}

	/**
	 * Retorna o valor do atributo <code>conteudoCriptografado</code>
	 * 
	 * @return <code>byte[]</code>
	 */
	public byte[] getConteudoCriptografado() {

		return conteudoCriptografado;
	}

	/**
	 * Define o valor do atributo <code>conteudoCriptografado</code>.
	 * 
	 * @param conteudoCriptografado
	 */
	public void setConteudoCriptografado(byte[] conteudoCriptografado) {

		this.conteudoCriptografado = conteudoCriptografado;
	}

	public static long getSerialversionuid() {

		return serialVersionUID;
	}

	/**
	 * Retorna o valor do atributo <code>origem</code>
	 * 
	 * @return <code>Processo</code>
	 */
	@JsonView({ Views.ProcessoEdit.class })
	public Processo getOrigem() {

		return origem;
	}

	/**
	 * Define o valor do atributo <code>origem</code>.
	 * 
	 * @param origem
	 */
	public void setOrigem(Processo origem) {

		this.origem = origem;
	}

	/**
	 * Retorna o valor do atributo <code>formaCriacao</code>
	 * 
	 * @return <code>Dominio</code>
	 */
	public Dominio getFormaCriacao() {

		return formaCriacao;
	}

	/**
	 * Define o valor do atributo <code>formaCriacao</code>.
	 * 
	 * @param formaCriacao
	 */
	public void setFormaCriacao(Dominio formaCriacao) {

		this.formaCriacao = formaCriacao;
	}

	/**
	 * Retorna o valor do atributo <code>tipoAssunto</code>
	 * 
	 * @return <code>String</code>
	 */
	public String getTipoAssunto() {

		return tipoAssunto;
	}

	/**
	 * Define o valor do atributo <code>tipoAssunto</code>.
	 * 
	 * @param tipoAssunto
	 */
	public void setTipoAssunto(String tipoAssunto) {

		this.tipoAssunto = tipoAssunto;
	}

	/**
	 * Retorna o valor do atributo <code>dataReferencia</code>
	 * 
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataReferencia() {

		return dataReferencia;
	}

	/**
	 * Define o valor do atributo <code>dataReferencia</code>.
	 * 
	 * @param dataReferencia
	 */
	public void setDataReferencia(Calendar dataReferencia) {

		this.dataReferencia = dataReferencia;
	}

	@Override
	public DocumentoGed clone() {

		DocumentoGed documentoGed = null;
		try {
			documentoGed = (DocumentoGed) super.clone();
		} catch (CloneNotSupportedException e) {

			e.printStackTrace();
		}
		return documentoGed;

	}

	/**
	 * Retorna o valor do atributo <code>prazoSigilo</code>
	 * 
	 * @return <code>Calendar</code>
	 */
	public Calendar getPrazoSigilo() {

		return prazoSigilo;
	}

	/**
	 * Define o valor do atributo <code>prazoSigilo</code>.
	 * 
	 * @param prazoSigilo
	 */
	public void setPrazoSigilo(Calendar prazoSigilo) {

		this.prazoSigilo = prazoSigilo;
	}

	/**
	 * Retorna o valor do atributo <code>aprovado</code>
	 * 
	 * @return <code>Boolean</code>
	 */
	public Boolean getAprovado() {

		return aprovado;
	}

	/**
	 * Define o valor do atributo <code>aprovado</code>.
	 * 
	 * @param aprovado
	 */
	public void setAprovado(Boolean aprovado) {

		this.aprovado = aprovado;
	}

	/**
	 * Retorna o valor do atributo <code>nivelAcessoOriginal</code>
	 * 
	 * @return <code>NivelAcessoTipoDocumento</code>
	 */
	public NivelAcessoTipoDocumento getNivelAcessoOriginal() {

		return nivelAcessoOriginal;
	}

	/**
	 * Define o valor do atributo <code>nivelAcessoOriginal</code>.
	 * 
	 * @param nivelAcessoOriginal
	 */
	public void setNivelAcessoOriginal(NivelAcessoTipoDocumento nivelAcessoOriginal) {

		this.nivelAcessoOriginal = nivelAcessoOriginal;
	}

	/**
	 * Retorna o valor do atributo <code>sigiloOriginal</code>
	 * 
	 * @return <code>Sigilo</code>
	 */
	public Sigilo getSigiloOriginal() {

		return sigiloOriginal;
	}

	/**
	 * Define o valor do atributo <code>sigiloOriginal</code>.
	 * 
	 * @param sigiloOriginal
	 */
	public void setSigiloOriginal(Sigilo sigiloOriginal) {

		this.sigiloOriginal = sigiloOriginal;
	}

	/**
	 * Retorna o valor do atributo <code>tipoSuporteDocumento</code>
	 * 
	 * @return <code>TipoSuporteDocumento</code>
	 */
	public TipoSuporteDocumento getTipoSuporteDocumento() {

		return tipoSuporteDocumento;
	}

	/**
	 * Define o valor do atributo <code>tipoSuporteDocumento</code>.
	 * 
	 * @param tipoSuporteDocumento
	 */
	public void setTipoSuporteDocumento(TipoSuporteDocumento tipoSuporteDocumento) {

		this.tipoSuporteDocumento = tipoSuporteDocumento;
	}

	/**
	 * Retorna o valor do atributo <code>estado</code>
	 * 
	 * @return <code>Dominio</code>
	 */
	public Dominio getEstado() {

		return estado;
	}

	/**
	 * Define o valor do atributo <code>estado</code>.
	 * 
	 * @param estado
	 */
	public void setEstado(Dominio estado) {

		this.estado = estado;
	}

	/**
	 * Retorna o valor do atributo <code>assinaturas</code>
	 * 
	 * @return <code>Collection<Assinatura></code>
	 */
	public Collection<Assinatura> getAssinaturas() {

		return assinaturas;
	}

	/**
	 * Define o valor do atributo <code>assinaturas</code>.
	 * 
	 * @param assinaturas
	 */
	public void setAssinaturas(Collection<Assinatura> assinaturas) {

		this.assinaturas = assinaturas;
	}

	/**
	 * Retorna o valor do atributo <code>conteudoTarja</code>
	 * 
	 * @return <code>ConteudoTarja</code>
	 */
	public ConteudoTarja getConteudoTarja() {

		return conteudoTarja;
	}

	/**
	 * Define o valor do atributo <code>conteudoTarja</code>.
	 * 
	 * @param conteudoTarja
	 */
	public void setConteudoTarja(ConteudoTarja conteudoTarja) {

		this.conteudoTarja = conteudoTarja;
	}

	/**
	 * Retorna o valor do atributo <code>tresAssinaturas</code>
	 * 
	 * @return <code>String</code>
	 */
	public String getTresAssinaturas() {

		return tresAssinaturas;
	}

	/**
	 * Define o valor do atributo <code>tresAssinaturas</code>.
	 * 
	 * @param tresAssinaturas
	 */
	public void setTresAssinaturas(String tresAssinaturas) {

		this.tresAssinaturas = tresAssinaturas;
	}

	/**
	 * Retorna o valor do atributo <code>conteudoAssinado</code>
	 * 
	 * @return <code>byte[]</code>
	 */
	public byte[] getConteudoAssinado() {

		return conteudoAssinado;
	}

	/**
	 * Define o valor do atributo <code>conteudoAssinado</code>.
	 * 
	 * @param conteudoAssinado
	 */
	public void setConteudoAssinado(byte[] conteudoAssinado) {

		this.conteudoAssinado = conteudoAssinado;
	}

}
