package br.com.centralit.api.model;

import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.MappedSuperclass;
import javax.persistence.PrePersist;
import javax.persistence.PreUpdate;

import org.springframework.security.core.context.SecurityContextHolder;

import com.fasterxml.jackson.annotation.JsonView;

import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.Unidade;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAudit;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>PersistentObjectUnidade
 * </p>
 * 
 * <p>
 * <b>Description: </b>Classe abstract responsável por abstrair o relacionamento com uninidade/estruturaOrganizacional
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 10/12/2015 - 08:19:09
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
@MappedSuperclass
public abstract class PersistentObjectUnidade extends PersistentObjectAudit {

	/** Atributo serialVersionUID. */
	private static final long serialVersionUID = -7351444695048320756L;

	/** Atributo estruturaOrganizacional. */
	@ManyToOne(fetch = FetchType.EAGER)
	@JoinColumn(name = "unidade_id")
	@JsonView({ Views.GenericView.class })
	protected Unidade unidade;

	@PrePersist
	public void prePersist() {

		if (SecurityContextHolder.getContext().getAuthentication() != null) {
			if (SecurityContextHolder.getContext().getAuthentication().getPrincipal() instanceof Usuario) {
				this.setUnidade(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getUnidade());
			}
		}
	}

	@PreUpdate
	public void preUpdate() {

		if (SecurityContextHolder.getContext().getAuthentication() != null) {
			if (SecurityContextHolder.getContext().getAuthentication().getPrincipal() instanceof Usuario) {
				this.setUnidade(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getUnidade());
			}
		}
	}

	/**
	 * Retorna o valor do atributo <code>unidade</code>
	 * 
	 * @return <code>Unidade</code>
	 */
	public Unidade getUnidade() {

		return unidade;
	}

	/**
	 * Define o valor do atributo <code>unidade</code>.
	 * 
	 * @param unidade
	 */
	public void setUnidade(Unidade unidade) {

		this.unidade = unidade;
	}

}
