package br.com.centralit.api.model;

import java.util.Calendar;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import br.com.centralit.api.framework.json.ViewsEcm;
import br.com.centralit.framework.json.JsonCalendarDeserializer;
import br.com.centralit.framework.json.JsonCalendarSerializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilObjeto;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>Processo
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 16/10/2015 - 15:06:31
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
@Entity
@JsonIgnoreProperties({ "sugestao", "assuntoPlanoClassificacao" })
public class Processo extends PersistentObjectUnidade {

	public static final String GRID_PROCESSO_NA_UNIDADE = "GRID_PROCESSO_NA_UNIDADE";

	public static final String GRID_CLASSIFICACAO_PROCESSO_DOCUMENTO = "GRID_CLASSIFICACAO_PROCESSO_DOCUMENTO";

	/** Atributo serialVersionUID. */
	private static final long serialVersionUID = -6167513900417461697L;

	/** Atributo id. */
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@JsonView({ Views.GenericView.class })
	private Long id;

	/** Atributo assuntoComplementar. */
	@JsonView({ Views.ProcessoList.class, Views.UnidadeProcessoListView.class, ViewsEcm.MetadadoProcessoView.class, Views.ProcessoVHView.class })
	private String assuntoComplementar;

	/** Atributo dataReferencia. */
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ Views.UnidadeProcessoListView.class })
	private Calendar dataReferencia;

	/** Atributo dataConclusao. */
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSerializer.class)
	@JsonDeserialize(using = JsonCalendarDeserializer.class)
	private Calendar dataConclusao;

	/** Atributo nup. */
	@JsonView({ Views.ProcessoList.class, Views.ProcessoRelacionadoList.class, Views.AnexarProcesso.class, Views.ProcessoAutoCompleteView.class, Views.UnidadeProcessoListView.class, ViewsEcm.MetadadoProcessoView.class, Views.ProcessoVHView.class, Views.ReabrirProcessoListView.class})
	private String nup;

	/** Atributo nome. */
	private String nome;

	/** Atributo idTask. */
	@Transient
	private Long idTask;

	@JsonView({ ViewsEcm.ProcessoList.class })
	private Boolean aprovado;

	/** Atributo observacaoGeral. */
	@JsonView({ Views.ProcessoEdit.class, ViewsEcm.MetadadoProcessoView.class, Views.ProcessoVHView.class })
	private String observacaoGeral;

	@JsonView({ Views.ProcessoEdit.class })
	private String tipoAssunto;

	/** Atributo idProcessInstance. */
	@JsonView({ Views.ProcessoList.class, Views.UnidadeProcessoListView.class, Views.ProcessoVHView.class })
	private Long idProcessInstance;

	/** Atributo aprovado. */
	@JsonView({ ViewsEcm.ProcessoList.class })
	private Boolean validado;

	/** Atributo prazoSigilo. */
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsEcm.ProcessoList.class })
	private Calendar prazoSigilo;

	/** Atributo nup. */
	@JsonView({ Views.ProcessoEdit.class, ViewsEcm.MetadadoProcessoView.class })
	@ManyToOne(fetch = FetchType.LAZY)
	private Dominio tipoProtocolo;

	/** Atributo status. */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ Views.ProcessoEdit.class, Views.UnidadeProcessoListView.class, Views.ProcessoAutoCompleteView.class, ViewsEcm.MetadadoProcessoView.class, Views.ProcessoVHView.class })
	private Dominio status;

	/** Atributo tramitarDocumento. */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ Views.ProcessoEdit.class })
	private TramitarDocumento tramitarDocumento;

	/** Atributo tipoProcesso. */
	@JsonView({ Views.ProcessoList.class, Views.ProcessoRelacionadoList.class, Views.AnexarProcesso.class, Views.UnidadeProcessoListView.class, ViewsEcm.MetadadoProcessoView.class, Views.ProcessoAutoCompleteView.class, Views.ProcessoVHView.class })
	@ManyToOne(fetch = FetchType.LAZY)
	private TipoProcesso tipoProcesso;

	/** Atributo assunto. */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ Views.ProcessoList.class, Views.UnidadeProcessoListView.class, ViewsEcm.MetadadoProcessoView.class, Views.ProcessoVHView.class })
	private PlanoClassificacao assunto;

	/** Atributo nivelAcesso. */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ Views.ProcessoList.class, Views.UnidadeProcessoListView.class, ViewsEcm.MetadadoProcessoView.class, Views.ProcessoVHView.class })
	private NivelAcessoTipoProcesso nivelAcesso;

	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ Views.ProcessoList.class, ViewsEcm.MetadadoProcessoView.class })
	private NivelAcessoTipoProcesso nivelAcessoOriginal;

	/** Atributo anexoProcesso. */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ Views.ProcessoEdit.class })
	private AnexoProcesso anexoProcesso;

	/** Atributo hipoteseLegal. */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ Views.ProcessoEdit.class, ViewsEcm.MetadadoProcessoView.class, Views.ProcessoVHView.class })
	private HipoteseLegal hipoteseLegal;

	/** Atributo cienciaProcesso. */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ Views.ProcessoEdit.class })
	private CienciaProcesso cienciaProcesso;

	/** Atributo sigilo. */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ Views.ProcessoList.class, ViewsEcm.MetadadoProcessoView.class })
	private Sigilo sigilo;

	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ Views.ProcessoList.class, ViewsEcm.MetadadoProcessoView.class })
	private Sigilo sigiloOriginal;

	/** Atributo temporalidade. */
	@OneToOne
	@JoinColumn(name = "temporalidade")
	@JsonView({ Views.ProcessoList.class })
	private Temporalidade temporalidade;

	/** Atributo Interessados. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "processo", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ Views.ProcessoEdit.class, ViewsEcm.MetadadoProcessoView.class, Views.ProcessoVHView.class })
	private Collection<InteressadoProcesso> interessados;

	/** Atributo documentos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "processo", cascade = CascadeType.ALL)
	@JsonView({ Views.ProcessoEdit.class, Views.ProcessoVHView.class })
	private Collection<DocumentoGed> documentos;

	/** Atributo relacionados. */
	@ManyToMany
	@JoinTable(name = "processo_processorelacionado", joinColumns = { @JoinColumn(name = "processo_id") }, inverseJoinColumns = { @JoinColumn(name = "processo_relacionado_id") })
	private Collection<ProcessoRelacionado> relacionados;

	@OneToMany(fetch = FetchType.LAZY, mappedBy = "processo", cascade = CascadeType.ALL)
	@JsonView({ Views.ProcessoEdit.class })
	private Collection<JustificativaAcaoProcesso> justicativas;

	/** Atributo anexados. */
	@ManyToMany
	@JoinTable(name = "processo_anexado", joinColumns = { @JoinColumn(name = "processo_id") }, inverseJoinColumns = { @JoinColumn(name = "processo_anexado_id") })
	private Collection<AnexoProcesso> anexados;

	/** Atributo credenciais. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "processo", cascade = CascadeType.ALL)
	private Collection<CredencialProcesso> credenciais;

	/** Atributo unidadesProcesso. */
	@JsonView({ Views.ProcessoVHView.class })
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "processo", cascade = CascadeType.ALL)
	private Collection<UnidadeProcesso> unidadesProcesso;

	@Transient
	@JsonView({ Views.ProcessoEdit.class })
	private boolean tramitado;

	/**
	 * Retorna o valor do atributo <code>id</code>
	 * 
	 * @return <code>Long</code>
	 */
	public Long getId() {

		return id;
	}

	/**
	 * Define o valor do atributo <code>id</code>.
	 * 
	 * @param id
	 */
	public void setId(Long id) {

		this.id = id;
	}

	/**
	 * Retorna o valor do atributo <code>assuntoComplementar</code>
	 * 
	 * @return <code>String</code>
	 */
	public String getAssuntoComplementar() {

		return assuntoComplementar;
	}

	/**
	 * Define o valor do atributo <code>assuntoComplementar</code>.
	 * 
	 * @param assuntoComplementar
	 */
	public void setAssuntoComplementar(String assuntoComplementar) {

		this.assuntoComplementar = assuntoComplementar;
	}

	/**
	 * Retorna o valor do atributo <code>dataReferencia</code>
	 * 
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataReferencia() {

		return dataReferencia;
	}

	/**
	 * Define o valor do atributo <code>dataReferencia</code>.
	 * 
	 * @param dataReferencia
	 */
	public void setDataReferencia(Calendar dataReferencia) {

		this.dataReferencia = dataReferencia;
	}

	/**
	 * Retorna o valor do atributo <code>nup</code>
	 * 
	 * @return <code>String</code>
	 */
	public String getNup() {

		return nup;
	}

	/**
	 * Define o valor do atributo <code>nup</code>.
	 * 
	 * @param nup
	 */
	public void setNup(String nup) {

		this.nup = nup;
	}

	/**
	 * Retorna o valor do atributo <code>tipoProtocolo</code>
	 * 
	 * @return <code>Dominio</code>
	 */
	public Dominio getTipoProtocolo() {

		return tipoProtocolo;
	}

	/**
	 * Define o valor do atributo <code>tipoProtocolo</code>.
	 * 
	 * @param tipoProtocolo
	 */
	public void setTipoProtocolo(Dominio tipoProtocolo) {

		this.tipoProtocolo = tipoProtocolo;
	}

	/**
	 * Retorna o valor do atributo <code>observacaoGeral</code>
	 * 
	 * @return <code>String</code>
	 */
	public String getObservacaoGeral() {

		return observacaoGeral;
	}

	/**
	 * Define o valor do atributo <code>observacaoGeral</code>.
	 * 
	 * @param observacaoGeral
	 */
	public void setObservacaoGeral(String observacaoGeral) {

		this.observacaoGeral = observacaoGeral;
	}

	/**
	 * Retorna o valor do atributo <code>tramitarDocumento</code>
	 * 
	 * @return <code>TramitarDocumento</code>
	 */
	public TramitarDocumento getTramitarDocumento() {

		return tramitarDocumento;
	}

	/**
	 * Define o valor do atributo <code>tramitarDocumento</code>.
	 * 
	 * @param tramitarDocumento
	 */
	public void setTramitarDocumento(TramitarDocumento tramitarDocumento) {

		this.tramitarDocumento = tramitarDocumento;
	}

	/**
	 * Retorna o valor do atributo <code>tipoProcesso</code>
	 * 
	 * @return <code>TipoProcesso</code>
	 */
	public TipoProcesso getTipoProcesso() {

		return tipoProcesso;
	}

	/**
	 * Define o valor do atributo <code>tipoProcesso</code>.
	 * 
	 * @param tipoProcesso
	 */
	public void setTipoProcesso(TipoProcesso tipoProcesso) {

		this.tipoProcesso = tipoProcesso;
	}

	/**
	 * Retorna o valor do atributo <code>assunto</code>
	 * 
	 * @return <code>PlanoClassificacao</code>
	 */
	public PlanoClassificacao getAssunto() {

		return assunto;
	}

	/**
	 * Define o valor do atributo <code>assunto</code>.
	 * 
	 * @param assunto
	 */
	public void setAssunto(PlanoClassificacao assunto) {

		this.assunto = assunto;
	}

	/**
	 * Retorna o valor do atributo <code>nivelAcesso</code>
	 * 
	 * @return <code>NivelAcessoTipoProcesso</code>
	 */
	public NivelAcessoTipoProcesso getNivelAcesso() {

		return nivelAcesso;
	}

	/**
	 * Define o valor do atributo <code>nivelAcesso</code>.
	 * 
	 * @param nivelAcesso
	 */
	public void setNivelAcesso(NivelAcessoTipoProcesso nivelAcesso) {

		this.nivelAcesso = nivelAcesso;
	}

	/**
	 * Retorna o valor do atributo <code>anexoProcesso</code>
	 * 
	 * @return <code>AnexoProcesso</code>
	 */
	public AnexoProcesso getAnexoProcesso() {

		return anexoProcesso;
	}

	/**
	 * Define o valor do atributo <code>anexoProcesso</code>.
	 * 
	 * @param anexoProcesso
	 */
	public void setAnexoProcesso(AnexoProcesso anexoProcesso) {

		this.anexoProcesso = anexoProcesso;
	}

	/**
	 * Retorna o valor do atributo <code>hipoteseLegal</code>
	 * 
	 * @return <code>HipoteseLegal</code>
	 */
	public HipoteseLegal getHipoteseLegal() {

		return hipoteseLegal;
	}

	/**
	 * Define o valor do atributo <code>hipoteseLegal</code>.
	 * 
	 * @param hipoteseLegal
	 */
	public void setHipoteseLegal(HipoteseLegal hipoteseLegal) {

		this.hipoteseLegal = hipoteseLegal;
	}

	/**
	 * Retorna o valor do atributo <code>cienciaProcesso</code>
	 * 
	 * @return <code>CienciaProcesso</code>
	 */
	public CienciaProcesso getCienciaProcesso() {

		return cienciaProcesso;
	}

	/**
	 * Define o valor do atributo <code>cienciaProcesso</code>.
	 * 
	 * @param cienciaProcesso
	 */
	public void setCienciaProcesso(CienciaProcesso cienciaProcesso) {

		this.cienciaProcesso = cienciaProcesso;
	}

	/**
	 * Retorna o valor do atributo <code>interessados</code>
	 * 
	 * @return <code>Collection<InteressadoProcesso></code>
	 */
	public Collection<InteressadoProcesso> getInteressados() {

		return interessados;
	}

	/**
	 * Define o valor do atributo <code>interessados</code>.
	 * 
	 * @param interessados
	 */
	public void setInteressados(Collection<InteressadoProcesso> interessados) {

		this.interessados = interessados;
	}

	public String getTipoAssunto() {

		return tipoAssunto;
	}

	public void setTipoAssunto(String tipoAssunto) {

		this.tipoAssunto = tipoAssunto;
	}

	public Collection<DocumentoGed> getDocumentos() {

		return documentos;
	}

	public void setDocumentos(Collection<DocumentoGed> documentos) {

		this.documentos = documentos;
	}

	public static long getSerialversionuid() {

		return serialVersionUID;
	}

	/**
	 * Retorna o valor do atributo <code>relacionados</code>
	 * 
	 * @return <code>Collection<ProcessoRelacionado></code>
	 */
	public Collection<ProcessoRelacionado> getRelacionados() {

		return relacionados;
	}

	/**
	 * Define o valor do atributo <code>relacionados</code>.
	 * 
	 * @param relacionados
	 */
	public void setRelacionados(Collection<ProcessoRelacionado> relacionados) {

		this.relacionados = relacionados;
	}

	/**
	 * Retorna o valor do atributo <code>status</code>
	 * 
	 * @return <code>Dominio</code>
	 */
	public Dominio getStatus() {

		return status;
	}

	/**
	 * Define o valor do atributo <code>status</code>.
	 * 
	 * @param status
	 */
	public void setStatus(Dominio status) {

		this.status = status;
	}

	/**
	 * Retorna o valor do atributo <code>idTask</code>
	 * 
	 * @return <code>Long</code>
	 */
	public Long getIdTask() {

		return idTask;
	}

	/**
	 * Define o valor do atributo <code>idTask</code>.
	 * 
	 * @param idTask
	 */
	public void setIdTask(Long idTask) {

		this.idTask = idTask;
	}

	/**
	 * Retorna o valor do atributo <code>sigilo</code>
	 * 
	 * @return <code>Sigilo</code>
	 */
	public Sigilo getSigilo() {

		return sigilo;
	}

	/**
	 * Define o valor do atributo <code>sigilo</code>.
	 * 
	 * @param sigilo
	 */
	public void setSigilo(Sigilo sigilo) {

		this.sigilo = sigilo;
	}

	/**
	 * Retorna o valor do atributo <code>anexados</code>
	 * 
	 * @return <code>Collection<AnexoProcesso></code>
	 */
	public Collection<AnexoProcesso> getAnexados() {

		return anexados;
	}

	/**
	 * Define o valor do atributo <code>anexados</code>.
	 * 
	 * @param anexados
	 */
	public void setAnexados(Collection<AnexoProcesso> anexados) {

		this.anexados = anexados;
	}

	/**
	 * Retorna o valor do atributo <code>justicativas</code>
	 * 
	 * @return <code>Collection<JustificativaAcaoProcesso></code>
	 */
	public Collection<JustificativaAcaoProcesso> getJusticativas() {

		return justicativas;
	}

	/**
	 * Define o valor do atributo <code>justicativas</code>.
	 * 
	 * @param justicativas
	 */
	public void setJusticativas(Collection<JustificativaAcaoProcesso> justicativas) {

		this.justicativas = justicativas;
	}

	/**
	 * Retorna o valor do atributo <code>temporalidade</code>
	 * 
	 * @return <code>Temporalidade</code>
	 */
	public Temporalidade getTemporalidade() {

		return temporalidade;
	}

	/**
	 * Define o valor do atributo <code>temporalidade</code>.
	 * 
	 * @param temporalidade
	 */
	public void setTemporalidade(Temporalidade temporalidade) {

		this.temporalidade = temporalidade;
	}

	/**
	 * Retorna o valor do atributo <code>idProcessInstance</code>
	 * 
	 * @return <code>Long</code>
	 */
	public Long getIdProcessInstance() {

		return idProcessInstance;
	}

	/**
	 * Define o valor do atributo <code>idProcessInstance</code>.
	 * 
	 * @param idProcessInstance
	 */
	public void setIdProcessInstance(Long idProcessInstance) {

		this.idProcessInstance = idProcessInstance;
	}

	/**
	 * Retorna o valor do atributo <code>nome</code>
	 * 
	 * @return <code>String</code>
	 */
	public String getNome() {

		return nome;
	}

	/**
	 * Define o valor do atributo <code>nome</code>.
	 * 
	 * @param nome
	 */
	public void setNome(String nome) {

		this.nome = nome;
	}

	/**
	 * Retorna o valor do atributo <code>dataConclusao</code>
	 * 
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataConclusao() {

		return dataConclusao;
	}

	/**
	 * Define o valor do atributo <code>dataConclusao</code>.
	 * 
	 * @param dataConclusao
	 */
	public void setDataConclusao(Calendar dataConclusao) {

		this.dataConclusao = dataConclusao;
	}

	/**
	 * Retorna o valor do atributo <code>credenciais</code>
	 * 
	 * @return <code>Collection<CredencialProcesso></code>
	 */
	public Collection<CredencialProcesso> getCredenciais() {

		return credenciais;
	}

	/**
	 * Define o valor do atributo <code>credenciais</code>.
	 * 
	 * @param credenciais
	 */
	public void setCredenciais(Collection<CredencialProcesso> credenciais) {

		this.credenciais = credenciais;
	}

	/**
	 * Retorna o valor do atributo <code>prazoSigilo</code>
	 * 
	 * @return <code>Calendar</code>
	 */
	public Calendar getPrazoSigilo() {

		return prazoSigilo;
	}

	/**
	 * Define o valor do atributo <code>prazoSigilo</code>.
	 * 
	 * @param prazoSigilo
	 */
	public void setPrazoSigilo(Calendar prazoSigilo) {

		this.prazoSigilo = prazoSigilo;
	}

	/**
	 * Retorna o valor do atributo <code>nivelAcessoOriginal</code>
	 * 
	 * @return <code>NivelAcessoTipoProcesso</code>
	 */
	public NivelAcessoTipoProcesso getNivelAcessoOriginal() {

		return nivelAcessoOriginal;
	}

	/**
	 * Define o valor do atributo <code>nivelAcessoOriginal</code>.
	 * 
	 * @param nivelAcessoOriginal
	 */
	public void setNivelAcessoOriginal(NivelAcessoTipoProcesso nivelAcessoOriginal) {

		this.nivelAcessoOriginal = nivelAcessoOriginal;
	}

	/**
	 * Retorna o valor do atributo <code>sigiloOriginal</code>
	 * 
	 * @return <code>Sigilo</code>
	 */
	public Sigilo getSigiloOriginal() {

		return sigiloOriginal;
	}

	/**
	 * Define o valor do atributo <code>sigiloOriginal</code>.
	 * 
	 * @param sigiloOriginal
	 */
	public void setSigiloOriginal(Sigilo sigiloOriginal) {

		this.sigiloOriginal = sigiloOriginal;
	}

	/**
	 * Retorna o valor do atributo <code>validado</code>
	 * 
	 * @return <code>Boolean</code>
	 */
	public Boolean getValidado() {

		if (!UtilObjeto.isReferencia(this.validado)) {

			this.validado = Boolean.FALSE;

		}

		return validado;
	}

	/**
	 * Define o valor do atributo <code>validado</code>.
	 * 
	 * @param validado
	 */
	public void setValidado(Boolean validado) {

		this.validado = validado;
	}

	/**
	 * Retorna o valor do atributo <code>unidadesProcesso</code>
	 * 
	 * @return <code>Collection<UnidadeProcesso></code>
	 */
	public Collection<UnidadeProcesso> getUnidadesProcesso() {

		return unidadesProcesso;
	}

	/**
	 * Define o valor do atributo <code>unidadesProcesso</code>.
	 * 
	 * @param unidadesProcesso
	 */
	public void setUnidadesProcesso(Collection<UnidadeProcesso> unidadesProcesso) {

		this.unidadesProcesso = unidadesProcesso;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Verifica se o processo foi tramitado. Levando em consideração a unidadeProcesso criada apartir da criação do processo.
	 * 
	 * @author maycon.silva
	 * 
	 * @return boolean
	 */
	public boolean getTramitado() {

		return !UtilColecao.isVazio(this.getUnidadesProcesso()) && this.getUnidadesProcesso().size() > 1;
	}

}
