package br.com.centralit.api.service;

import org.springframework.web.multipart.MultipartFile;

import br.com.centralit.api.model.Assinatura;
import br.com.centralit.api.model.TarjaAssinatura;
import br.com.centralit.framework.service.arquitetura.GenericService;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governan&ccedil;a Corporativa -
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * Classe de servi&ccedil;os para a entidade TarjaAssinatura, a qual mant&eacute;m as regras de neg&oacute;cio.
 * 
 * @since 13 de abr de 2016 - 13:40:36
 * @version 1.0.0
 * @author ciro.junior (<a href="mailto:ciro.junior@centralit.com.br">ciro.junior@centralit.com.br</a>)
 * 
 */
public interface TarjaAssinaturaService extends GenericService<TarjaAssinatura, Long> {

	/**
	 * 
	 * M&eacute;todo respons&aacute;vel por fazer o upload da imagem do logotipo de uma tarja de assinatura.
	 * 
	 * @author ciro.junior (<a href="mailto:ciro.junior@centralit.com.br">ciro.junior@centralit.com.br</a>)
	 * 
	 * @param file
	 * @param idOrganizacao
	 * @param idConfiguracao
	 */
	void uploadAnexoImagemTarja(MultipartFile file, Long idOrganizacao, Long idConfiguracao);

	/**
	 * 
	 * M&eacute;todo respons&aacute;vel por salvar os dois tipos de tarjas de assinatura ao mesmo tempo
	 * 
	 * @author ciro.junior (<a href="mailto:ciro.junior@centralit.com.br">ciro.junior@centralit.com.br</a>)
	 * 
	 * @param tarjaAssinaturaVH
	 * @return
	 */
	TarjaAssinatura save(TarjaAssinatura tarjaAssinatura);

	/**
	 * 
	 * M&eacute;todo respons&aacute;vel por remover a imagem associada a uma tarja de assinatura.
	 * 
	 * @author ciro.junior (<a href="mailto:ciro.junior@centralit.com.br">ciro.junior@centralit.com.br</a>)
	 * 
	 * @param tarjaAssinatura
	 * @return
	 */
	TarjaAssinatura removeImagem(TarjaAssinatura tarjaAssinatura);

	/**
	 * Método responsável por obter tarja assinatura interna
	 * 
	 * @author rogerio.cassimiro
	 * @return {@link TarjaAssinatura}
	 */
	TarjaAssinatura getTarjaAssinaturaInterna();

	/**
	 * Método responsável por gerar assinatura interna por usuário e por documento
	 * 
	 * @author rogerio.cassimiro
	 * @param assinatura
	 * @param formaCriacao
	 */
	void gerarAssinaturaInternaUsuario(Assinatura assinatura, Long formaCriacao);

	/**
	 * Método responsável por criptografar o texto através da chave
	 * @author rogerio.cassimiro
	 * @param text
	 * @return {@link Byte}
	 */
	byte[] encrypted(String text);

	/**
	 * Método responsável por descriptografar texto
	 * @author rogerio.cassimiro
	 * @param text
	 * @return {@link String}
	 */
	String decrypted(byte[] text);
}
