package br.com.centralit.api.service.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import br.com.centralit.api.dao.AnexarProcessoDao;
import br.com.centralit.api.model.AnexoProcesso;
import br.com.centralit.api.model.Processo;
import br.com.centralit.api.service.AnexarProcessoService;
import br.com.centralit.api.service.AnexoGedService;
import br.com.centralit.api.service.DocumentoGedService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.ProcessoService;
import br.com.centralit.api.viewHelper.AnexarProcessoVH;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.service.arquitetura.GedFileService;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilDate;

/**
 * 
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 10/02/2016 - 16:46:44
 * 
 * @version 1.0.0
 * 
 * @author andre.silva
 * 
 */
@Service("anexarProcessoService")
public class AnexarProcessoServiceImpl extends GenericServiceImpl<AnexoProcesso, Long> implements AnexarProcessoService {

	private static final String VALIDAR_EXISTENCIA_ANEXO = "ECM.VALIDACAO.VALIDAR_EXISTENCIA_ANEXO";

	private static final String VALIDAR_TEMPO_CRIACAO_ANEXO = "ECM.VALIDACAO.VALIDAR_TEMPO_CRIACAO_ANEXO";

	/** Atributo unidadeProcessoDao. */
	private AnexarProcessoDao anexarProcessoDao;

	/** Atributo processoService. */
	@Autowired
	private ProcessoService processoService;

	/** Atributo anexoGedService. */
	@Autowired
	private AnexoGedService anexoGedService;

	/** Atributo GedFileService. */
	@Autowired
	private GedFileService gedFileService;

	@Autowired
	private DocumentoGedService documentoGedService;
	
	@Autowired 
	private DominioService dominioService;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * 
	 * @param unidadeProcessoDao
	 */
	@Autowired
	public AnexarProcessoServiceImpl( AnexarProcessoDao anexarProcessoDao ) {

		this.dao = anexarProcessoDao;

		this.anexarProcessoDao = anexarProcessoDao;
	}

	/**
	 * 
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author andre.silva
	 * 
	 * @param anexarProcessoVH
	 * @return AnexarProcessoVH
	 */
	@Override
	public AnexarProcessoVH save(AnexarProcessoVH anexarProcessoVH) {

		this.validarExistenciaAnexo(anexarProcessoVH);

		List<Processo> listProcesso = new ArrayList<Processo>();
		Processo processoPai = this.processoService.getReference(anexarProcessoVH.getProcesso().getId());

		if (!UtilColecao.isVazio(anexarProcessoVH.getListaProcessoAnexado())) {

			for (AnexoProcesso anexoProcesso : anexarProcessoVH.getListaProcessoAnexado()) {

				Processo processoFilho = this.processoService.getReference(anexoProcesso.getProcessoAnexado().getId());
				
				processoFilho.setStatus(this.dominioService.findByChaveAndCodigo(Dominio.TIPO_STATUS_PROCESSO_ECM, 5L));

				if (UtilDate.diferencaHoras(processoPai.getDataCriacao().getTime(), processoFilho.getDataCriacao().getTime()) < 0) {
					
					listProcesso.add(processoFilho);
	
				}else{
					
					// Vincula o processo filho ao pai
					this.vincularProcesso(processoPai, processoFilho);
					
					// Realiza o vinculo inverso, o pai vira filho e o filho vira pai.
					this.vincularProcesso(processoFilho, processoPai);
					
										
					if (!UtilColecao.isVazio(processoFilho.getDocumentos())) {
						documentoGedService.anexarDocumento(processoPai, processoFilho);
					}
					
					this.finalizeProcesso("FINALIZAR_PROCESSO_SIGAD", false, processoFilho.getIdProcessInstance());
					this.processoService.atualizarTemporalidadeProcessoAnexado(processoPai, processoFilho);
				}
			}
			
			if (!UtilColecao.isVazio(listProcesso)) {
				this.validarTempoCriacaoProcesso(listProcesso, processoPai);
				return null;
			}

		}
		
		return anexarProcessoVH;
	}
	
	
	/**
	 * 
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por finalizar o processo
	 *
	 * @author andre.silva
	 *
	 * @param eventTarget
	 * @param eventName
	 * @param flowName
	 * @param inputVariables
	 * @param synchronous
	 * @param processId
	 */
	private void finalizeProcesso(String eventName, boolean synchronous, Long processId){
		this.startNodeWithSignalEvent(eventName, processId, false, null);
//		SignalEventInput input = new SignalEventInput(eventTarget, eventName, flowName, null, false);
//		input.setProcessInstanceId(processId);
//		this.signalEvent(input);
	}
	
	

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por criar o vinculo do processo com 
	 * 
	 * 
	 * @param processoPrincipal
	 *            - Processo receberar a lista de relação de processos
	 * @param processo
	 *            - Processo que será vinculado ao processo pai
	 * 
	 */
	private void vincularProcesso(Processo processoPai, Processo processoFilho) {

		AnexoProcesso processoAnexo = new AnexoProcesso();

		processoAnexo.setProcessoAnexado(processoFilho);

		processoAnexo.setProcesso(processoPai);

		processoAnexo.setAnexado(Boolean.TRUE);

		processoAnexo = this.save(processoAnexo);

		if (!UtilColecao.isVazio(processoPai.getAnexados())) {
			// Adiciona a relação do filho ao pai
			processoPai.getAnexados().add(processoAnexo);

		} else {
			// Cria lista de processo filho no pai
			processoPai.setAnexados(new LinkedList<AnexoProcesso>());
			// Adiciona a relação do filho ao pai
			processoPai.getAnexados().add(processoAnexo);
		}
	}

	@Override
	public Collection<AnexoProcesso> findByIdProcesso(Long id) {

		return this.anexarProcessoDao.findByIdProcesso(id);
	}

	/**
	 * 
	 */
	public boolean removerProcessoAnexado(AnexarProcessoVH anexarProcessoVH) {

		Processo processoPai = this.processoService.getReference(anexarProcessoVH.getProcesso().getId());

		if (!UtilColecao.isVazio(anexarProcessoVH.getListRemoverAnexado())) {

			for (AnexoProcesso processoAnexado : anexarProcessoVH.getListRemoverAnexado()) {
				// Remove vinculo do processoRelacionado(Processo Filho) do pai
				this.removerVinculoProcesoAnexado(processoAnexado, processoPai);

				// Consulta o processo filho e transforma ele em pai para fazer a remoção inversa.
				Processo processoPaiInverso = this.processoService.getReference(processoAnexado.getProcessoAnexado().getId());

				// Remove o vinculo inverso o Processo pai vira filho e o fiho vira Pai.
				this.removerVinculoProcesoAnexado(this.anexarProcessoDao.anexoProcessoByIdProcesso(processoPai.getId(), processoAnexado.getProcessoAnexado().getId()), processoPaiInverso);

			}
		}

		return true;
	}

	/**
	 * 
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author andre.silva
	 * 
	 * @param anexoProcesso
	 * @param processoPai
	 */
	private void removerVinculoProcesoAnexado(AnexoProcesso anexoProcesso, Processo processoPai) {

		processoPai.getAnexados().remove(anexoProcesso);

		AnexoProcesso anexoProcessoRemocao = null;

		if (anexoProcesso != null && anexoProcesso.getId() != null) {
			anexoProcessoRemocao = this.getReference(anexoProcesso.getId());
			anexoProcessoRemocao.setProcessoRemocao(anexoProcesso.getProcessoAnexado());
			anexoProcessoRemocao.setProcessoAnexado(null);
		}

	}

	/**
	 * 
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author andre.silva
	 * 
	 * @param dataInicial
	 * @param dataFinal
	 * @return
	 */
	private void validarTempoCriacaoProcesso(List<Processo> listProcesso, Processo processoPai) {

		StringBuffer mensagem = new StringBuffer("");

		for (Processo processo : listProcesso) {
			mensagem.append(processo.getNup()).append(",");
		}

		if (mensagem.length() > 0) {
			throw new BusinessException(VALIDAR_TEMPO_CRIACAO_ANEXO, CodigoErro.REGRA_NEGOCIO.getValue(), mensagem.toString().substring(0, mensagem.length() - 1));
		}
	}

	/**
	 * 
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author andre.silva
	 * 
	 * @param anexarProcessoVH
	 * @return
	 */
	@Override
	public void validarExistenciaAnexo(AnexarProcessoVH anexarProcessoVH) {

		StringBuffer mensagem = new StringBuffer("");

		if (!UtilColecao.isVazio(anexarProcessoVH.getListaProcessoAnexado())) {

			for (AnexoProcesso processoAnexado : anexarProcessoVH.getListaProcessoAnexado()) {

				if (this.anexarProcessoDao.isExisteAnexoProcesso(processoAnexado.getProcessoAnexado().getId())) {
					mensagem.append(processoAnexado.getProcessoAnexado().getNup()).append(",");
				}

			}
		}

		if (mensagem.length() > 0) {
			throw new BusinessException(VALIDAR_EXISTENCIA_ANEXO, CodigoErro.REGRA_NEGOCIO.getValue(), mensagem.toString().substring(0, mensagem.length() - 1));
		}
	}

	@Override
	public Collection<AnexoProcesso> findByIdAnexoProcesso(Long id) {

		return this.anexarProcessoDao.findByIdAnexoProcesso(id);
	}

}
