package br.com.centralit.api.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import br.com.centralit.api.dao.AnexoGedDao;
import br.com.centralit.api.model.AnexoGed;
import br.com.centralit.api.model.DocumentoGed;
import br.com.centralit.api.service.AnexoGedService;
import br.com.centralit.api.service.DocumentoGedService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.SolrService;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.service.arquitetura.GedFileService;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 16/12/2015 - 15:40:28
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
@Service("anexoGedService")
public class AnexoGedServiceImpl extends GenericServiceImpl<AnexoGed, Long> implements AnexoGedService {

	/** Atributo anexoGedDao. */
	private AnexoGedDao anexoGedDao;

	/** Atributo GedFileService. */
	@Autowired
	private GedFileService gedFileService;

	/** Atributo processoService. */
	@Autowired
	private DocumentoGedService documentoGedService;

	/** Atributo dominioService. */
	@Autowired
	private DominioService dominioService;

	/** Atributo solrService. */
	@Autowired
	private SolrService solrService;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * 
	 * @param anexoGedDao
	 */
	@Autowired
	private AnexoGedServiceImpl( AnexoGedDao anexoGedDao ) {

		this.dao = anexoGedDao;

		this.anexoGedDao = anexoGedDao;

	}

	@Override
	public void saveAnexo(MultipartFile multipartFile, Long idDocumento) {

		if (UtilObjeto.isReferencia(multipartFile)) {

			DocumentoGed documentoGed = this.documentoGedService.find(idDocumento);

			StringBuilder sb = new StringBuilder();

			sb.append(UtilDate.getAno(documentoGed.getProcesso().getDataCriacao().getTime())).append("/").append(UtilDate.getMes(documentoGed.getProcesso().getDataCriacao().getTime()));

			AnexoGed anexo = new AnexoGed(multipartFile.getOriginalFilename(), multipartFile.getSize(), sb.toString(), documentoGed.getProcesso().getId().toString(), idDocumento.toString(), String.valueOf(documentoGed.getVersaoDocumento()), multipartFile);

			try {

				this.gedFileService.addFile(anexo);

				// FAZ OCR DO DOCUMENTO ANEXADO E JÁ MANDA PARA INDEXAÇÃO

				documentoGed.setConteudo(this.gedFileService.doOcr(anexo));

//				this.solrService.addDocumento(documentoGed);

				anexo.setDocumentoGed(documentoGed);

				this.montarDadosAnexo(multipartFile, anexo);

				this.save(anexo);

				this.documentoGedService.saveIconeDocumento(documentoGed, anexo.getDominioTipoAnexo());

			} catch (Exception e) {
				e.printStackTrace();
			}

		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author rogerio.costa
	 * 
	 * @param multipartFile
	 * @param anexo
	 */
	private void montarDadosAnexo(MultipartFile multipartFile, AnexoGed anexo) {

		String extensao = multipartFile.getOriginalFilename().contains(".") ? multipartFile.getOriginalFilename().split("\\.")[1] : "";

		anexo.setDescricao(multipartFile.getOriginalFilename());

		// verifica se nao existe estensão, se existir e realizada a busca do dominio desse tipo de extenção
		if (StringUtils.isEmpty(extensao.trim())) {

			anexo.setDominioTipoAnexo(this.dominioService.findByChaveAndCodigo(Dominio.TIPO_ANEXO, Dominio.TIPO_ANEXO_XLS_SEM_EXTENSAO_CODIGO));

		} else {

			// repetição para verifica e comparar as extensões que estão presentes no sistema
			for (Dominio dominio : this.dominioService.listarPorChave(Dominio.TIPO_ANEXO)) {

				if (dominio.getNome().equalsIgnoreCase(extensao.trim())) {

					anexo.setDominioTipoAnexo(dominio);

					break;
				}
			}

			if (!UtilObjeto.isReferencia(anexo.getDominioTipoAnexo())) {

				anexo.setDominioTipoAnexo(this.dominioService.findByChaveAndCodigo(Dominio.TIPO_ANEXO, Dominio.TIPO_ANEXO_XLS_SEM_EXTENSAO_CODIGO));
			}
		}

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idDocumento
	 * @return
	 */
	@Override
	public AnexoGed obterPorIdDocumento(Long idDocumento) {

		return this.anexoGedDao.obterPorIdDocumento(idDocumento);
	}

	@Override
	public boolean removeById(Long id) {

		AnexoGed anexoGed = this.getReference(id);

		anexoGed.setDocumentoRemocao(anexoGed.getDocumentoGed());

		anexoGed.setDocumentoGed(null);

		return super.removeById(id);
	}

}
