package br.com.centralit.api.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import br.com.centralit.api.dao.AssinaturaDao;
import br.com.centralit.api.model.Assinatura;
import br.com.centralit.api.model.DocumentoGed;
import br.com.centralit.api.service.AssinaturaService;
import br.com.centralit.api.service.DocumentoGedService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.FuncaoService;
import br.com.centralit.api.service.HistoricoAlteracaoProcessoService;
import br.com.centralit.api.service.PessoaService;
import br.com.centralit.api.service.TarjaAssinaturaService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilObjeto;
import br.com.centralit.framework.util.UtilString;

/**
 * <p>
 * <b>Title: </b>
 * </p>
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * @since 22/04/2016 - 11:50:56
 * @author rogerio.cassimiro
 * 
 */
@Service("assinaturaService")
public class AssinaturaServiceImpl extends GenericServiceImpl<Assinatura, Long> implements AssinaturaService {

	@Autowired
	private AssinaturaDao assinaturaDao;

	@Autowired
	private DocumentoGedService documentoGedService;

	@Autowired
	private UsuarioService usuarioService;

	@Autowired
	private DominioService dominioService;

	@Autowired
	private FuncaoService funcaoService;

	@Autowired
	private PessoaService pessoaService;

	@Autowired
	private HistoricoAlteracaoProcessoService historicoAlteracaoProcessoService;

	@Autowired
	private TarjaAssinaturaService tarjaAssinaturaService;

	@Value("${metodo.autenticacao.ldap}")
	private String ldap;

	@Autowired
	public AssinaturaServiceImpl( AssinaturaDao assinaturaDao ) {

		this.dao = assinaturaDao;
	}

	/**
	 * Salva assinatura do documento e gera histórico
	 */
	@Override
	public Assinatura saveAssinaturaInterna(Assinatura assinatura) {

		Usuario usuario = (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
		validarSenha(assinatura.getPassword(), usuario);
		montarTransients(assinatura, usuario);
		historicoAlteracaoProcessoService.gerarHistoricoAssinatura(assinatura);
		tarjaAssinaturaService.gerarAssinaturaInternaUsuario(assinatura, assinatura.getDocumentoGed().getFormaCriacao().getCodigo());
		alterarEstadoDocumento(assinatura.getDocumentoGed());
		return super.save(assinatura);
	}

	/**
	 * Método responsável por alterar estado do documento se a forma for online
	 * 
	 * @author rogerio.cassimiro
	 * @param documentoGed
	 */
	private void alterarEstadoDocumento(DocumentoGed documentoGed) {

		if (documentoGed.getFormaCriacao().getCodigo().equals(1L) && !documentoGed.getEstado().getCodigo().equals(2L))
			documentoGed.setEstado(dominioService.findByChaveAndCodigo("estadoDocumentoGed", 2L));
	}

	/**
	 * Método responsável por montar transients da assinatura
	 * 
	 * @author rogerio.cassimiro
	 * @param assinatura
	 * @param usuario
	 */
	private void montarTransients(Assinatura assinatura, Usuario usuario) {

		assinatura.setDocumentoGed(this.documentoGedService.getReference(assinatura.getDocumentoGed().getId()));
		assinatura.setUsuario(usuario);
		assinatura.setPessoa(this.pessoaService.getReference(assinatura.getPessoa().getId()));
		assinatura.setTipoAssinatura(dominioService.findByChaveAndCodigo(Dominio.TIPO_ASSINATURA, Dominio.TIPO_ASSINATURA_INTERNA));
		assinatura.setFuncao(funcaoService.getReference(assinatura.getFuncao().getId()));
	}

	/**
	 * Método responsável por validar senha para assinar o documento
	 * 
	 * @author rogerio.cassimiro
	 * @param password
	 * @param usuario
	 */
	private void validarSenha(String password, Usuario usuario) {

		if (!UtilString.isNullOrEmpty(ldap) && ldap.equals("false")) {

			if (UtilString.isNullOrEmpty(password))
				throw new BusinessException("ECM.VALIDACAO.SENHA_ASSINATURA", CodigoErro.REGRA_NEGOCIO.getValue(), "");

			if (!UtilObjeto.isReferencia(this.usuarioService.loadUserByUsernamePasswordMobile(usuario.getUsername(), password)))
				throw new BusinessException("ECM.VALIDACAO.SENHA_ASSINATURA_INVALIDA", CodigoErro.REGRA_NEGOCIO.getValue(), "");

		} else {
			// TODO LDAP validação
		}
	}

	/**
	 * Método responsável por validar se o usuário já teve assinatura anterior para o documento informado
	 * 
	 * @author rogerio.cassimiro
	 * @return {@link Boolean}
	 */
	@Override
	public Boolean validarAssinaturaPorUsuario(Long idDocumento) {

		return this.assinaturaDao.validarAssinaturaPorUsuario(idDocumento, ( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
	}

	@Override
	public Assinatura getReference(Long id) {

		Assinatura assinatura = (Assinatura) this.assinaturaDao.getReference(id);
		assinatura.getConteudoTarja().setConteudo(tarjaAssinaturaService.decrypted(assinatura.getConteudoTarja().getConteudoCriptografado()));
		if (UtilObjeto.isReferencia(assinatura.getDocumentoGed().getConteudoTarja())) {
			assinatura.getDocumentoGed().getConteudoTarja().setConteudo(tarjaAssinaturaService.decrypted(assinatura.getDocumentoGed().getConteudoTarja().getConteudoCriptografado()));
		}
		return assinatura;
	}

}
