package br.com.centralit.api.service.impl;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.security.InvalidKeyException;
import java.security.KeyFactory;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import br.com.centralit.api.service.AssinaturaUtilService;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>AssinaturaUtilServiceImpl
 * </p>
 * 
 * <p>
 * <b>Description: </b> Classe responsável por assinar e validar autenticidade do documento.
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 30/04/2016 - 13:47:59
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
@Service("assinaturaUtilService")
public class AssinaturaUtilServiceImpl implements AssinaturaUtilService {

	/** Atributo publicKeyFile. */
	@Value("${ecm.signature.publicKeyFile}")
	private File publicKeyFile;

	/** Atributo privateKeyFile. */
	@Value("${ecm.signature.privateKeyFile}")
	private File privateKeyFile;

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por assinar o documento
	 * 
	 * @author rogerio.costa
	 * 
	 * @param mensagem
	 * @return byte[]
	 * @throws InvalidKeyException
	 * @throws Exception
	 */
	public byte[] geraAssinatura(byte[] conteudo) throws Exception {

		Signature sig = Signature.getInstance("SHA1withRSA");

		// Inicializando Obj Signature com a Chave Privada
		sig.initSign(this.getPrivateKey());

		// Gerar assinatura
		sig.update(this.gerarHash(conteudo));

		byte[] assinatura = sig.sign();

		return assinatura;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author rogerio.costa
	 * 
	 * @param conteudo
	 * @param assinatura
	 * @throws Exception
	 */
	public Boolean validarAutenticidade(byte[] conteudo, byte[] assinatura) throws Exception {

		Signature clientSig = Signature.getInstance("SHA1withRSA");

		clientSig.initVerify(getPublicKey());

		clientSig.update(this.gerarHash(conteudo));

		return clientSig.verify(assinatura);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter a chave privada
	 * 
	 * @author rogerio.costa
	 * 
	 * @return PrivateKey
	 * 
	 * @throws Exception
	 */
	public PrivateKey getPrivateKey() throws Exception {

		FileInputStream fis = new FileInputStream(this.privateKeyFile);

		DataInputStream dataInputStream = new DataInputStream(fis);

		byte[] keyBytes = new byte[(int) this.privateKeyFile.length()];

		dataInputStream.readFully(keyBytes);

		dataInputStream.close();

		PKCS8EncodedKeySpec spec = new PKCS8EncodedKeySpec(keyBytes);

		KeyFactory keyFactory = KeyFactory.getInstance("RSA");

		return keyFactory.generatePrivate(spec);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter a chace pública
	 * 
	 * @author rogerio.costa
	 * 
	 * @return
	 * @throws Exception
	 */
	public PublicKey getPublicKey() throws Exception {

		FileInputStream fis = new FileInputStream(this.publicKeyFile);

		DataInputStream dataInputStream = new DataInputStream(fis);

		byte[] keyBytes = new byte[(int) this.publicKeyFile.length()];

		dataInputStream.readFully(keyBytes);

		dataInputStream.close();

		X509EncodedKeySpec spec = new X509EncodedKeySpec(keyBytes);

		KeyFactory keyFactory = KeyFactory.getInstance("RSA");

		return keyFactory.generatePublic(spec);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por gerar o hash do documento
	 * 
	 * @author rogerio.costa
	 * 
	 * @param doc
	 * @return
	 * @throws NoSuchAlgorithmException
	 */
	public byte[] gerarHash(byte[] doc) throws NoSuchAlgorithmException {

		MessageDigest md = MessageDigest.getInstance("SHA-1");

		md.update(doc);

		return md.digest();
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por por gerar o hash do documento
	 * 
	 * @author rogerio.costa
	 * 
	 * @param doc
	 * @return
	 * @throws NoSuchAlgorithmException
	 */
	public String gerarHash(String doc) throws NoSuchAlgorithmException {

		MessageDigest digest = MessageDigest.getInstance("SHA-1");

		digest.update(doc.getBytes());

		return convertByteArrayToHexString(digest.digest());
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por converter byte para String
	 * 
	 * @author rogerio.costa
	 * 
	 * @param arrayBytes
	 * 
	 * @return String
	 */
	private static String convertByteArrayToHexString(byte[] arrayBytes) {

		StringBuffer stringBuffer = new StringBuffer();
		for (int i = 0; i < arrayBytes.length; i++) {
			stringBuffer.append(Integer.toString(( arrayBytes[i] & 0xff ) + 0x100, 16).substring(1));
		}
		return stringBuffer.toString();
	}

}
