package br.com.centralit.api.service.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import br.com.centralit.api.dao.CredencialProcessoDao;
import br.com.centralit.api.model.CredencialProcesso;
import br.com.centralit.api.model.HistoricoCredencialProcesso;
import br.com.centralit.api.model.Processo;
import br.com.centralit.api.model.Sigilo;
import br.com.centralit.api.model.SigiloPrivilegio;
import br.com.centralit.api.service.CredencialProcessoService;
import br.com.centralit.api.service.GrupoService;
import br.com.centralit.api.service.HistoricoCredencialProcessoService;
import br.com.centralit.api.service.ProcessoService;
import br.com.centralit.api.service.SigiloService;
import br.com.centralit.api.service.UnidadeProcessoService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.api.viewHelper.GrupoUsuarioVH;
import br.com.centralit.api.viewHelper.HerancaSistemaVH;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Grupo;
import br.com.centralit.framework.model.GrupoUsuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 08/03/2016 - 16:48:07
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
@Service("credencialProcessoService")
public class CredencialProcessoServiceImpl extends GenericServiceImpl<CredencialProcesso, Long> implements CredencialProcessoService {

	/** Atributo credencialProcessoDao. */
	private CredencialProcessoDao credencialProcessoDao;

	/** Atributo processoService. */
	@Autowired
	private ProcessoService processoService;

	/** Atributo grupoService. */
	@Autowired
	private GrupoService grupoService;

	/** Atributo usuarioService. */
	@Autowired
	private UsuarioService usuarioService;

	/** Atributo historicoCredencialProcessoService. */
	@Autowired
	private HistoricoCredencialProcessoService historicoCredencialProcessoService;

	/** Atributo sigiloService. */
	@Autowired
	private SigiloService sigiloService;

	/** Atributo unidadeProcessoService. */
	@Autowired
	private UnidadeProcessoService unidadeProcessoService;

	/** Atributo request. */
	@Autowired
	private HttpServletRequest request;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * 
	 * @param credencialProcessoDao
	 */
	private static final String SEM_PERMISSAO_OPERACAO = "ECM.VALIDACAO.PERMISSAO_OPERACAO";

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * 
	 * @param credencialProcessoDao
	 */
	@Autowired
	public CredencialProcessoServiceImpl( CredencialProcessoDao credencialProcessoDao ) {

		this.dao = credencialProcessoDao;

		this.credencialProcessoDao = credencialProcessoDao;

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por salvar credenciais
	 * 
	 * @author rogerio.costa
	 * 
	 * @param credenciais
	 * @return
	 */
	@Override
	public Collection<CredencialProcesso> save(Collection<CredencialProcesso> credenciais) {

		this.resolveTransiente(credenciais);

		credenciais = super.saveList((List<CredencialProcesso>) credenciais);

		return this.montarPermissaoEdicao(credenciais);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por salvar credenciais iniciais do processo/ Grupos que contem permissão para acessar o processo e Usuário que criou o processo.
	 * 
	 * @author rogerio.costa
	 * 
	 * @param processo
	 */
	@Override
	public void saveCredenciais(Processo processo) {

		if (UtilObjeto.isReferencia(processo.getSigilo())) {

			Sigilo sigilo = this.sigiloService.getReference(processo.getSigilo().getId());

			if (!UtilColecao.isVazio(sigilo.getSigiloPrivilegios())) {

				Collection<Grupo> grupos = findGrupoPorSigilo(sigilo);

				if (!UtilColecao.isVazio(grupos)) {

					for (Grupo grupo : grupos) {

						// Gera a credencial dos grupos padrão.
						super.save(new CredencialProcesso(processo, grupo, Boolean.TRUE));
					}

				}

			}
			// Gera a credencial do usuario que crio o processo
			super.save(new CredencialProcesso(processo, getUsuario()));

		}

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por retornar os privilegios através do sigilo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param sigilo
	 * @return Collection<Privilegio>
	 */
	private Collection<Long> getPrivilegios(Sigilo sigilo) {

		Collection<Long> listIdsPrivilegio = new LinkedList<Long>();
		// Cria uma lista de privilegios relacionados ao sigilo do processo
		for (SigiloPrivilegio sigiloPrivilegio : sigilo.getSigiloPrivilegios()) {

			listIdsPrivilegio.add(sigiloPrivilegio.getPrivilegio().getId());
		}
		return listIdsPrivilegio;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por retornar os privilegios através do sigilo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param sigilo
	 * @return Collection<Privilegio>
	 */
	private Collection<String> getRolesPorSigilo(Sigilo sigilo) {

		Collection<String> roles = new ArrayList<String>();
		// Cria uma lista de privilegios relacionados ao sigilo do processo
		for (SigiloPrivilegio sigiloPrivilegio : sigilo.getSigiloPrivilegios()) {

			roles.add(sigiloPrivilegio.getPrivilegio().getNome());
		}
		return roles;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por resolver as propriedades transiente
	 * 
	 * @author rogerio.costa
	 * 
	 * @param credenciais
	 */
	private void resolveTransiente(Collection<CredencialProcesso> credenciais) {

		List<String> grupos = new LinkedList<String>();
		List<String> usuarios = new LinkedList<String>();

		for (CredencialProcesso credencialProcesso : credenciais) {

			credencialProcesso.setProcesso(this.processoService.getReference(credencialProcesso.getProcesso().getId()));
			// Verifica se a credencial é do grupo
			if (UtilObjeto.isReferencia(credencialProcesso.getGrupo())) {

				credencialProcesso.setGrupo(this.grupoService.getReference(credencialProcesso.getGrupo().getId()));
				grupos.add(credencialProcesso.getGrupo().getSigla());
			} else {

				credencialProcesso.setUsuario(this.usuarioService.find(credencialProcesso.getUsuario().getId()));
				usuarios.add(credencialProcesso.getUsuario().getUsername());

			}
			// Verifica se a credencial é nova, caso seja cria um histórico
			if (credencialProcesso.isNew()) {

				credencialProcesso.setHistoricoCredencialProcessos(new LinkedList<HistoricoCredencialProcesso>());
				credencialProcesso.getHistoricoCredencialProcessos().add(this.addHistorico(credencialProcesso));
				// Delega o processo para os grupos e usuario credenciados
				this.delegateUserTask(credencialProcesso.getIdTask(), grupos, usuarios);
			}
		}

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar se o usuario logado não é o criador do processo, e se contem uma credencial para o mesmo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idProcesso
	 * @return boolean
	 */
	public Boolean permiteRenunciar(Long idProcesso) {

		Processo processo = this.processoService.getReference(idProcesso);

		if (!this.getUsuario().getId().equals(processo.getAutor().getId())) {

			return this.credencialProcessoDao.contemCredencial(this.getUsuario().getId(), idProcesso);
		}

		return Boolean.FALSE;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por criar histórico
	 * 
	 * @author rogerio.costa
	 * 
	 * @param credencialProcesso
	 * @return HistoricoCredencialProcesso
	 */
	private HistoricoCredencialProcesso addHistorico(CredencialProcesso credencialProcesso) {

		HistoricoCredencialProcesso historicoCredencialProcesso = new HistoricoCredencialProcesso();

		historicoCredencialProcesso.setCredencialProcesso(credencialProcesso);

		if (UtilObjeto.isReferencia(credencialProcesso.getGrupo())) {

			historicoCredencialProcesso.setGrupo(credencialProcesso.getGrupo());
		} else {
			historicoCredencialProcesso.setUsuario(credencialProcesso.getUsuario());
		}

		return historicoCredencialProcesso;
	}

	@Override
	public boolean removeById(Long id) {

		CredencialProcesso credencialProcesso = this.getReference(id);

		this.validarPermissaoRemover(credencialProcesso);

		this.executarAcoesRemocao(credencialProcesso);

		return super.removeById(id);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por validar a permissão
	 * 
	 * @author rogerio.costa
	 * 
	 * @param credencialProcesso
	 */
	private void validarPermissaoRemover(CredencialProcesso credencialProcesso) {

		if (!credencialProcesso.getAutor().getId().equals(this.getUsuario().getId()) && !this.contemPermissaoHerdada(credencialProcesso.getProcesso().getSigilo())) {

			throw new BusinessException(SEM_PERMISSAO_OPERACAO, CodigoErro.REGRA_NEGOCIO.getValue());

		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por executar ações da remoção
	 * 
	 * @author rogerio.costa
	 * 
	 * @param credencialProcesso
	 */
	private void executarAcoesRemocao(CredencialProcesso credencialProcesso) {

		this.removeDelegacao(credencialProcesso);
		Processo processo = this.processoService.getReference(credencialProcesso.getProcesso().getId());
		credencialProcesso.setProcessoRemocao(processo);
		credencialProcesso.setProcesso(null);
		this.saveHistoricoDescredenciado(credencialProcesso);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por remover a delegação do grupo/usuario no processo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param credencialProcesso
	 */
	private void removeDelegacao(CredencialProcesso credencialProcesso) {

		List<String> listGrupoUsuarioRemocao = new LinkedList<String>();

		if (UtilObjeto.isReferencia(credencialProcesso.getGrupo())) {

			listGrupoUsuarioRemocao.add(credencialProcesso.getGrupo().getSigla());

			this.removeUserTaskDelegation(credencialProcesso.getIdTask(), listGrupoUsuarioRemocao, null);
		} else {

			listGrupoUsuarioRemocao.add(credencialProcesso.getUsuario().getUsername());

			this.removeUserTaskDelegation(credencialProcesso.getIdTask(), null, listGrupoUsuarioRemocao);

		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por salvar o histórico do descredenciamento
	 * 
	 * @author rogerio.costa
	 * 
	 * @param credencialProcesso
	 */
	private void saveHistoricoDescredenciado(CredencialProcesso credencialProcesso) {

		HistoricoCredencialProcesso historicoCredencialProcesso = this.historicoCredencialProcessoService.findByCredencialAndCredenciado(credencialProcesso);

		historicoCredencialProcesso.setDataDescredenciado(UtilDate.getDataAtualCalendar());
		historicoCredencialProcesso.setDescredenciador(getUsuario());

		this.historicoCredencialProcessoService.save(historicoCredencialProcesso);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por renunciar credencial.
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idProcesso
	 */
	public Boolean renunciar(Long idProcesso) {

		CredencialProcesso credencialProcesso = this.credencialProcessoDao.obterPorUsuarioEProcesso(idProcesso, this.getUsuario().getId());

		this.executarAcoesRemocao(credencialProcesso);

		return super.removeById(credencialProcesso.getId());
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por listar grupo e usuarios que tem credenciais herdadas do sistema.
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idProcesso
	 * 
	 * @return Collection<HerancaSistemaVH>
	 */
	public Collection<HerancaSistemaVH> findCredenciaisHerdadas(Long idProcesso) {

		Processo processo = this.processoService.getReference(idProcesso);

		Collection<HerancaSistemaVH> herancaList = new LinkedList<HerancaSistemaVH>();

		if (UtilObjeto.isReferencia(processo.getSigilo())) {

			Sigilo sigilo = this.sigiloService.getReference(processo.getSigilo().getId());

			Collection<Grupo> grupos = findGrupoPorSigilo(sigilo);

			if (!UtilColecao.isVazio(grupos)) {

				for (Grupo grupo : grupos) {

					herancaList.add(new HerancaSistemaVH(grupo.getId(), grupo.getNome(), "-"));
				}
			}

			herancaList.add(new HerancaSistemaVH(processo.getAutor().getId(), processo.getAutor().getUsername(), UtilObjeto.isReferencia(processo.getAutor().getUnidade()) ? processo.getAutor().getUnidade().getNome() : "-"));
		}

		return herancaList;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter grupos através do sigilo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param sigilo
	 * 
	 * @return Collection<Grupo>
	 */
	public Collection<Grupo> findGrupoPorSigilo(Sigilo sigilo) {

		Collection<Long> idPrivilegios = this.getPrivilegios(sigilo);

		Collection<Grupo> grupos = this.grupoService.findByPrivilegio(idPrivilegios);

		return grupos;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar se o usuarioLogado tem permissão herdadas do sistema para o sigilo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param sigilo
	 */
	public Boolean contemPermissaoHerdada(Sigilo sigilo) {

		Collection<Grupo> gruposHerdados = this.findGrupoPorSigilo(sigilo);

		for (Grupo grupo : gruposHerdados) {

			if (contemUsuarioLogadoNoGrupo(grupo))

				return Boolean.TRUE;

		}

		return Boolean.FALSE;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar se o usuario logado está alocado no grupo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param grupo
	 * 
	 * @return Boolean
	 */
	private Boolean contemUsuarioLogadoNoGrupo(Grupo grupo) {

		for (GrupoUsuario grupoUsuario : grupo.getGrupoUsuarios()) {

			if (grupoUsuario.getUsuario().getId().equals(this.getUsuario().getId())) {

				return Boolean.TRUE;

			}
		}

		return Boolean.FALSE;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por listar grupos e usuarios do sistema
	 * 
	 * @author rogerio.costa
	 * 
	 * @return Collection<GrupoUsuarioVH>
	 */
	@Override
	public Collection<GrupoUsuarioVH> findGrupoUsuarioByName(String nome) {

		return this.credencialProcessoDao.findGrupoUsuarioByName(nome);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar se o usuario tem permissão para credenciar
	 * 
	 * @author rogerio.costa
	 * 
	 */
	@Override
	public Boolean permiteCredenciar(Long idProcesso) {

		Processo processo = this.processoService.getReference(idProcesso);

		if (UtilObjeto.isReferencia(processo.getSigilo())) {

			Sigilo sigilo = this.sigiloService.getReference(processo.getSigilo().getId());

			// valida de se o usuario logado é autor do processo
			if (processo.getAutor().getId().equals(this.getUsuario().getId())) {

				return Boolean.TRUE;

			} else {

				CredencialProcesso credencialProcesso = this.credencialProcessoDao.obterPorUsuarioEProcesso(idProcesso, this.getUsuario().getId());

				// Verifica se o usuario contem credencial no processo.
				if (UtilObjeto.isReferencia(credencialProcesso)) {

					return credencialProcesso.getPermiteCredenciar();

					// Verifica se o usuario tem permissão herdada.
				} else if (contemPermissaoHerdada(sigilo)) {

					return Boolean.TRUE;

				} else {
					// Verifica as credenciais vinculadas no processo
					for (CredencialProcesso credencialProcessoVinculada : processo.getCredenciais()) {

						if (UtilObjeto.isReferencia(credencialProcessoVinculada.getGrupo())) {

							Grupo grupo = this.grupoService.getReference(credencialProcessoVinculada.getGrupo().getId());

							if (this.contemUsuarioLogadoNoGrupo(grupo)) {

								return credencialProcessoVinculada.getPermiteCredenciar();
							}
						}
					}

				}

			}
		}

		return Boolean.FALSE;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por listar através do id do processo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idProcesso
	 * @return Collection<CredencialProcesso>
	 */
	public Collection<CredencialProcesso> findByIdProcesso(Long id) {

		Collection<CredencialProcesso> credenciais = this.credencialProcessoDao.findByIdProcesso(id);

		return this.montarPermissaoEdicao(credenciais);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por montar a parmissão de edição do usuarioLogado
	 * 
	 * @author rogerio.costa
	 * 
	 * @param credenciais
	 * @return
	 */
	private Collection<CredencialProcesso> montarPermissaoEdicao(Collection<CredencialProcesso> credenciais) {

		if (!UtilColecao.isVazio(credenciais)) {

			for (CredencialProcesso credencialProcesso : credenciais) {

				credencialProcesso.setPermiteEdicao(credencialProcesso.getAutor().getId().equals(this.getUsuario().getId()) || this.contemPermissaoHerdada(credencialProcesso.getProcesso().getSigilo()));
			}

		}
		return credenciais;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar se o usuarioLogado tem permissão no processo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idProcesso
	 * @return boolean
	 */
	@Override
	public boolean contemPermissaoProcesso(Long idProcesso) {

		if (UtilObjeto.isReferencia(idProcesso)) {

			Processo processo = this.processoService.getReference(idProcesso);

			if (UtilObjeto.isReferencia(processo.getSigilo())) {

				CredencialProcesso credencialProcesso = this.credencialProcessoDao.obterPorUsuarioEProcesso(idProcesso, this.getUsuario().getId());

				if (UtilObjeto.isReferencia(credencialProcesso)) {

					return Boolean.TRUE;
				}

				Sigilo sigilo = this.sigiloService.getReference(processo.getSigilo().getId());

				if (contemPermissaoHerdada(sigilo)) {

					return Boolean.TRUE;
				}

				// Verifica as credenciais vinculadas no processo
				for (CredencialProcesso credencialProcessoVinculada : processo.getCredenciais()) {

					if (UtilObjeto.isReferencia(credencialProcessoVinculada.getGrupo())) {

						Grupo grupo = this.grupoService.getReference(credencialProcessoVinculada.getGrupo().getId());

						if (this.contemUsuarioLogadoNoGrupo(grupo)) {

							return Boolean.TRUE;
						}
					}
				}

			}

		}

		return Boolean.FALSE;
	}

	@Override
	public void validarAcessoPesquisa(Long idProcessInstance) {

		Processo processo = (Processo) this.processoService.findByKeyValueUnique("idProcessInstance", idProcessInstance.toString());

		this.request.getSession().setAttribute("idProcesso", processo.getId());

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar se o usuario tem permissão para executar o processo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idProcesso
	 * 
	 * @return Boolean
	 */
	public Boolean permiteExecutar(Long idProcesso) {

		Processo processo = this.processoService.find(idProcesso);

		if (processo.getNivelAcesso().getNivelAcesso().getCodigo().equals(2L)) {

			return Boolean.TRUE;

		}

		return this.unidadeProcessoService.isProcessoNaUnidadeUsuarioLogado(idProcesso);

	}

}
