package br.com.centralit.api.service.impl;

import java.io.File;
import java.io.FileInputStream;
import java.security.Key;
import java.util.Calendar;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;
import org.springframework.web.multipart.MultipartFile;

import br.com.centralit.api.dao.DocumentoGedDao;
import br.com.centralit.api.model.AnexoGed;
import br.com.centralit.api.model.Assinatura;
import br.com.centralit.api.model.DocumentoGed;
import br.com.centralit.api.model.InteressadoDocumento;
import br.com.centralit.api.model.Processo;
import br.com.centralit.api.service.AnexoGedService;
import br.com.centralit.api.service.CredencialProcessoService;
import br.com.centralit.api.service.DocumentoGedService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.NivelAcessoTipoDocumentoService;
import br.com.centralit.api.service.NotificacaoService;
import br.com.centralit.api.service.PlanoClassificacaoService;
import br.com.centralit.api.service.ProcessoService;
import br.com.centralit.api.service.SigiloService;
import br.com.centralit.api.service.SolrService;
import br.com.centralit.api.service.TipoDocumentoService;
import br.com.centralit.api.service.UnidadeProcessoService;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Grupo;
import br.com.centralit.framework.model.Notificacao;
import br.com.centralit.framework.model.NotificacaoGrupo;
import br.com.centralit.framework.service.arquitetura.GedFileService;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 16/12/2015 - 14:47:36
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
@Service("documentoGedService")
public class DocumentoGedServiceImpl extends GenericServiceImpl<DocumentoGed, Long> implements DocumentoGedService {

	private static final String VALIDACAO_UNICIDADE_DOCUMENTO = "ECM.VALIDACAO.UNICIDADE_DOCUMENTO";

	private static final String VALIDAR_NIVEL_ACESSO = "ECM.VALIDACAO.VALIDAR_NIVEL_ACESSO";

	private static final String VALIDAR_OUTRAS_UNIDADES = "ECM.VALIDACAO.VALIDAR_ALTERACAO_NIVEL_ACESSO_OUTRAS_UNIDADES";

	/** Atributo documentoGedDao. */
	private DocumentoGedDao documentoGedDao;

	/** Atributo processoService. */
	@Autowired
	private ProcessoService processoService;

	/** Atributo nivelAcessoTipoDocumentoService. */
	@Autowired
	private NivelAcessoTipoDocumentoService nivelAcessoTipoDocumentoService;

	/** Atributo tipoDocumentoService. */
	@Autowired
	private TipoDocumentoService tipoDocumentoService;

	/** Atributo planoClassificacaoService. */
	@Autowired
	private PlanoClassificacaoService planoClassificacaoService;

	/** Atributo anexoGedService. */
	@Autowired
	private AnexoGedService anexoGedService;

	/** Atributo GedFileService. */
	@Autowired
	private GedFileService gedFileService;

	/** Atributo dominioService. */
	@Autowired
	private DominioService dominioService;

	/** Atributo sigiloService. */
	@Autowired
	private SigiloService sigiloService;

	/** Atributo unidadeProcessoService. */
	@Autowired
	private UnidadeProcessoService unidadeProcessoService;

	/** Atributo notificacaoService. */
	@Autowired
	private NotificacaoService notificacaoService;

	/** Atributo credencialProcessoService. */
	@Autowired
	private CredencialProcessoService credencialProcessoService;

	/** Atributo solrService. */
	@Autowired
	private SolrService solrService;

	private static final String ICON_FILE_SEM_FORMATO = "fa fa-file-o fa-2x";

	private static final String ICON_CODE = "fa fa-file-code-o fa-2x";

	private static final String ICON_PDF = "fa fa-file-pdf-o fa-2x";

	private static final String ICON_XLS = "fa fa-file-excel-o fa-2x";

	private static final String ICON_TXT = "fa fa-file-text-o fa-2x";

	private static final String ICON_DOCX = "fa fa-file-word-o fa-2x";

	private static final String ICON_DOC_ONLINE = "fa fa-file-text fa-2x";

	private static final String ICON_PICTURE = "glyphicon glyphicon-picture fa-2x";

	private static final String ICON_PLAY = "fa fa-file-audio-o fa-2x";

	/**
	 * quantidade de documentos que podem ser incluídos.
	 */
	@Value("${ecm.doc.quantidadeDocumentos}")
	public String quantidadeDocumentos;

	@Value("${ecm.doc.key.decrypted.document}")
	public String keyDocument;

	@Autowired
	public DocumentoGedServiceImpl( DocumentoGedDao documentoGedDao, @Qualifier("documentoGedValidator") Validator validator ) {

		this.dao = documentoGedDao;
		this.documentoGedDao = documentoGedDao;
		this.validator = validator;
	}

	@Override
	public DocumentoGed save(DocumentoGed entity) {

		this.validarEntidade(entity, this.validator);
		entity.setVersaoDocumento(1d);
		this.montarEntidade(entity);
		this.executarAcoesDocumentoOnline(entity);
		this.validarUnicidadeDocumento(entity);
		this.validarNivelAcessoDocumento(entity);
		this.atribuirPrazoPadraoDocumento(entity);

		entity = super.save(entity);

		if (entity.getFormaCriacao().getCodigo().equals(1L)) {

			entity.setConteudo(this.decrypted(entity.getConteudoCriptografado()));
		}

		this.saveSolr(entity);

		this.verificaTemporalidadeProcesso(entity);

		return entity;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por atualizar o plano de classificação
	 * 
	 * @author rogerio.costa
	 * 
	 * @param entity
	 */
	private void verificaTemporalidadeProcesso(DocumentoGed entity) {

		this.processoService.verificaTemporalidadeProcessoAtravesDoPlanoClassificacacao(entity.getPlanoClassificacao(), entity.getProcesso());

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por indexar documento no servidor solr
	 * 
	 * @author rogerio.costa
	 * 
	 * @param entity
	 */
	private void saveSolr(DocumentoGed entity) {

		try {
			// Verifica se o documento é sigiloso
			if (!UtilObjeto.isReferencia(entity.getSigilo())) {

				//this.solrService.addDocumento(entity);
			}

		} catch (Exception e) {

			e.printStackTrace();
		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por montar os objetos transiente
	 * 
	 * @author rogerio.costa
	 * 
	 * @param entity
	 */
	private void montarEntidade(DocumentoGed entity) {

		entity.setAnexo(null);

		if (!UtilColecao.isVazio(entity.getInteressados())) {
			for (InteressadoDocumento interessado : entity.getInteressados()) {

				interessado.setDocumentoGed(entity);
			}
		}

		entity.setProcesso(this.processoService.find(entity.getProcesso().getId()));
		entity.setTipoDocumento(this.tipoDocumentoService.getReference(entity.getTipoDocumento().getId()));
		entity.setPlanoClassificacao(this.planoClassificacaoService.getReference(entity.getPlanoClassificacao().getId()));
		entity.setNivelAcesso(this.nivelAcessoTipoDocumentoService.getReference(entity.getNivelAcesso().getId()));
		entity.setFormaCriacao(this.dominioService.findByChaveAndCodigo("tipoFormaCriacao", entity.getFormaCriacao().getCodigo()));

		if (UtilObjeto.isReferencia(entity.getSigilo()) && UtilObjeto.isReferencia(entity.getSigilo().getId())) {

			entity.setSigilo(this.sigiloService.getReference(entity.getSigilo().getId()));

			if (entity.isNew()) {
				entity.setStatus(this.dominioService.findByChaveAndCodigo("statusDocumentoGed", 0L));

			}
			this.atribuirPrazoPadraoDocumento(entity);
			this.enviarNotificacaoDocumentoSigiloso(entity);
		} else {

			entity.setSigilo(null);
		}

	}

	/**
	 * 
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">3</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por validar o nivel de acesso do documento
	 * 
	 * @author andre.silva
	 * 
	 * @param entity
	 *            História de usuário #3216_CA004
	 */
	private void validarNivelAcessoDocumento(DocumentoGed entity) {

		if (entity.getProcesso().getNivelAcesso().getNivelAcesso().getCodigo() != 2L) {

			if (entity.getProcesso().getNivelAcesso().getNivelAcesso().getCodigo() == 0L && entity.getNivelAcesso().getNivelAcesso().getCodigo() == 2L) {
				throw new BusinessException(DocumentoGedServiceImpl.VALIDAR_NIVEL_ACESSO, CodigoErro.REGRA_NEGOCIO.getValue());
			} else if (entity.getProcesso().getNivelAcesso().getNivelAcesso().getCodigo() == 1L) {
				if (entity.getNivelAcesso().getNivelAcesso().getCodigo() != 1L && !UtilObjeto.isReferencia(entity.getSigilo())) {
					throw new BusinessException(DocumentoGedServiceImpl.VALIDAR_NIVEL_ACESSO, CodigoErro.REGRA_NEGOCIO.getValue());
				}
			} else if (entity.getProcesso().getNivelAcesso().getNivelAcesso().getCodigo() == 2L) {
				if (entity.getNivelAcesso().getNivelAcesso().getCodigo() == 0L) {
					throw new BusinessException(DocumentoGedServiceImpl.VALIDAR_NIVEL_ACESSO, CodigoErro.REGRA_NEGOCIO.getValue());
				}
			}

			if (UtilObjeto.isReferencia(entity.getProcesso().getSigilo()) && UtilObjeto.isReferencia(entity.getSigilo())) {
				if (entity.getProcesso().getSigilo().getTipoSigilo().getCodigo() == 1L) {
					if (entity.getSigilo().getTipoSigilo().getCodigo() == 0L) {
						throw new BusinessException(DocumentoGedServiceImpl.VALIDAR_NIVEL_ACESSO, CodigoErro.REGRA_NEGOCIO.getValue());
					}
				} else if (entity.getProcesso().getSigilo().getTipoSigilo().getCodigo() == 2L) {
					if (!UtilObjeto.isReferencia(entity.getSigilo()) || entity.getSigilo().getTipoSigilo().getCodigo() != 2L) {
						throw new BusinessException(DocumentoGedServiceImpl.VALIDAR_NIVEL_ACESSO, CodigoErro.REGRA_NEGOCIO.getValue());
					}
				}
			}else{
				throw new BusinessException(DocumentoGedServiceImpl.VALIDAR_NIVEL_ACESSO, CodigoErro.REGRA_NEGOCIO.getValue());
			}
		}

	}

	/**
	 * 
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author andre.silva
	 * 
	 * @param entity
	 *            História de usuário #3216_CA005
	 */
	private void atribuirPrazoPadraoDocumento(DocumentoGed entity) {

		Calendar calendarPrazoPadrao = Calendar.getInstance();

		if (UtilObjeto.isReferencia(entity.getProcesso().getSigilo())) {

			// Reservado – prazo padrão de sigilo é de 5 anos a partir da data de criação, podendo ser prorrogado
			if (entity.getProcesso().getSigilo().getTipoSigilo().getCodigo() == 0L) {
				calendarPrazoPadrao.add(Calendar.YEAR, 5);

				// Secreto – prazo padrão de sigilo é de 15 anos a partir da data de criação, podendo ser prorrogado
			} else if (entity.getProcesso().getSigilo().getTipoSigilo().getCodigo() == 1L) {
				calendarPrazoPadrao.add(Calendar.YEAR, 15);

				// Ultrassecreto – prazo padrão de sigilo é de 25 anos a partir da data de criação, podendo ser prorrogado
			} else if (entity.getProcesso().getSigilo().getTipoSigilo().getCodigo() == 2L) {
				calendarPrazoPadrao.add(Calendar.YEAR, 25);

			}
		}
	}

	/**
	 * 
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por: Verificar se os documentos estão abertos em mais de uma unidade não poderão ter seu nível de acesso alterado para Sigiloso
	 * 
	 * @author andre.silva
	 * 
	 * @param entity
	 *            História de usuário #3216_CA007
	 */
	private void validarAbertosEmOutrasUnidades(DocumentoGed entity) {

		// verifica se o documento está passando a ser sigiloso
		if (UtilObjeto.isReferencia(entity.getSigilo()) && !this.documentoGedDao.isSigiloso(entity.getId())) {

			if (this.unidadeProcessoService.isProcessoEnviado(entity.getProcesso().getId())) {

				throw new BusinessException(DocumentoGedServiceImpl.VALIDAR_OUTRAS_UNIDADES, CodigoErro.REGRA_NEGOCIO.getValue());
			}
		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por executar ações para salvar o documento online
	 * 
	 * @author rogerio.costa
	 * 
	 */
	public void executarAcoesDocumentoOnline(DocumentoGed documentoGed) {

		// Verifica se é um documento online
		if (documentoGed.getFormaCriacao().getCodigo().equals(1L)) {

			if (documentoGed.isNew()) {

				this.gerarNumeroDocumentOnline(documentoGed);
			} else {

				this.substituirParametrosDinamicos(documentoGed);
			}

			documentoGed.setConteudoCriptografado(this.encrypted(documentoGed.getConteudo()));

		}

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por criptografar o texto através da chave
	 * 
	 * @author rogerio.costa
	 * 
	 * @param text
	 * @return
	 */
	public byte[] encrypted(String text) {

		try {

			byte[] encrypted = null;
			Key aesKey = new SecretKeySpec(this.keyDocument.getBytes(), "AES");
			Cipher cipher = Cipher.getInstance("AES");
			cipher.init(Cipher.ENCRYPT_MODE, aesKey);
			encrypted = cipher.doFinal(text.getBytes());

			return encrypted;

		} catch (Exception e) {
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por descriptografar texto
	 * 
	 * @author rogerio.costa
	 * 
	 * @param text
	 * @return
	 */
	public String decrypted(byte[] text) {

		try {

			Cipher cipher = Cipher.getInstance("AES");
			Key aesKey = new SecretKeySpec(this.keyDocument.getBytes(), "AES");

			cipher.init(Cipher.DECRYPT_MODE, aesKey);
			String decrypted = new String(cipher.doFinal(text));

			return decrypted;

		} catch (Exception e) {
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por gerar numero do documento
	 * 
	 * @author rogerio.costa
	 * 
	 * @param entity
	 * 
	 * @return numero/String
	 */
	private void gerarNumeroDocumentOnline(DocumentoGed entity) {

		entity.setIcon(DocumentoGedServiceImpl.ICON_DOC_ONLINE);

		Calendar calendarInicial = Calendar.getInstance();

		Calendar calendarFinal = Calendar.getInstance();

		calendarInicial.set(Calendar.YEAR, calendarInicial.get(Calendar.YEAR));
		calendarInicial.set(Calendar.MONTH, Calendar.JANUARY);
		calendarInicial.set(Calendar.DAY_OF_MONTH, 1);

		calendarFinal.set(Calendar.YEAR, calendarFinal.get(Calendar.YEAR));
		calendarFinal.set(Calendar.MONTH, Calendar.DECEMBER);
		calendarFinal.set(Calendar.DAY_OF_MONTH, 31);

		calendarFinal.set(calendarFinal.get(Calendar.YEAR), Calendar.DECEMBER, 31);
		calendarInicial.set(calendarInicial.get(Calendar.YEAR), Calendar.JANUARY, 1);

		Calendar calendar = Calendar.getInstance();
		StringBuilder sb = new StringBuilder();

		// Gera o ano atual
		calendar.set(calendar.get(Calendar.YEAR), 11, 31);

		Long quantidade = this.documentoGedDao.obterQuantidadePorTipoDocumentoEDataCriacao(entity.getTipoDocumento().getId(), calendarInicial, calendarFinal);

		// Verifica se não existe documento do cadastrado para o tipo documento selecionado
		if (UtilObjeto.isReferencia(quantidade)) {

			entity.setNumero(sb.append(quantidade + 1).append("/").append(UtilDate.getAnoAtual()).toString());
		} else {
			// Gera o primeiro numero de documento
			sb.append("1/").append(UtilDate.getAnoAtual());
			entity.setNumero(sb.toString());
		}

		this.substituirParametrosDinamicos(entity);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por substituir os parametros dinamicos do documento
	 * 
	 * @author rogerio.costa
	 * 
	 * @param entity
	 */
	private void substituirParametrosDinamicos(DocumentoGed entity) {

		entity.setConteudo(entity.getConteudo().replace("@numeroDocumento@", entity.getNumero()));
	}

	@Override
	public DocumentoGed merge(DocumentoGed entity) {

		this.validarEntidade(entity, this.validator);
		AnexoGed anexo = null;

		if (entity.getAnexo() != null) {
			anexo = this.anexoGedService.getReference(entity.getAnexo().getId());
		}

		this.validarAbertosEmOutrasUnidades(entity);

		DocumentoGed versaoAnterior = this.find(entity.getId());

		this.montarEntidade(entity);

		this.gerarVersao(entity, versaoAnterior);

		this.executarAcoesDocumentoOnline(entity);

		entity.setDataEdicao(Calendar.getInstance());

		entity.setId(null);

		entity = super.merge(entity);

		entity.setConteudo(this.decrypted(entity.getConteudoCriptografado()));

		this.saveSolr(entity);

		try {
			if (anexo != null) {
				File file = this.gedFileService.restoreFile(anexo);
				file.length();
				FileInputStream input = new FileInputStream(file);
				MultipartFile multipartFile = new MockMultipartFile("file", anexo.getDescricao(), "text/plain", IOUtils.toByteArray(input));

				this.anexoGedService.saveAnexo(multipartFile, entity.getId());
			}

		} catch (Exception e) {
			e.printStackTrace();

		}
		versaoAnterior.setVersaoPosterior(entity);

		return entity;

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author andre.luiz
	 * 
	 * @param entity
	 * @param versaoAnterior
	 */
	private void gerarVersao(DocumentoGed entity, DocumentoGed versaoAnterior) {

		entity.setVersaoAnterior(versaoAnterior);
		entity.setEditor(this.getUsuario());
		entity.setAutor(this.getUsuario());
		entity.setOrigem(versaoAnterior.getOrigem());

		if (entity != null && entity.getNovaVersao()) {
			entity.setVersaoDocumento(Math.floor(entity.getVersaoDocumento()) + 1);
		} else {
			entity.setVersaoDocumento(entity.getVersaoDocumento() + 0.1);

		}

		if (entity.getIdRaiz() == null) {
			if (versaoAnterior.getIdRaiz() == null) {
				entity.setIdRaiz(entity.getId());
			} else {
				entity.setIdRaiz(versaoAnterior.getIdRaiz());
			}
		}
	}

	@Override
	public DocumentoGed getReference(Long id) {

		DocumentoGed documentoGed = super.getReference(id);

		documentoGed.setAnexo(this.anexoGedService.obterPorIdDocumento(documentoGed.getId()));

		if (documentoGed.getFormaCriacao() != null && documentoGed.getFormaCriacao().getCodigo().equals(1L)) {

			// Verifica se o documento é do tipo online
			if (documentoGed.getFormaCriacao().getCodigo().equals(1L)) {

				documentoGed.setConteudo(this.decrypted(documentoGed.getConteudoCriptografado()));
			}

		}

		return documentoGed;
	}

	@Override
	public boolean removeById(Long id) {

		DocumentoGed documentoGed = this.getReference(id);

		documentoGed.setProcessoRemocao(documentoGed.getProcesso());

		documentoGed.setProcesso(null);

		return super.removeById(id);
	}

	/**
	 * 
	 */
	@Override
	public Collection<DocumentoGed> findByProcesso(Long id) {

		return this.documentoGedDao.findByProcesso(id);
	}

	@Override
	public Collection<DocumentoGed> findByProcessoSigilo(Long idProcesso) {

		return this.documentoGedDao.findByProcessoSigilo(idProcesso);
	}

	/**
	 * 
	 */
	@Override
	public Collection<DocumentoGed> findByIdProcessoUltimaVersao(Long id) {

		return this.documentoGedDao.findByIdProcessoUltimaVersao(id);
	}

	/**
	 * 
	 */
	@Override
	public Collection<DocumentoGed> findByIdDocumentoTodasVersoes(Long id) {

		return this.documentoGedDao.findByIdDocumentoTodasVersoes(id);
	}

	/**
	 * valida se já atingiu a quantidade de versões limite
	 */
	@Override
	public boolean validaQuantidadeVersoes(Long id) {

		List<DocumentoGed> listaDocumentos = (List<DocumentoGed>) this.documentoGedDao.findByIdDocumentoTodasVersoesAtivas(id);
		if (listaDocumentos == null || listaDocumentos.isEmpty()) {
			return false;
		} else if (listaDocumentos.size() == Long.valueOf(this.quantidadeDocumentos)) {
			return true;
		}
		return false;
	}

	/**
	 * remove o documento pelo id
	 */
	@Override
	public boolean removerDocumento(Long id) {

		List<DocumentoGed> listaDocumento = (List<DocumentoGed>) this.documentoGedDao.findByIdDocumentoTodasVersoesAtivas(id);
		DocumentoGed entity = listaDocumento.get(0);
		try {
			this.gedFileService.remove(entity.getAnexo());
		} catch (Exception e) {
			e.printStackTrace();
		}
		this.anexoGedService.remove(entity.getAnexo());

		return this.remove(entity);
	}

	/**
	 * 
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author andre.silva
	 * 
	 * @param processoOrigem
	 * @param anexarProcessoVH
	 */
	@Override
	public void anexarDocumento(Processo processoDestinoAnexo, Processo processoOrigem) {

		if (!UtilColecao.isVazio(processoOrigem.getDocumentos())) {

			for (DocumentoGed documentoGedOrigem : processoOrigem.getDocumentos()) {
				vincularAnexo(documentoGedOrigem, processoDestinoAnexo, processoOrigem);
			}
		}

	}

	/**
	 * 
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author andre.silva
	 * 
	 * @param listaDocumentos
	 */
	@Override
	public Collection<DocumentoGed> atualizarSigiloDocumentos(Collection<DocumentoGed> listaDocumentos) {

		Collection<DocumentoGed> documentosAlterados = new LinkedList<DocumentoGed>();

		if (!UtilColecao.isVazio(listaDocumentos)) {
			for (DocumentoGed documento : listaDocumentos) {
				DocumentoGed entity = (DocumentoGed) this.documentoGedDao.getReference(documento.getId());
				documento.setProcesso(entity.getProcesso());
				this.validarNivelAcessoDocumento(documento);
				entity.setSigiloOriginal(documento.getSigilo());
				entity.setNivelAcessoOriginal(documento.getNivelAcesso());
				entity.setSigilo(documento.getSigilo());
				entity.setNivelAcesso(documento.getNivelAcesso());
				entity.setStatus(this.dominioService.findByChaveAndCodigo("statusDocumentoGed", 1L));
				documentosAlterados.add(entity);
			}

		}

		return documentosAlterados;

	}

	/**
	 * 
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author andre.silva
	 * 
	 * @param documentoGedOrigem
	 * @param anexarProcessoVH
	 */
	private void vincularAnexo(DocumentoGed documentoGedOrigem, Processo processoDestinoAnexo, Processo processoOrigem) {

		/* Processo processoDestinoAnexo = this.processoService.getReference(anexarProcessoVH.getProcesso().getId()); */

		AnexoGed anexoDocumentoOrigem = null;

		DocumentoGed documentoGed = new DocumentoGed();

		documentoGed = documentoGedOrigem.clone();

		documentoGed.setProcesso(processoDestinoAnexo);
		documentoGed.setOrigem(processoOrigem);
		documentoGed.setAnexo(null);
		documentoGed.setId(null);
		documentoGed.setAssinaturas(null);

		if (!UtilColecao.isVazio(documentoGedOrigem.getInteressados())) {

			documentoGed.setInteressados(new LinkedList<InteressadoDocumento>());

			// Monta os dados do novo interessado para o novo documento.
			for (InteressadoDocumento interessadoDocumento : documentoGedOrigem.getInteressados()) {

				InteressadoDocumento interessadoDocumentoDestino = new InteressadoDocumento();
				interessadoDocumentoDestino.setInteressado(interessadoDocumento.getInteressado());
				interessadoDocumentoDestino.setDocumentoGed(documentoGed);
				documentoGed.getInteressados().add(interessadoDocumentoDestino);
			}

		}

		documentoGed = super.save(documentoGed);

		if (documentoGedOrigem.getFormaCriacao() != null && !documentoGedOrigem.getFormaCriacao().getCodigo().equals(1L)) {

			if (documentoGedOrigem.getAnexo() != null) {
				anexoDocumentoOrigem = this.anexoGedService.getReference(documentoGedOrigem.getAnexo().getId());
			}
		}

		try {
			if (anexoDocumentoOrigem != null) {
				File file = this.gedFileService.restoreFile(anexoDocumentoOrigem);
				file.length();
				FileInputStream input = new FileInputStream(file);
				MultipartFile multipartFile = new MockMultipartFile("file", anexoDocumentoOrigem.getDescricao(), "text/plain", IOUtils.toByteArray(input));

				this.anexoGedService.saveAnexo(multipartFile, documentoGed.getId());
			}

		} catch (Exception e) {
			e.printStackTrace();

		}

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por validar a unidade do documento
	 * 
	 * @author rogerio.costa
	 * 
	 * @param pessoaFisica
	 */
	public void validarUnicidadeDocumento(DocumentoGed documentoGed) {

		DocumentoGed documentoGedPersistido = this.documentoGedDao.obterPorProcessoTipoDocumentoENumero(documentoGed);

		if (UtilObjeto.isReferencia(documentoGedPersistido) && ( !UtilObjeto.isReferencia(documentoGed.getId()) || UtilObjeto.isReferencia(documentoGed.getId()) && !documentoGed.getId().equals(documentoGedPersistido.getId()) )) {

			throw new BusinessException(DocumentoGedServiceImpl.VALIDACAO_UNICIDADE_DOCUMENTO, CodigoErro.REGRA_NEGOCIO.getValue());
		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por salvar icon do documentoGed
	 * 
	 * @author rogerio.costa
	 * 
	 * @param documentoGed
	 * @param tipoAnexo
	 */
	@Override
	public void saveIconeDocumento(DocumentoGed documentoGed, Dominio tipoAnexo) {

		documentoGed.setIcon(this.getIcon(tipoAnexo));
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por gerar o icone
	 * 
	 * @author rogerio.costa
	 * 
	 * @param codigoTipoAnexo
	 * @return
	 */
	private String getIcon(Dominio tipoAnexo) {

		if (Dominio.TIPO_ANEXO_JPEG_CODIGO.equals(tipoAnexo.getCodigo()) || Dominio.TIPO_ANEXO_JPG_CODIGO.equals(tipoAnexo.getCodigo()) || Dominio.TIPO_ANEXO_PNG_CODIGO.equals(tipoAnexo.getCodigo())) {

			return DocumentoGedServiceImpl.ICON_PICTURE;

		} else if (tipoAnexo.getCodigo().equals(10L)) {

			return DocumentoGedServiceImpl.ICON_DOCX;

		} else if (Dominio.TIPO_ANEXO_TXT_CODIGO.equals(tipoAnexo.getCodigo())) {

			return DocumentoGedServiceImpl.ICON_TXT;

		} else if (Dominio.TIPO_ANEXO_XLS_CODIGO.equals(tipoAnexo.getCodigo()) || tipoAnexo.getCodigo().equals(9L)) {

			return DocumentoGedServiceImpl.ICON_XLS;

		} else if (Dominio.TIPO_ANEXO_PDF_CODIGO.equals(tipoAnexo.getCodigo())) {

			return DocumentoGedServiceImpl.ICON_PDF;

		} else if (Dominio.TIPO_ANEXO_XML_CODIGO.equals(tipoAnexo.getCodigo())) {

			return DocumentoGedServiceImpl.ICON_CODE;

		} else if (Dominio.TIPO_ANEXO_MP3_CODIGO.equals(tipoAnexo.getCodigo()) || Dominio.TIPO_ANEXO_WMA_CODIGO.equals(tipoAnexo.getCodigo()) || Dominio.TIPO_ANEXO_WAV_CODIGO.equals(tipoAnexo.getCodigo()) || Dominio.TIPO_ANEXO_AAC_CODIGO.equals(tipoAnexo.getCodigo()) || Dominio.TIPO_ANEXO_OGG_CODIGO.equals(tipoAnexo.getCodigo()) || Dominio.TIPO_ANEXO_M4A_CODIGO.equals(tipoAnexo.getCodigo()) || Dominio.TIPO_ANEXO_FLAC_CODIGO.equals(tipoAnexo.getCodigo())) {
			return DocumentoGedServiceImpl.ICON_PLAY;
		}

		return DocumentoGedServiceImpl.ICON_FILE_SEM_FORMATO;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por enviar notificação para os responsável sobre a criação do documento sigiloso
	 * 
	 * @author rogerio.costa
	 * 
	 * @param DocumentoGed
	 */
	private void enviarNotificacaoDocumentoSigiloso(DocumentoGed documentoGed) {

		StringBuilder sb = new StringBuilder();

		sb.append("O documento ").append(documentoGed.getNumero()).append(" foi criado como ").append(documentoGed.getSigilo().getTipoSigilo().getDescricao());

		Dominio tipoNotificacao = this.dominioService.findByChaveAndCodigo("tipoNotificacao", 4L);

		Dominio tipoPrioridade = this.dominioService.findByChaveAndCodigo("tipoPrioridade", 2L);

		Notificacao notificacao = new Notificacao("Documento sigiloso", sb.toString(), tipoNotificacao, tipoPrioridade, null, null, null);

		Collection<Grupo> grupos = this.credencialProcessoService.findGrupoPorSigilo(documentoGed.getSigilo());

		if (!UtilColecao.isVazio(grupos)) {

			notificacao.setNotificacaoGrupos(new LinkedList<NotificacaoGrupo>());

			for (Grupo grupo : grupos) {

				NotificacaoGrupo notificacaoGrupo = new NotificacaoGrupo(grupo, notificacao);

				notificacao.getNotificacaoGrupos().add(notificacaoGrupo);
			}

		}

		this.notificacaoService.save(notificacao);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter através do id do PlanoClassificacao
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idPlanoClassificacao
	 * 
	 * @return Collection<Processo>
	 */
	public Collection<DocumentoGed> findPorPlanoClassificacao(Long idPlanoClassificacao) {

		return this.documentoGedDao.findPorPlanoClassificacao(idPlanoClassificacao);
	}
	
	/**
	 * Método responsável por montar as três primeiras assinaturas por documento para apresentação no tooltip da tela de execução de tarefa
	 * @author rogerio.cassimiro
	 * @param documentos
	 */
	@Override
	public void montarTresAssinantesPorDocumento(Collection<DocumentoGed> documentos) {

		if(!UtilColecao.isVazio(documentos)){
			for (DocumentoGed documentoGed : documentos) {
				if(!UtilColecao.isVazio(documentoGed.getAssinaturas())){
					int maxAssinatura = 0;
					StringBuilder tresAssinaturas = new StringBuilder("Assinador por: ");
					for (Assinatura assinatura : documentoGed.getAssinaturas()) {
						if(++maxAssinatura < 4)
							tresAssinaturas.append(UtilObjeto.isReferencia(assinatura.getPessoa()) ? assinatura.getPessoa().getNome() : "").append(" / ").append(assinatura.getFuncao().getNome()).append(". ");
						else {
							tresAssinaturas.append("...");	
							break;
						}
					}
					documentoGed.setTresAssinaturas(tresAssinaturas.toString());
				}
			}
		}
	}

}
