package br.com.centralit.api.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.EstruturaOrganizacionalECMDao;
import br.com.centralit.api.model.ConfiguracaoUsuarioUnidade;
import br.com.centralit.api.model.EstruturaOrganizacional;
import br.com.centralit.api.model.EstruturaOrganizacionalECM;
import br.com.centralit.api.service.ConfiguracaoUsuarioUnidadeService;
import br.com.centralit.api.service.EstruturaOrganizacionalECMService;
import br.com.centralit.api.service.GrupoService;
import br.com.centralit.api.service.UnidadeService;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.GrupoUnidade;
import br.com.centralit.framework.model.Unidade;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 14/12/2015 - 09:30:11
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
@Service("estruturaOrganizacionalECMService")
public class EstruturaOrganizacionalECMServiceImpl extends GenericServiceImpl<EstruturaOrganizacionalECM, Long> implements EstruturaOrganizacionalECMService {

	private static final String VALIDACAO_GRUPO_PADRAO_EM_OUTRA_UNIDADE = "ECM.VALIDACAO.GRUPO_PADRAO_EM_OUTRA_UNIDADE";

	/** Atributo estruturaOrganizacionalECMDao. */
	private EstruturaOrganizacionalECMDao estruturaOrganizacionalECMDao;

	/** Atributo configuracaoUsuarioUnidadeService. */
	@Autowired
	private ConfiguracaoUsuarioUnidadeService configuracaoUsuarioUnidadeService;

	/** Atributo grupoService. */
	@Autowired
	private GrupoService grupoService;

	/** Atributo unidadeService. */
	@Autowired
	private UnidadeService unidadeService;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * 
	 * @param estruturaOrganizacionalECMDao
	 */
	@Autowired
	public EstruturaOrganizacionalECMServiceImpl( EstruturaOrganizacionalECMDao estruturaOrganizacionalECMDao, @Qualifier("estruturaOrganizacionalECMValidator") Validator validator ) {

		this.dao = estruturaOrganizacionalECMDao;

		this.estruturaOrganizacionalECMDao = estruturaOrganizacionalECMDao;

		this.validator = validator;
	}

	@Override
	public EstruturaOrganizacionalECM save(EstruturaOrganizacionalECM entity) {

		this.validarEntidade(entity, this.validator);

		EstruturaOrganizacional estruturaOrganizacional = entity.getEstruturaOrganizacional();

		EstruturaOrganizacionalECM entityOld = this.estruturaOrganizacionalECMDao.obterPorIdEstruturaOrganizacional(entity.getEstruturaOrganizacional().getId());

		if (null != entityOld) {
			entityOld.setCodigoSiorg(entity.getCodigoSiorg());
			entity = entityOld;
		}

		entity.setNome(estruturaOrganizacional.getNome());
		entity.setSigla(estruturaOrganizacional.getSigla());

		this.resolveTransiente(entity);

		return super.save(entity);
	}

	@Override
	public EstruturaOrganizacionalECM merge(EstruturaOrganizacionalECM entity) {

		this.validarEntidade(entity, this.validator);

		this.configuracaoUsuarioUnidadeService.verificarRemocaoConfiguracaoUsuarioUnidade(entity);

		this.resolveTransiente(entity);

		EstruturaOrganizacional estruturaOrganizacional = entity.getEstruturaOrganizacional();

		entity.setNome(estruturaOrganizacional.getNome());
		entity.setSigla(estruturaOrganizacional.getSigla());

		return super.merge(entity);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author rogerio.costa
	 * 
	 * @param entity
	 */
	private void resolveTransiente(EstruturaOrganizacionalECM entity) {

		if (!UtilColecao.isVazio(entity.getConfiguracaoUsuariosUnidade())) {

			for (ConfiguracaoUsuarioUnidade configuracaoUsuarioUnidade : entity.getConfiguracaoUsuariosUnidade()) {

				configuracaoUsuarioUnidade.setEstruturaOrganizacionalECM(entity);

			}

		}

		if (!UtilColecao.isVazio(entity.getGruposUnidade())) {

			for (GrupoUnidade grupoUnidade : entity.getGruposUnidade()) {

				grupoUnidade.setUnidade(entity);
			}

			if (UtilObjeto.isReferencia(entity.getGrupo()) && UtilObjeto.isReferencia(entity.getGrupo().getId())) {

				Unidade unidadeUtilizandoGrupoPadrao = this.unidadeService.obterPorGrupoPadrao(entity.getGrupo().getId());

				if (UtilObjeto.isReferencia(unidadeUtilizandoGrupoPadrao) && !entity.isNew() && unidadeUtilizandoGrupoPadrao.getId().longValue() != entity.getId().longValue()) {

					throw new BusinessException(VALIDACAO_GRUPO_PADRAO_EM_OUTRA_UNIDADE, CodigoErro.REGRA_NEGOCIO.getValue(), unidadeUtilizandoGrupoPadrao.getNome());
				}

				entity.setGrupo(this.grupoService.getReference(entity.getGrupo().getId()));

			} else {

				entity.setGrupo(new ArrayList<GrupoUnidade>(entity.getGruposUnidade()).get(0).getGrupo());
			}
		}

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author rogerio.costa
	 * 
	 * @param nome
	 * @return
	 */
	@Override
	public List<EstruturaOrganizacionalECM> findPorNomeEstruturaOrganizacional(String nome) {

		return this.estruturaOrganizacionalECMDao.findPorNomeEstruturaOrganizacional(nome, this.getUsuario().getOrganizacao().getId());
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter estrutura ecm por id estrutura generica
	 * 
	 * @author halan.lima
	 * 
	 * @param idEstrutura
	 * @return
	 */
	@Override
	public EstruturaOrganizacionalECM obterPorIdEstruturaOrganizacional(Long idEstrutura) {

		return this.estruturaOrganizacionalECMDao.obterPorIdEstruturaOrganizacional(idEstrutura);
	}

}
