package br.com.centralit.api.service.impl;

import java.util.Collection;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.HipoteseLegalDao;
import br.com.centralit.api.model.HipoteseLegal;
import br.com.centralit.api.service.DocumentoGedService;
import br.com.centralit.api.service.HipoteseLegalService;
import br.com.centralit.api.service.ProcessoService;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;

@Service("hipoteseLegalService")
public class HipoteseLegalServiceImpl extends GenericServiceImpl<HipoteseLegal, Long> implements HipoteseLegalService {

	private HipoteseLegalDao hipoteseLegalDao;

	/** Atributo processoService. */
	@Autowired
	private ProcessoService processoService;

	/** Atributo documentoGedService. */
	@Autowired
	private DocumentoGedService documentoGedService;

	/** Atributo VALIDACAO_REMOCAO_COM_VINCULO. */
	private static final String VALIDACAO_REMOCAO_COM_VINCULO = "ECM.VALIDACAO.REMOCAO_COM_VINCULO";

	@Autowired
	public HipoteseLegalServiceImpl( HipoteseLegalDao hipoteseLegalDao, @Qualifier("hipoteseLegalValidator") Validator validator ) {

		this.dao = hipoteseLegalDao;
		this.hipoteseLegalDao = hipoteseLegalDao;
		this.validator = validator;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar vinculo com outros cadastros
	 * 
	 * @author rogerio.costa
	 * 
	 * @param id
	 */
	public boolean contemVinculo(Long id) {

		Boolean contemVinculoDocumento = this.documentoGedService.existeVinculo("planoClassificacao.id", id);

		Boolean contemVinculoProcesso = this.processoService.existeVinculo("assunto.id", id);

		// Verifica se contem vinculo com documento ou processo, caso contem lancha uma bussinessException.
		return contemVinculoDocumento || contemVinculoProcesso;

	}

	@Override
	public boolean removeById(Long id) {

		if (this.contemVinculo(id)) {

			throw new BusinessException(VALIDACAO_REMOCAO_COM_VINCULO, CodigoErro.REGRA_NEGOCIO.getValue());
		}

		return super.removeById(id);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por listar através do nivelAcesso
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idNivelAcesso
	 * 
	 * @return Collection<HipoteseLegal>
	 */
	public Collection<HipoteseLegal> findPorNivelAcesso(Long idNivelAcesso) {

		return this.hipoteseLegalDao.findPorNivelAcesso(idNivelAcesso);
	}

}
