package br.com.centralit.api.service.impl;

import java.util.Collection;
import java.util.LinkedList;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.PlanoClassificacaoDao;
import br.com.centralit.api.model.PlanoClassificacao;
import br.com.centralit.api.service.DocumentoGedService;
import br.com.centralit.api.service.PlanoClassificacaoService;
import br.com.centralit.api.service.ProcessoService;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 21/10/2015 - 16:05:34
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
@Service("planoClassificacaoService")
public class PlanoClassificacaoServiceImpl extends GenericServiceImpl<PlanoClassificacao, Long> implements PlanoClassificacaoService {

	/** Atributo planoClassificacaoDao. */
	private PlanoClassificacaoDao planoClassificacaoDao;

	/** Atributo documentoGedService. */
	@Autowired
	private DocumentoGedService documentoGedService;

	/** Atributo processoService. */
	@Autowired
	private ProcessoService processoService;

	/** Constante VALIDACAO_REMOCAO_PLANO_CLASSIFICACAO. */
	private static final String VALIDACAO_REMOCAO_PLANO_CLASSIFICACAO = "ECM.VALIDACAO.REMOCAO_PLANO_CLASSIFICACA_VINCULO";

	/** Atributo MAXIMO_NIVEL_PERMITIDO. */
	public static final String MAXIMO_NIVEL_PERMITIDO = "ECM.VALIDACAO.PLANO_CLASSIFICACAO_ULTRAPASSA_NIVEL_PERMITIDO";

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * 
	 * @param planoClassificacaoDao
	 * @param validator
	 */
	@Autowired
	public PlanoClassificacaoServiceImpl( PlanoClassificacaoDao planoClassificacaoDao, @Qualifier("planoClassificacaoValidator") Validator validator ) {

		this.dao = planoClassificacaoDao;
		this.planoClassificacaoDao = planoClassificacaoDao;
		this.validator = validator;
	}

	@Override
	public PlanoClassificacao merge(PlanoClassificacao entity) {

		if (UtilObjeto.isReferencia(entity.getDataBloqueio())) {

			this.bloquearChildrens(entity);
		}

		return super.merge(entity);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por bloquar os filhos do plano classificação selecionado
	 * 
	 * @author rogerio.costa
	 * 
	 */
	public void bloquearChildrens(PlanoClassificacao planoClassificacao) {

		Collection<PlanoClassificacao> listChildrens = this.planoClassificacaoDao.findChildrens(planoClassificacao.getId());

		if (!UtilColecao.isVazio(listChildrens)) {

			for (PlanoClassificacao planoClassificacaoFilho : listChildrens) {

				planoClassificacaoFilho.setDataBloqueio(planoClassificacao.getDataBloqueio());

				this.bloquearChildrens(planoClassificacaoFilho);

			}

		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por gerar codigo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param planoClassificacao
	 * @param idOrganizacao
	 */
	public void gerarCodigo(PlanoClassificacao planoClassificacao, Long idOrganizacao) {

		// Verifica se o nível do plano de classificação é uma classe
		if (!UtilObjeto.isReferencia(planoClassificacao.getPlanoClassificacaoParent())) {

			this.gerarCodigoClasse(planoClassificacao, idOrganizacao);

		} else {
			// Obtem o parent do plano que está sendo cadastrado
			PlanoClassificacao parent = this.getReference(planoClassificacao.getPlanoClassificacaoParent().getId());

			if (!UtilObjeto.isReferencia(parent.getPlanoClassificacaoParent())) {
				// SubClasse
				this.gerarCodigoSubClasse(planoClassificacao);

			} else if (!UtilObjeto.isReferencia(parent.getPlanoClassificacaoParent().getPlanoClassificacaoParent())) {
				// Grupo
				this.gerarCodigoGrupo(planoClassificacao);

			} else if (!UtilObjeto.isReferencia(parent.getPlanoClassificacaoParent().getPlanoClassificacaoParent().getPlanoClassificacaoParent())) {
				// SubGrupo
				this.gerarCodigoSubGrupo(planoClassificacao);
			} else {
				// demais grupos
				this.gerarCodigoSubGrupoDistinto(planoClassificacao);
			}

		}

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por gerar codigo do nível filhos do subGrupo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param planoClassificacao
	 */
	private void gerarCodigoSubGrupoDistinto(PlanoClassificacao planoClassificacao) {

		Integer qtdChildrens = this.planoClassificacaoDao.getCountChildrens(planoClassificacao.getPlanoClassificacaoParent().getId());

		StringBuilder sb = new StringBuilder();

		sb.append(planoClassificacao.getPlanoClassificacaoParent().getCodigo());

		// Verifica se é o primeiro Subgrupo
		if (qtdChildrens.equals(0)) {

			sb.append("1");

			planoClassificacao.setCodigo(sb.toString());

		} else {

			sb.append(qtdChildrens + 1);

			planoClassificacao.setCodigo(sb.toString());
		}

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por gerar codigo do nível subGrupo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param planoClassificacao
	 */
	private void gerarCodigoSubGrupo(PlanoClassificacao planoClassificacao) {

		Integer qtdChildrens = this.planoClassificacaoDao.getCountChildrens(planoClassificacao.getPlanoClassificacaoParent().getId());

		StringBuilder sb = new StringBuilder();

		sb.append(planoClassificacao.getPlanoClassificacaoParent().getCodigo());

		// Verifica se é o primeiro Subgrupo
		if (qtdChildrens.equals(0)) {

			sb.append(".1");

			planoClassificacao.setCodigo(sb.toString());

		} else {

			sb.append(".").append(qtdChildrens + 1);

			planoClassificacao.setCodigo(sb.toString());
		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por gerar codigo do nível grupo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param planoClassificacao
	 */
	private void gerarCodigoGrupo(PlanoClassificacao planoClassificacao) {

		Integer qtdChildrens = this.planoClassificacaoDao.getCountChildrens(planoClassificacao.getPlanoClassificacaoParent().getId());

		StringBuilder sb = new StringBuilder();

		sb.append(planoClassificacao.getPlanoClassificacaoParent().getCodigo().substring(0, 2));

		// Verifica se é o primeiro grupo da determinada subClasse
		if (qtdChildrens.equals(0)) {

			sb.append("2");

			planoClassificacao.setCodigo(sb.toString());

		} else {

			// Verifica se contem 8 planoClassificacao no nível de grupo
			if (qtdChildrens.equals(8)) {

				throw new BusinessException(PlanoClassificacaoServiceImpl.MAXIMO_NIVEL_PERMITIDO, CodigoErro.REGRA_NEGOCIO.getValue());
			}

			sb.append(qtdChildrens + 2);

			planoClassificacao.setCodigo(sb.toString());
		}

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por gerar codigo do nível subClasse
	 * 
	 * @author rogerio.costa
	 * 
	 * @param planoClassificacao
	 * @param idOrganizacao
	 */
	private void gerarCodigoSubClasse(PlanoClassificacao planoClassificacao) {

		Integer qtdChildrens = this.planoClassificacaoDao.getCountChildrens(planoClassificacao.getPlanoClassificacaoParent().getId());

		StringBuilder sb = new StringBuilder();

		sb.append(planoClassificacao.getPlanoClassificacaoParent().getCodigo().charAt(0));

		// Verifica se é a primeira subClasse de determinada classe
		if (qtdChildrens.equals(0)) {

			sb.append("10");

			planoClassificacao.setCodigo(sb.toString());

		} else {

			// Verifica se contem 9 planoClassificacao no nível de subClasse
			if (qtdChildrens == 9) {

				throw new BusinessException(PlanoClassificacaoServiceImpl.MAXIMO_NIVEL_PERMITIDO, CodigoErro.REGRA_NEGOCIO.getValue());
			}

			sb.append(qtdChildrens + 1).append("0");

			planoClassificacao.setCodigo(sb.toString());
		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por gerar o código do nível classe
	 * 
	 * @author rogerio.costa
	 * 
	 * @param planoClassificacao
	 * @param idOrganizacao
	 */
	private void gerarCodigoClasse(PlanoClassificacao planoClassificacao, Long idOrganizacao) {

		Collection<PlanoClassificacao> listaParents = this.planoClassificacaoDao.findParent(idOrganizacao);

		StringBuilder sb = new StringBuilder();

		if (UtilColecao.isVazio(listaParents)) {

			sb.append("000");

			planoClassificacao.setCodigo(sb.toString());

		} else {

			// Verifica se contem 10 planoClassificacao no nível de classe
			if (listaParents.size() == 10) {

				throw new BusinessException(PlanoClassificacaoServiceImpl.MAXIMO_NIVEL_PERMITIDO, CodigoErro.REGRA_NEGOCIO.getValue());
			}

			sb.append(listaParents.size()).append("00");

			planoClassificacao.setCodigo(sb.toString());
		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por listar a entidade<code>PlanoClassificacao</code> que não contem parent, ou sejá o primeiro nó da arvore
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idOrganizacao
	 * 
	 * @return Collection<PlanoClassificacao>
	 */
	@Override
	public Collection<PlanoClassificacao> findParent(Long idOrganizacao) {

		Collection<PlanoClassificacao> listaPlanoClassificacao = this.planoClassificacaoDao.findParent(idOrganizacao);

		for (PlanoClassificacao planoClassificacao : listaPlanoClassificacao) {

			planoClassificacao.setPlanoClassificacaoFilhas(this.planoClassificacaoDao.findChildrens(planoClassificacao.getId()));
		}

		return listaPlanoClassificacao;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por buscar PlanoClassificacao filhas através do id do parent
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idPlanoClassificacao
	 * 
	 * @return Collection<PlanoClassificacao>
	 */
	@Override
	public Collection<PlanoClassificacao> findChildrens(Long idPlanoClassificacao) {

		Collection<PlanoClassificacao> listaPlanoClassificacao = this.planoClassificacaoDao.findChildrens(idPlanoClassificacao);

		for (PlanoClassificacao planoClassificacao : listaPlanoClassificacao) {

			planoClassificacao.setPlanoClassificacaoFilhas(this.planoClassificacaoDao.findChildrens(planoClassificacao.getId()));
		}

		return listaPlanoClassificacao;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por listar recursivo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param nome
	 * @param idOrganizacao
	 * @return Collection<PlanoClassificacao>
	 */
	@Override
	public Collection<PlanoClassificacao> findTree(String nome) {

		Usuario usuarioLogado = ( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() );

		Collection<PlanoClassificacao> planoClassificacaoes = new LinkedList<PlanoClassificacao>();

		Collection<PlanoClassificacao> resultBusca = this.planoClassificacaoDao.findTree(nome, usuarioLogado.getOrganizacao().getId());

		for (PlanoClassificacao planoClassificacao : resultBusca) {

			if (!UtilObjeto.isReferencia(planoClassificacao.getPlanoClassificacaoParent())) {

				PlanoClassificacao planoCLassificacao = planoClassificacao.clone();

				this.montarFilhosRecursivo(planoCLassificacao, resultBusca);

				planoClassificacaoes.add(planoCLassificacao);

			}

		}

		return planoClassificacaoes;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por montar PlanoClassificacao filhas recursivo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param planoClassificacao
	 * 
	 * @param resultBusca
	 */
	private void montarFilhosRecursivo(PlanoClassificacao planoClassificacao, Collection<PlanoClassificacao> resultBusca) {

		for (PlanoClassificacao classificacao : resultBusca) {

			if (UtilObjeto.isReferencia(classificacao.getPlanoClassificacaoParent()) && classificacao.getPlanoClassificacaoParent().getId().equals(planoClassificacao.getId())) {

				PlanoClassificacao temp1 = classificacao.clone();

				this.montarFilhosRecursivo(temp1, resultBusca);

				planoClassificacao.getPlanoClassificacaoFilhas().add(temp1);
			}
		}
	}

	@Override
	public boolean removeById(Long id) {

		Boolean contemVinculoDocumento = this.documentoGedService.existeVinculo("planoClassificacao.id", id);

		Boolean contemVinculoProcesso = this.processoService.existeVinculo("assunto.id", id);

		// Verifica se contem vinculo com documento ou processo, caso contem lancha uma bussinessException.
		if (contemVinculoDocumento || contemVinculoProcesso) {

			throw new BusinessException(PlanoClassificacaoServiceImpl.VALIDACAO_REMOCAO_PLANO_CLASSIFICACAO, CodigoErro.REGRA_NEGOCIO.getValue());

		}

		return super.removeById(id);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por listar ultimo nivel ou que possua a opção "permitirUsarComoClassificacao"
	 * 
	 * @author rogerio.costa
	 * 
	 * @param codigoNome
	 * 
	 * @return Collection<PlanoClassificacao>
	 */
	@Override
	public Collection<PlanoClassificacao> findAssuntoAutoComplete(String codigoNome) {

		Usuario usuarioLogado = ( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() );

		return this.planoClassificacaoDao.findAssuntoAutoComplete(codigoNome, usuarioLogado.getOrganizacao().getId());
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por atualizar a temporalidade e destinação dos processos e documentos
	 * 
	 * @author rogerio.costa
	 * 
	 * @param planoClassificacao
	 */
	public void atualizarTemporalidadeDestinacao() {

		Collection<PlanoClassificacao> listaPlanoClassificacao = this.planoClassificacaoDao.findPorDataEdicaoAtual();

		for (PlanoClassificacao planoClassificacao : listaPlanoClassificacao) {

			this.processoService.atualizarTemporalidadeDestinacao(planoClassificacao);
		}

	}

}
