package br.com.centralit.api.service.impl;

import java.util.Collection;
import java.util.LinkedList;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.ProcessoRelacionadoDao;
import br.com.centralit.api.model.Processo;
import br.com.centralit.api.model.ProcessoRelacionado;
import br.com.centralit.api.service.ProcessoRelacionadoService;
import br.com.centralit.api.service.ProcessoService;
import br.com.centralit.api.viewHelper.ProcessoRelacionadoVH;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;

@Service("processoRelacionadoService")
public class ProcessoRelacionadoServiceImpl extends GenericServiceImpl<ProcessoRelacionado, Long> implements ProcessoRelacionadoService {

	private ProcessoRelacionadoDao processoRelacionadoDao;

	/** Atributo processoService. */
	@Autowired
	private ProcessoService processoService;

	@Autowired
	public ProcessoRelacionadoServiceImpl( ProcessoRelacionadoDao processoRelacionadoDao, @Qualifier("processoRelacionadoValidator") Validator validator ) {

		this.dao = processoRelacionadoDao;
		this.processoRelacionadoDao = processoRelacionadoDao;
		this.validator = validator;
	}

	/**
	 *
	 */
	@Override
	public ProcessoRelacionadoVH salvarProcessoRelacionado(ProcessoRelacionadoVH processoRelacionadoVH) {

		Processo processoPai = this.processoService.getReference(processoRelacionadoVH.getProcesso().getId());

		if (!UtilColecao.isVazio(processoRelacionadoVH.getListaProcessoRelacionado())) {

			for (ProcessoRelacionado processoRelacionado : processoRelacionadoVH.getListaProcessoRelacionado()) {

				Processo processoFilho = this.processoService.getReference(processoRelacionado.getProcessoRelacionado().getId());

				// Vincula o processo filho ao pai
				this.vincularProcesso(processoPai, processoFilho);

				// Realiza o vinculo inverso,  o pai vira filho e o filho vira pai.
				this.vincularProcesso(processoFilho, processoPai);

			}
		}

		return processoRelacionadoVH;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por criar o vinculo do processo com processoRelacionado
	 * 
	 * @author maycon.silva
	 * 
	 * @param processoPrincipal - Processo receberar a lista de relação de processos 
	 * @param processo - Processo que será vinculado ao processo pai
	 */
	private void vincularProcesso(Processo processoPai, Processo processoFilho) {

		ProcessoRelacionado processoRelacionado = new ProcessoRelacionado();

		processoRelacionado.setProcessoRelacionado(processoFilho);
		
		processoRelacionado.setProcesso(processoPai);

		processoRelacionado = this.save(processoRelacionado);

		if (!UtilColecao.isVazio(processoPai.getRelacionados())) {
			//Adiciona a relação do filho ao pai
			processoPai.getRelacionados().add(processoRelacionado);

		} else {
			//Cria lista de processo filho no pai
			processoPai.setRelacionados(new LinkedList<ProcessoRelacionado>());
			//Adiciona a relação do filho ao pai
			processoPai.getRelacionados().add(processoRelacionado);
		}
	}


	/**
	 * 
	 *<b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por remover lista de processos filho vinculados a um processo Pai
	 * 
	 * @author maycon.silva
	 *
	 */
	@Override
	public boolean removerProcessoRelacionado(ProcessoRelacionadoVH processoRelacionadoVH) {

		Processo processoPai = this.processoService.getReference(processoRelacionadoVH.getProcesso().getId());

		if (!UtilColecao.isVazio(processoRelacionadoVH.getListRemoverRelacinamento())) {

			for (ProcessoRelacionado processoRelacionado : processoRelacionadoVH.getListRemoverRelacinamento()) {
				//Remove vinculo do processoRelacionado(Processo Filho) do pai
				this.removerVinculoProcesoRelacionado(processoRelacionado, processoPai);
				
				//Consulta o processo filho e transforma ele em pai para fazer a remoção inversa.
				Processo processoPaiInverso = this.processoService.getReference(processoRelacionado.getProcessoRelacionado().getId());
				
				//Remove o vinculo inverso o Processo pai vira filho e o fiho vira Pai.
				this.removerVinculoProcesoRelacionado(this.processoRelacionadoDao.processoRelacionadoByIdProcesso(processoPai.getId(), processoRelacionado.getProcessoRelacionado().getId()) ,processoPaiInverso); 
				
			}
		}

		return true;
	}

	public void removerVinculoProcesoRelacionado(ProcessoRelacionado processoRelacionado, Processo processoPai) {

		processoPai.getRelacionados().remove(processoRelacionado);
		
		ProcessoRelacionado processoRelacionadoRemocao = this.getReference(processoRelacionado.getId());
		
		processoRelacionadoRemocao.setProcessoRemocao(processoRelacionado.getProcessoRelacionado());
		processoRelacionadoRemocao.setProcessoRelacionado(null);
		
	}

	@Override
	public Collection<ProcessoRelacionado> findByIdProcesso(Long idProcesso) {

		return this.processoRelacionadoDao.findByIdProcesso(idProcesso);

	}

}
