package br.com.centralit.api.service.impl;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.StringUtils;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.ProcessoDao;
import br.com.centralit.api.model.DocumentoGed;
import br.com.centralit.api.model.EstruturaOrganizacionalECM;
import br.com.centralit.api.model.InteressadoProcesso;
import br.com.centralit.api.model.PlanoClassificacao;
import br.com.centralit.api.model.Processo;
import br.com.centralit.api.model.Sigilo;
import br.com.centralit.api.model.SigiloPrivilegio;
import br.com.centralit.api.model.Temporalidade;
import br.com.centralit.api.model.UnidadeProcesso;
import br.com.centralit.api.service.CredencialProcessoService;
import br.com.centralit.api.service.DocumentoGedService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.EstruturaOrganizacionalECMService;
import br.com.centralit.api.service.GrupoService;
import br.com.centralit.api.service.HistoricoAlteracaoProcessoService;
import br.com.centralit.api.service.HistoricoAtribuicaoProcessoService;
import br.com.centralit.api.service.NotificacaoService;
import br.com.centralit.api.service.PlanoClassificacaoService;
import br.com.centralit.api.service.ProcessoService;
import br.com.centralit.api.service.SigiloService;
import br.com.centralit.api.service.SolrService;
import br.com.centralit.api.service.TemporalidadeService;
import br.com.centralit.api.service.UnidadeProcessoService;
import br.com.centralit.api.service.UnidadeService;
import br.com.centralit.api.viewHelper.ProcessoVH;
import br.com.centralit.framework.esi.enumerated.VariableTypeEnum;
import br.com.centralit.framework.esi.environment.EnvironmentOutput;
import br.com.centralit.framework.esi.environment.EnvironmentVariable;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Grupo;
import br.com.centralit.framework.model.Notificacao;
import br.com.centralit.framework.model.NotificacaoGrupo;
import br.com.centralit.framework.model.Unidade;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;

import com.googlecode.genericdao.search.ISearch;
import com.googlecode.genericdao.search.SearchResult;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 02/02/2016 - 16:23:03
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
@Service("processoService")
public class ProcessoServiceImpl extends GenericServiceImpl<Processo, Long> implements ProcessoService {

	private static final String ECM_VALIDACAO_PERMISSAO_OPERACAO = "ECM.VALIDACAO.PERMISSAO_OPERACAO";

	/** Cosntante VALIDAR_OUTRAS_UNIDADES. */
	private static final String VALIDAR_OUTRAS_UNIDADES = "ECM.VALIDACAO.VALIDAR_ALTERACAO_NIVEL_ACESSO_OUTRAS_UNIDADES";

	private final ProcessoDao processoDao;

	@Autowired
	private DocumentoGedService documentoGedService;

	/** Atributo unidadeService. */
	@Autowired
	private UnidadeService unidadeService;

	/** Atributo solrService. */
	@Autowired
	private SolrService solrService;

	/** Atributo unidadeProcessoService. */
	@Autowired
	private UnidadeProcessoService unidadeProcessoService;

	@Autowired
	private DominioService dominioService;

	/** Atributo temporalidadeService. */
	@Autowired
	private TemporalidadeService temporalidadeService;

	/** Atributo planoClassificacaoService. */
	@Autowired
	private PlanoClassificacaoService planoClassificacaoService;

	/** Atributo sigiloService. */
	@Autowired
	private SigiloService sigiloService;

	/** Atributo grupoService. */
	@Autowired
	private GrupoService grupoService;

	/** Atributo credencialProcessoService. */
	@Autowired
	private CredencialProcessoService credencialProcessoService;

	/** Atributo historicoAtribuicaoProcessoService. */
	@Autowired
	private HistoricoAtribuicaoProcessoService historicoAtribuicaoProcessoService;

	/** Atributo request. */
	@Autowired
	private HttpServletRequest request;

	/** Atributo notificacaoService. */
	@Autowired
	private NotificacaoService notificacaoService;

	@Autowired
	private HistoricoAlteracaoProcessoService historicoAlteracaoProcessoService;
	
	@Autowired
	private EstruturaOrganizacionalECMService estruturaOrganizacionalECMService;

	@Autowired
	public ProcessoServiceImpl( final ProcessoDao processoDao, @Qualifier("processoValidator") final Validator validator ) {

		this.dao = processoDao;
		this.processoDao = processoDao;
		this.validator = validator;
	}

	@Override
	public Processo save(Processo entity) {

		this.validarEntidade(entity, this.validator);
		this.validarUnidade();

		this.montarEntidade(entity);
		entity.setSigiloOriginal(entity.getSigilo());
		entity.setNivelAcessoOriginal(entity.getNivelAcesso());

		Processo processo = super.save(entity);

		if (processo.getTipoProtocolo().getCodigo().equals(1L)) {
			this.gerarNUP(processo);
		}

		this.historicoAlteracaoProcessoService.gerarHistoricoIniciacaoProcesso(processo);

		this.unidadeProcessoService.saveUnidadeProcesso(processo);

		// Verifica se o processo é sigiloso, verdadeiro sejá estarta o businessProcess, falso envia uma notificação para os grupos responsáeis.
		if (!UtilObjeto.isReferencia(processo.getSigilo())) {

			entity.setStatus(this.dominioService.findByChaveAndCodigo(Dominio.TIPO_STATUS_PROCESSO_ECM, Dominio.TIPO_STATUS_PROCESSO_ECM_EM_ANDAMENTO));

			this.startBusinessProcess(processo);

			this.saveSolr(processo);

		} else {

			this.enviarNotificacaoProcessoSigiloso(processo);
		}

		return processo;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por enviar notificação para os responsável sobre a criação do processo sigiloso
	 * 
	 * @author rogerio.costa
	 * 
	 * @param processo
	 */
	private void enviarNotificacaoProcessoSigiloso(Processo processo) {

		StringBuilder sb = new StringBuilder();

		sb.append("O processo ").append(processo.getNup()).append(" foi criado como ").append(processo.getSigilo().getTipoSigilo().getDescricao());

		Dominio tipoNotificacao = this.dominioService.findByChaveAndCodigo("tipoNotificacao", 4L);

		Dominio tipoPrioridade = this.dominioService.findByChaveAndCodigo("tipoPrioridade", 2L);

		Notificacao notificacao = new Notificacao("Processo sigiloso", sb.toString(), tipoNotificacao, tipoPrioridade, null, null, null);

		Collection<Grupo> grupos = this.credencialProcessoService.findGrupoPorSigilo(processo.getSigilo());

		if (!UtilColecao.isVazio(grupos)) {

			notificacao.setNotificacaoGrupos(new LinkedList<NotificacaoGrupo>());

			for (Grupo grupo : grupos) {

				NotificacaoGrupo notificacaoGrupo = new NotificacaoGrupo(grupo, notificacao);

				notificacao.getNotificacaoGrupos().add(notificacaoGrupo);
			}

		}

		this.notificacaoService.save(notificacao);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por validar a unidade vigente
	 * 
	 * @author rogerio.costa
	 * 
	 */
	private void validarUnidade() {

		if (!UtilObjeto.isReferencia(this.getUsuario().getUnidade())) {
			throw new BusinessException("ECM.VALIDACAO.ERRO_UNIDADE_ECM_VIGENTE_USUARIO", CodigoErro.REGRA_NEGOCIO.getValue());
		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author rogerio.costa
	 * 
	 * @param processo
	 */
	private void saveSolr(Processo processo) {

		try {

			this.solrService.addProcesso(processo);

		} catch (final Exception e) {

			e.printStackTrace();
		}
	}

	/**
	 * 
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author andre.silva
	 * 
	 * @param entity
	 * @return Processo
	 */
	@Override
	public Processo atualizarSigiloProcesso(Processo entity) {

		Processo processo = (Processo) this.processoDao.getReference(entity.getId());

		if (entity.getStatus().getCodigo() == 3L) {

			this.validarEntidade(entity, this.validator);

			this.montarEntidade(processo);

			processo.setSigiloOriginal(processo.getSigilo());
			processo.setNivelAcessoOriginal(processo.getNivelAcesso());
			processo.setSigilo(entity.getSigilo());
			processo.setNivelAcesso(entity.getNivelAcesso());
			processo.setValidado(entity.getValidado());
			processo.setStatus(this.dominioService.findByChaveAndCodigo("statusProcesso", 4L));

			if (UtilObjeto.isReferencia(processo.getValidado()) && processo.getValidado()) {

				this.startBusinessProcess(processo);
			}

		}

		processo.setDocumentos(this.documentoGedService.atualizarSigiloDocumentos(entity.getDocumentos()));

		return processo;
	}

	@Override
	public Processo merge(Processo processo) {

		this.validarEntidade(processo, this.validator);

		this.montarEntidade(processo);

		this.saveSolr(processo);

		this.validarAbertosEmOutrasUnidades(processo);

		return super.merge(processo);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por startar o BusinessProcess
	 * 
	 * @author rogerio.costa
	 * 
	 * @param processo
	 */
	private void startBusinessProcess(Processo processo) {

		final Unidade unidade = this.unidadeService.getReference(processo.getAutor().getUnidade().getId());

		final List<EnvironmentVariable> variaveis = new ArrayList<EnvironmentVariable>();

		final Map<String, Object> mapProcesso = new HashMap<String, Object>();

		mapProcesso.put("nivelAcesso", processo.getNivelAcesso().getNivelAcesso().getDescricao());
		mapProcesso.put("nup", processo.getNup());
		mapProcesso.put("assunto", processo.getAssunto().getCodigo() + '/' + processo.getAssunto().getNome());
		final JSONObject processoJSON = new JSONObject(mapProcesso);

		variaveis.add(new EnvironmentVariable("idProcesso", VariableTypeEnum.LONG, processo.getId(), false));
		variaveis.add(new EnvironmentVariable("nup", VariableTypeEnum.JSON, processo.getNup(), false));
		variaveis.add(new EnvironmentVariable("solicitante", VariableTypeEnum.TEXT, processo.getAutor().getUsername(), false));

		// Verifica se o processo é sigiloso.
		if (UtilObjeto.isReferencia(processo.getSigilo())) {

			variaveis.add(new EnvironmentVariable("usuario_criador_processo_sigiloso", VariableTypeEnum.JSON, processo.getAutor().getUsername(), false));

			List<Grupo> gruposPadraoSigilo = (List<Grupo>) this.findGrupoPorSigilo(processo.getSigilo());

			if (!UtilColecao.isVazio(gruposPadraoSigilo)) {
				// No momento o sistema está aceitando somente um grupo para gerenciar cada nivel de sigilo.
				variaveis.add(new EnvironmentVariable("grupoPadrao", VariableTypeEnum.TEXT, gruposPadraoSigilo.get(0).getSigla(), false));
			}

			// Verifica se foi configurado um grupo padrão para unidade o usuario.
		} else if (UtilObjeto.isReferencia(unidade.getGrupo())) {

			variaveis.add(new EnvironmentVariable("grupoPadrao", VariableTypeEnum.TEXT, unidade.getGrupo().getSigla(), false));
		} else {
			throw new BusinessException("ECM.VALIDACAO.ERRO_CONFIGURACAO_GRUPO_PADRAO_USUARIO", CodigoErro.REGRA_NEGOCIO.getValue());
		}
		variaveis.add(new EnvironmentVariable("processoSIGAD", VariableTypeEnum.JSON, processoJSON, false));

		if (processo.getTipoProcesso().getProcessoNegocio() == null) {
			// O processo deverá estar vinculado a um fluxo no BPE/ESI
			throw new BusinessException("ECM.VALIDACAO.ERRO_VINCULO_FLUXO_BPE_ESI", CodigoErro.REGRA_NEGOCIO.getValue());
		}

		EnvironmentOutput environmentOutput = this.startBusinessProcess(processo.getTipoProcesso().getProcessoNegocio(), variaveis);

		processo.setIdProcessInstance(environmentOutput.getProcessInstanceId());

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por retornar os privilegios através do sigilo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param sigilo
	 * @return Collection<Privilegio>
	 */
	private Collection<Long> getPrivilegios(Sigilo sigilo) {

		Collection<Long> listIdsPrivilegio = new LinkedList<Long>();

		sigilo = this.sigiloService.getReference(sigilo.getId());

		if (!UtilColecao.isVazio(sigilo.getSigiloPrivilegios())) {
			// Cria uma lista de privilegios relacionados ao sigilo do processo
			for (SigiloPrivilegio sigiloPrivilegio : sigilo.getSigiloPrivilegios()) {

				listIdsPrivilegio.add(sigiloPrivilegio.getPrivilegio().getId());
			}
		}

		return listIdsPrivilegio;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter grupos através do sigilo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param sigilo
	 * 
	 * @return Collection<Grupo>
	 */
	private Collection<Grupo> findGrupoPorSigilo(Sigilo sigilo) {

		Collection<Long> idPrivilegios = this.getPrivilegios(sigilo);

		Collection<Grupo> grupos = this.grupoService.findByPrivilegio(idPrivilegios);

		return grupos;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por resolver as pripriedades transiente
	 * 
	 * @author rogerio.costa
	 * 
	 * @param processo
	 */
	private void montarEntidade(final Processo processo) {

		if (processo.getInteressados() != null && processo.getInteressados().size() > 0) {
			for (final InteressadoProcesso interessado : processo.getInteressados()) {
				interessado.setProcesso(processo);
			}
		}

		if (UtilObjeto.isReferencia(processo.getTipoProtocolo())) {

			processo.setTipoProtocolo(this.dominioService.findByChaveAndCodigo("tipoProtocoloNup", processo.getTipoProtocolo().getCodigo()));

		}

		// Verifica se o sigilo foi selecionado
		if (UtilObjeto.isReferencia(processo.getSigilo()) && UtilObjeto.isReferencia(processo.getSigilo().getId())) {
			processo.setSigilo(this.sigiloService.getReference(processo.getSigilo().getId()));

			if (processo.isNew()) {
				// Processo sigiloso inicia com o status Aguardando validação.
				processo.setStatus(this.dominioService.findByChaveAndCodigo("statusProcesso", 3L));
			}

			this.atribuirPrazoPadraoProcesso(processo);
		} else {
			processo.setSigilo(null);
		}

		this.saveTemporalidade(processo);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por incluir temporalidade. História de usuário #2182
	 * 
	 * @author rogerio.costa
	 * 
	 * @param processo
	 */
	private void saveTemporalidade(Processo processo) {

		PlanoClassificacao planoClassificacao = this.planoClassificacaoService.getReference(processo.getAssunto().getId());

		Temporalidade temporalidade = new Temporalidade(planoClassificacao.getFaseCorrente(), planoClassificacao.getFaseIntermediaria(), planoClassificacao.getTipoDestinacao());

		temporalidade = this.temporalidadeService.save(temporalidade);

		processo.setTemporalidade(temporalidade);
	}

	/**
	 * 
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author andre.silva
	 * 
	 * @param processo
	 * @return
	 */
	private String gerarSequencia() {

		Calendar calendarAtual = Calendar.getInstance();

		Calendar calendarInicial = Calendar.getInstance();

		Calendar calendarFinal = Calendar.getInstance();

		calendarInicial.set(Calendar.YEAR, calendarInicial.get(Calendar.YEAR));
		calendarInicial.set(Calendar.MONTH, Calendar.JANUARY);
		calendarInicial.set(Calendar.DAY_OF_MONTH, 1);

		calendarFinal.set(Calendar.YEAR, calendarFinal.get(Calendar.YEAR));
		calendarFinal.set(Calendar.MONTH, Calendar.DECEMBER);
		calendarFinal.set(Calendar.DAY_OF_MONTH, 31);

		calendarFinal.set(calendarFinal.get(Calendar.YEAR), Calendar.DECEMBER, 31);
		calendarInicial.set(calendarInicial.get(Calendar.YEAR), Calendar.JANUARY, 1);

		Long quantidade = this.processoDao.obterQuantidadePorProcessoEDataCriacao(calendarInicial, calendarFinal);

		if (calendarAtual.after(calendarFinal)) {
			quantidade = 0L;
		} else {
			quantidade++;
		}

		return StringUtils.leftPad(quantidade.toString(), 8, "0");

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author andre.silva
	 * 
	 * @param processo
	 */
	private void gerarNUP(final Processo processo) {

		String nup = "";
		EstruturaOrganizacionalECM unidadeVigenteUsuario = null;

		try {
			unidadeVigenteUsuario = (EstruturaOrganizacionalECM) this.getUsuario().getUnidade();
			if (null == unidadeVigenteUsuario) {
				throw new BusinessException("ECM.VALIDACAO.ERRO_UNIDADE_ECM_VIGENTE_USUARIO", CodigoErro.REGRA_NEGOCIO.getValue());
			}
		} catch (final Exception e) {
			throw new BusinessException("ECM.VALIDACAO.ERRO_UNIDADE_ECM_VIGENTE_USUARIO", CodigoErro.REGRA_NEGOCIO.getValue());
		}
		if(UtilObjeto.isReferencia(unidadeVigenteUsuario) && UtilObjeto.isReferencia(unidadeVigenteUsuario.getId())){
			EstruturaOrganizacionalECM unidade = this.estruturaOrganizacionalECMService.getReference(unidadeVigenteUsuario.getId());
			nup += unidade.getCodigoSiorg() + ".";
		}
		
		nup += this.gerarSequencia() + "/";
		nup += new GregorianCalendar().get(Calendar.YEAR) + "-";
		processo.setNup(nup + this.calculaDvNupNovo(nup));

	}

	private String calculaDvNupNovo(final String nup) {

		// Recebe o NUP sem o DV e retorna a string com o DV no final...
		// ... utilizando o algoritmo previsto na Portaria Interministerial MP
		// MJ n 2321 de 30 de dezembro de 2014.

		String toReturn = "";
		try {
			// Gera representacao do numero
			final BigInteger numeroOriginal = new BigInteger(nup.replaceAll("\\D+", ""));

			// Multiplica por 100 e retorna valor no objeto numeroMultiplicado
			final BigInteger numeroMultiplicado = numeroOriginal.multiply(new BigInteger("100"));

			// Resto da divisao inteira, por 97
			final BigInteger restoDivisao = numeroMultiplicado.remainder(new BigInteger("97"));

			// Calculando o DV, que e, pelo algoritmo, igual a 98 - resto da
			// divisao inteira.
			final int DV = 98 - restoDivisao.intValue();

			// Converte o DV para texto
			String DV_emString = String.valueOf(DV);

			// Se o DV foi menor que 10, tem que adicionar um zero a esquerda
			if (DV < 10) {
				DV_emString = "0" + DV_emString;
			}
			toReturn = DV_emString;

		} catch (NumberFormatException exception) {

			this.log.error("Nao foi possivel transformar o valor =  " + nup + " , em um numero. \n" + "A mensagem retornada foi: " + exception.getMessage());
		}

		return toReturn;
	}

	@Override
	public boolean remove(Processo processo) {

		// Regra de negócio, ao excluir um processo, excluir também os
		// documentos vinculados logicamente

		final Collection<DocumentoGed> docs = this.documentoGedService.findByProcesso(processo.getId());
		for (final DocumentoGed doc : docs) {
			this.documentoGedService.remove(doc);
		}
		return super.remove(processo);
	}

	@Override
	public Processo getReference(Long id) {

		Processo processo = super.getReference(id);

		processo.setDocumentos(this.documentoGedService.findByIdProcessoUltimaVersao(id));

		return processo;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter o processo.
	 * 
	 * @author rogerio.costa
	 * 
	 * @param id
	 * @return
	 */
	public ProcessoVH getProcessoView(Long id) {

		Processo processo = super.getReference(id);

		this.verificarPermissaoAcessoProcesso(id, processo);

		processo.setDocumentos(this.documentoGedService.findByIdProcessoUltimaVersao(id));

		ProcessoVH processoVH = new ProcessoVH();

		processoVH.setProcesso(processo);

		return processoVH;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar se o usuario contem permissão para acessar o processo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param id
	 * @param processo
	 */
	private void verificarPermissaoAcessoProcesso(Long id, Processo processo) {

		if (processo.getNivelAcesso().getNivelAcesso().getCodigo().equals(0L)) {

			if (!this.credencialProcessoService.contemPermissaoProcesso(id)) {

				throw new BusinessException(ECM_VALIDACAO_PERMISSAO_OPERACAO, CodigoErro.REGRA_NEGOCIO.getValue());
			}

		} else if (processo.getNivelAcesso().getNivelAcesso().getCodigo().equals(1L)) {

			if (!this.unidadeProcessoService.isProcessoNaUnidadeUsuarioLogado(id)) {

				throw new BusinessException(ECM_VALIDACAO_PERMISSAO_OPERACAO, CodigoErro.REGRA_NEGOCIO.getValue());

			}
		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por iniciar o gerenciamento do processo
	 * 
	 * @author rogerio.costa
	 * 
	 * @return Processo
	 */
	@Override
	public Processo getProcessoGerenciamento(Long id) {

		this.request.getSession().setAttribute("idProcesso", id);

		Processo processo = super.getReference(id);

		processo.setDocumentos(this.documentoGedService.findByIdProcessoUltimaVersao(id));
		
		this.documentoGedService.montarTresAssinantesPorDocumento(processo.getDocumentos());

		return processo;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por atribuir processo para o usuario logado no primeiro acesso
	 * 
	 * @author rogerio.costa
	 * 
	 * @param processo
	 */
	@Override
	public Boolean verificarAtribuicaoProcessoPrimeiroAcesso(Long idProcesso) {

		Processo processo = super.getReference(idProcesso);

		// Verifica se o usuario logado está lotado na unidade.
		if (UtilObjeto.isReferencia(this.getUsuario().getUnidade())) {

			UnidadeProcesso unidadeProcesso = this.unidadeProcessoService.obterPorUnidadeEProcessoEmExecucao(processo.getId(), this.getUsuario().getUnidade().getId());
			// Verifica se o processo foi enviado para essa unidade e se é o primeiro acesso
			if (UtilObjeto.isReferencia(unidadeProcesso) && unidadeProcesso.getPrimeiroAcesso()) {

				unidadeProcesso.setUsuarioResponsavel(this.getUsuario());
				unidadeProcesso.setPrimeiroAcesso(Boolean.FALSE);

				this.historicoAtribuicaoProcessoService.saveHistorico(unidadeProcesso);

				this.historicoAlteracaoProcessoService.gerarHistoricoCapturaDeProcesso(unidadeProcesso);

				return Boolean.TRUE;
			}

		}

		return Boolean.FALSE;

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por concluír o processo na unidade
	 * 
	 * @author rogerio.costa
	 * 
	 * @param processo
	 */
	@Override
	public Boolean concluir(Long idProcesso, Long idTask) {

		Processo processo = this.getReference(idProcesso);

		this.unidadeProcessoService.executarConcluirProcessoNaUnidade(idTask, processo);

		Boolean concluidoEmTodasUnidades = this.unidadeProcessoService.isProcessoConcluidoEmTodasUnidades(idProcesso);

		// Verifica se o processo foi concluído em todas unidades.
		if (concluidoEmTodasUnidades) {

			processo.setStatus(this.dominioService.findByChaveAndCodigo("statusProcesso", 1L));
			processo.getTemporalidade().setDataFimTemporalidade(this.executarCalculoTemporalidade(processo));
			processo.setDataConclusao(UtilDate.getDataAtualCalendar());

		}

		return concluidoEmTodasUnidades;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por realizar o cálculo da temporalidade
	 * 
	 * @author rogerio.costa
	 * 
	 * @param processo
	 * 
	 * @return dataFimTemporalidade
	 */
	private Calendar executarCalculoTemporalidade(Processo processo) {

		GregorianCalendar calendar = new GregorianCalendar();

		// quantidade de dias do ano
		int diasAnos = calendar.isLeapYear(Calendar.getInstance().get(Calendar.YEAR)) ? 366 : 365;

		// vida util do processo é a data atual menos a data de criação.
		Long tempoVidaUtilProcessoMinu = ( Calendar.getInstance().getTime().getTime() - processo.getDataCriacao().getTime().getTime() ) / 60L / 60L / 60L;

		// faseCorrente em minutos.
		Integer tempoFaseCorrente = processo.getTemporalidade().getFaseCorrente() * diasAnos * 24 * 60;

		// faseIntermediaria em minutos.
		Integer TempoFaseIntermediaria = processo.getTemporalidade().getFaseIntermediaria() * diasAnos * 24 * 60;
		// tempoFinal para descartar o processo em minutos
		Integer temporalizacao = (int) ( ( tempoFaseCorrente - tempoVidaUtilProcessoMinu ) + TempoFaseIntermediaria );

		Calendar dataFimTemporalidade = Calendar.getInstance();
		// add os minutos para obter a data final
		dataFimTemporalidade.add(Calendar.MINUTE, temporalizacao);

		return dataFimTemporalidade;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por atualizar a temporalidade e destinação dos processos e documentos
	 * 
	 * @author rogerio.costa
	 * 
	 * @param planoClassificacao
	 */
	@Override
	public void atualizarTemporalidadeDestinacao(PlanoClassificacao planoClassificacao) {

		this.atualizarTemporalidadeProcesso(planoClassificacao);

		this.atualizarTemporalidadeDocumento(planoClassificacao);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por atualizar a temporalidade do documento
	 * 
	 * @author rogerio.costa
	 * 
	 * @param planoClassificacao
	 */
	private void atualizarTemporalidadeDocumento(PlanoClassificacao planoClassificacao) {

		Collection<DocumentoGed> listaDocumentoGed = this.documentoGedService.findPorPlanoClassificacao(planoClassificacao.getId());

		for (DocumentoGed documentoGed : listaDocumentoGed) {

			this.verificarAtualizacaoTemporalidadeAtravesDoPlanoClassificacao(planoClassificacao, documentoGed.getProcesso());

		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por atualizar temporalidade do processo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param planoClassificacao
	 */
	private void atualizarTemporalidadeProcesso(PlanoClassificacao planoClassificacao) {

		Collection<Processo> listaProcesso = this.processoDao.findPorPlanoClassificacao(planoClassificacao.getId());

		for (Processo processo : listaProcesso) {

			this.verificarAtualizacaoTemporalidadeAtravesDoPlanoClassificacao(planoClassificacao, processo);

		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por atualizar a temporalidade do processo através do processo anexado. História de usuário #2182
	 * 
	 * @author rogerio.costa
	 * 
	 * @param processo
	 * @param processoAnexo
	 */
	@Override
	public void atualizarTemporalidadeProcessoAnexado(Processo processo, Processo processoAnexo) {

		this.verificarAtualizacaoTemporalidadeAtravesDaTemporalidadeDoProcesso(processoAnexo.getTemporalidade(), processo);

		if (!UtilColecao.isVazio(processoAnexo.getDocumentos())) {

			for (DocumentoGed documentoGed : processoAnexo.getDocumentos()) {

				PlanoClassificacao planoClassificacao = this.planoClassificacaoService.getReference(documentoGed.getPlanoClassificacao().getId());

				this.verificaTemporalidadeProcessoAtravesDoPlanoClassificacacao(planoClassificacao, processoAnexo);

			}

		}

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar atualização da temporalidade através do planoClassificação. História de usuário #2182
	 * 
	 * @author rogerio.costa
	 * 
	 * @param planoClassificacao
	 * @param processo
	 */
	private void verificarAtualizacaoTemporalidadeAtravesDoPlanoClassificacao(PlanoClassificacao planoClassificacao, Processo processo) {

		boolean alterouTemporalidade = false;

		if (planoClassificacao.getPermitirUsarComoClassificacao()) {

			// Verifica se o tipo destinacao do planoClassificacao é Guarda Permanente e se a do processo é eliminação, caso seja atualiza o processo
			if (processo.getTemporalidade().getTipoDestinacao().getCodigo().equals(2L) && planoClassificacao.getTipoDestinacao().getCodigo().equals(1L)) {

				processo.getTemporalidade().setTipoDestinacao(planoClassificacao.getTipoDestinacao());

				alterouTemporalidade = Boolean.TRUE;

			}
			// Verifica se a fase corrente do documento é maior do que a do processo, caso seja atualiza o processo
			if (planoClassificacao.getFaseCorrente() > processo.getTemporalidade().getFaseCorrente()) {

				processo.getTemporalidade().setFaseCorrente(planoClassificacao.getFaseCorrente());

				alterouTemporalidade = Boolean.TRUE;

			}
			// Verifica se a fase intermediaria do documento é maior do que a do processo, caso seja atualiza o processo
			if (planoClassificacao.getFaseIntermediaria() > processo.getTemporalidade().getFaseIntermediaria()) {

				processo.getTemporalidade().setFaseIntermediaria(planoClassificacao.getFaseIntermediaria());

				alterouTemporalidade = Boolean.TRUE;
			}

			// Verifica se algum parametro da temporalidade do processo foi alterado, caso seja raliza o calculo da temporalidade novamente.
			if (alterouTemporalidade) {

				this.executarCalculoTemporalidade(processo);
			}
		}

	}

	/**
	 * 
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author andre.silva
	 * 
	 * @param entity
	 *            História de usuário #3216_CA005
	 */
	private void atribuirPrazoPadraoProcesso(Processo entity) {

		Calendar calendarPrazoSigilo = Calendar.getInstance();

		if (UtilObjeto.isReferencia(entity.getSigilo())) {

			// Reservado – prazo padrão de sigilo é de 5 anos a partir da data de criação, podendo ser prorrogado
			if (entity.getSigilo().getTipoSigilo().getCodigo() == 0L) {
				calendarPrazoSigilo.add(Calendar.YEAR, 5);
				entity.setPrazoSigilo(calendarPrazoSigilo);
				// Secreto – prazo padrão de sigilo é de 15 anos a partir da data de criação, podendo ser prorrogado
			} else if (entity.getSigilo().getTipoSigilo().getCodigo() == 1L) {
				calendarPrazoSigilo.add(Calendar.YEAR, 15);
				entity.setPrazoSigilo(calendarPrazoSigilo);
				// Ultrassecreto – prazo padrão de sigilo é de 25 anos a partir da data de criação, podendo ser prorrogado
			} else if (entity.getSigilo().getTipoSigilo().getCodigo() == 2L) {
				calendarPrazoSigilo.add(Calendar.YEAR, 25);
				entity.setPrazoSigilo(calendarPrazoSigilo);

			}
		}
	}

	/**
	 * 
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author andre.silva
	 * 
	 * @param entity
	 *            História de usuário #3216_CA007
	 */
	private void validarAbertosEmOutrasUnidades(Processo entity) {

		// verifica se o documento está passando a ser sigiloso
		if (UtilObjeto.isReferencia(entity.getSigilo()) && !this.processoDao.isSigiloso(entity.getId())) {

			if (this.unidadeProcessoService.isProcessoEnviado(entity.getId())) {

				throw new BusinessException(VALIDAR_OUTRAS_UNIDADES, CodigoErro.REGRA_NEGOCIO.getValue());
			}
		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por atualizar temporalidade através da temporalidade do processo. História de usuário #2182
	 * 
	 * @author rogerio.costa
	 * 
	 * @param temporalidade
	 * @param processo
	 */
	public void verificarAtualizacaoTemporalidadeAtravesDaTemporalidadeDoProcesso(Temporalidade temporalidade, Processo processo) {

		// Verifica se o tipo destinacao do planoClassificacao é Guarda Permanente e se a do processo é eliminação, caso seja atualiza o processo
		if (processo.getTemporalidade().getTipoDestinacao().getCodigo().equals(2L) && temporalidade.getTipoDestinacao().getCodigo().equals(1L)) {

			processo.getTemporalidade().setTipoDestinacao(temporalidade.getTipoDestinacao());

		}
		// Verifica se a fase corrente do documento é maior do que a do processo, caso seja atualiza o processo
		if (temporalidade.getFaseCorrente() > processo.getTemporalidade().getFaseCorrente()) {

			processo.getTemporalidade().setFaseCorrente(temporalidade.getFaseCorrente());

		}
		// Verifica se a fase intermediaria do documento é maior do que a do processo, caso seja atualiza o processo
		if (temporalidade.getFaseIntermediaria() > processo.getTemporalidade().getFaseIntermediaria()) {

			processo.getTemporalidade().setFaseIntermediaria(temporalidade.getFaseIntermediaria());

		}

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por atualizar a temporalidade do processo através do plano de classificação. História de usuário #2182
	 * 
	 * @author rogerio.costa
	 * 
	 * @param entity
	 */
	@Override
	public void verificaTemporalidadeProcessoAtravesDoPlanoClassificacacao(PlanoClassificacao planoClassificacao, Processo processo) {

		planoClassificacao = this.planoClassificacaoService.getReference(planoClassificacao.getId());

		if (UtilObjeto.isReferencia(planoClassificacao.getPermitirUsarComoClassificacao()) && planoClassificacao.getPermitirUsarComoClassificacao()) {

			// Verifica se a fase corrente do documento é maior do que a do processo, caso seja atualiza o processo
			if (planoClassificacao.getFaseCorrente() > processo.getTemporalidade().getFaseCorrente()) {

				processo.getTemporalidade().setFaseCorrente(planoClassificacao.getFaseCorrente());

			}
			// Verifica se a fase intermediaria do documento é maior do que a do processo, caso seja atualiza o processo
			if (planoClassificacao.getFaseIntermediaria() > processo.getTemporalidade().getFaseIntermediaria()) {

				processo.getTemporalidade().setFaseIntermediaria(planoClassificacao.getFaseIntermediaria());
			}
			// Verifica se o tipo destinacao do documento é Guarda Permanente e se a do processo é eliminação, caso seja atualiza o processo
			if (planoClassificacao.getTipoDestinacao().getCodigo().equals(1L) && processo.getTemporalidade().getTipoDestinacao().getCodigo().equals(2L)) {

				processo.getTemporalidade().setTipoDestinacao(planoClassificacao.getTipoDestinacao());
			}
		}
	}
	
	/**
	 * Método responsável por Reabrir processo  BPE para uma unidade
	 *
	 *O processo será aberto no ponto que final da execução do fluxo
	 *
	 * @author maycon.silva
	 *
	 * @param processo
	 * @param unidade
	 */
	public void reopenProcessIntance(Processo processo, Unidade unidade ){
		
		final Unidade unidadeReabertura = this.unidadeService.getReference(unidade.getId());

		final Processo processoReabertura = this.getReference(processo.getId());		
		
		final List<EnvironmentVariable> variaveis = new ArrayList<EnvironmentVariable>();		

		processoReabertura.setStatus(this.dominioService.findByChaveAndCodigo("statusProcesso", 2L));	
		
		 UnidadeProcesso unidadeProcesso = unidadeProcessoService.obterPorUnidadeEProcesso(processoReabertura.getId(), unidade.getId());
		
		 unidadeProcesso.setConcluido(false);

		variaveis.add(new EnvironmentVariable("grupoPadrao", VariableTypeEnum.TEXT, unidadeReabertura.getGrupo().getSigla(), false));

		reopenProcessInstance(processoReabertura.getIdProcessInstance(), variaveis);
		
		historicoAlteracaoProcessoService.gerarHistoricoReaberturProcesso(processo, unidade);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por listar através da permissão que o usuario tem
	 * 
	 * @author rogerio.costa
	 * 
	 * @param search
	 * 
	 * @return <RT> SearchResult<RT>
	 */
	public <RT> SearchResult<RT> searchAndCountPorSigilo(ISearch search) {

		return this.processoDao.searchAndCountPorSigilo(search);
	}

	/**
	 *
	 */
	@Override
	public Processo getProcessoByIdProcessInstance(Long idProcessInstance) {

		return this.processoDao.getProcessoByIdProcessInstance(idProcessInstance);
	}

	@Override
	public Processo getProcessoByProtocolo(String nup) {

		return this.processoDao.getProcessoByProtocolo(nup);
	}

	/**
	 *
	 */
	public Collection<Processo> autoCompleteProcessoEmAndametoByNup(String nup) {

		return this.processoDao.autoCompleteProcessoEmAndamentoByNup(nup);

	}
	
	public Collection<Processo> autoCompleteProcessoConcluidoByNup(String nup) {

		return this.processoDao.autoCompleteProcessoConcluidoByNup(nup);

	}
	
	

}
