package br.com.centralit.api.service.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import br.com.centralit.api.dao.ReabrirProcessoDao;
import br.com.centralit.api.model.ReabrirProcesso;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.GrupoService;
import br.com.centralit.api.service.NotificacaoService;
import br.com.centralit.api.service.ProcessoService;
import br.com.centralit.api.service.ReabrirProcessoService;
import br.com.centralit.api.service.TemporalidadeService;
import br.com.centralit.api.service.UnidadeProcessoService;
import br.com.centralit.api.service.UnidadeService;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Grupo;
import br.com.centralit.framework.model.Notificacao;
import br.com.centralit.framework.model.NotificacaoGrupo;
import br.com.centralit.framework.model.NotificacaoUsuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 01/04/2016 - 19:11:27
 * 
 * @version 1.0.0
 * 
 * @author maycon.silva
 * 
 */
@Service("reabrirProcessoServiceImpl")
public class ReabrirProcessoServiceImpl extends GenericServiceImpl<ReabrirProcesso, Long> implements ReabrirProcessoService {

	private static final String SOLICITAÇÃO_DE_REABERTURA = "Solicitação de Reabertura";

	private static final String PROTOCOLO = "PROTOCOLO";

	/** Atributo reabrirProcessoDao. */
	private ReabrirProcessoDao reabrirProcessoDao;

	@Autowired
	private ProcessoService processoService;

	@Autowired
	private UnidadeService unidadeService;

	@Autowired
	private DominioService dominioService;

	@Autowired
	private UnidadeProcessoService unidadeProcessoService;

	@Autowired
	private TemporalidadeService temporalidadeService;

	@Autowired
	private NotificacaoService notificacaoService;

	/** Atributo grupoService. */
	@Autowired
	private GrupoService grupoService;

	@Autowired
	public ReabrirProcessoServiceImpl( ReabrirProcessoDao reabrirProcessoDao ) {

		this.dao = reabrirProcessoDao;

		this.reabrirProcessoDao = reabrirProcessoDao;

	}

	@Override
	public ReabrirProcesso save(ReabrirProcesso reabrirProcesso) {

		reabrirProcesso.setStatus(this.dominioService.findByChaveAndCodigo("statusSolicitaoProcesso", 1L));

		reabrirProcesso.setValidado(false);

		reabrirProcesso = super.save(reabrirProcesso);

		this.enviarNotificacaoReabertura(reabrirProcesso);

		return reabrirProcesso;
	}

	@Override
	public ReabrirProcesso merge(ReabrirProcesso reabrirProcesso) {

		reabrirProcesso = (ReabrirProcesso) this.reabrirProcessoDao.merge(reabrirProcesso);

		if (!reabrirProcesso.getStatus().getCodigo().equals(1L)) {

			reabrirProcesso.setValidado(true);

			if (reabrirProcesso.getStatus().getCodigo().equals(2L)) {

				this.processoService.reopenProcessIntance(reabrirProcesso.getProcesso(), reabrirProcesso.getUnidade());
				
				
			}

			this.enviarNotificacaoStatusReabertura(reabrirProcesso);

		}

		return reabrirProcesso;
	}

	private void enviarNotificacaoReabertura(ReabrirProcesso reabrirProcesso) {

		StringBuilder sb = new StringBuilder();

		sb.append("Foi aberta uma solicitação de reabertura para o proceso ").append(reabrirProcesso.getProcesso().getNup());

		Dominio tipoNotificacao = this.dominioService.findByChaveAndCodigo("tipoNotificacao", 4L);

		Dominio tipoPrioridade = this.dominioService.findByChaveAndCodigo("tipoPrioridade", 2L);

		Notificacao notificacao = new Notificacao(SOLICITAÇÃO_DE_REABERTURA, sb.toString(), tipoNotificacao, tipoPrioridade, null, null, null);

		Collection<Grupo> grupos = this.grupoService.findGrupoPorNomePrivilegio(PROTOCOLO);

		if (!UtilColecao.isVazio(grupos)) {

			notificacao.setNotificacaoGrupos(new LinkedList<NotificacaoGrupo>());

			for (Grupo grupo : grupos) {

				NotificacaoGrupo notificacaoGrupo = new NotificacaoGrupo(grupo, notificacao);

				notificacao.getNotificacaoGrupos().add(notificacaoGrupo);
			}

		}

		this.notificacaoService.save(notificacao);

	}

	private void enviarNotificacaoStatusReabertura(ReabrirProcesso reabrirProcesso) {

		StringBuilder sb = new StringBuilder();

		sb.append("A sua solicitação de reabertura para o proceso ").append(reabrirProcesso.getProcesso().getNup()).append(" foi ");

		if (reabrirProcesso.getStatus().getCodigo().equals(2L)) {

			sb.append("aprovada.");

		} else if (reabrirProcesso.getStatus().getCodigo().equals(3L)) {

			sb.append("rejeitada");
		}

		Dominio tipoNotificacao = this.dominioService.findByChaveAndCodigo("tipoNotificacao", 4L);

		Dominio tipoPrioridade = this.dominioService.findByChaveAndCodigo("tipoPrioridade", 2L);

		Notificacao notificacao = new Notificacao("Reabertura de processo", sb.toString(), tipoNotificacao, tipoPrioridade, null, null, null);

		NotificacaoUsuario notificacaoUsuario = new NotificacaoUsuario(notificacao, reabrirProcesso.getAutor());

		notificacao.setNotificacaoUsuarios(new ArrayList<NotificacaoUsuario>());

		notificacao.getNotificacaoUsuarios().add(notificacaoUsuario);

		this.notificacaoService.save(notificacao);
	}
	
	public Collection<ReabrirProcesso> findSolicitacaoEmAberto(){
		return reabrirProcessoDao.findSolicitacaoEmAberto();
	}

}
