package br.com.centralit.api.service.impl;

import java.io.IOException;
import java.text.SimpleDateFormat;

import org.apache.solr.client.solrj.SolrClient;
import org.apache.solr.client.solrj.SolrServerException;
import org.apache.solr.client.solrj.impl.HttpSolrClient;
import org.apache.solr.common.SolrInputDocument;
import org.jsoup.Jsoup;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import br.com.centralit.api.model.DocumentoGed;
import br.com.centralit.api.model.Processo;
import br.com.centralit.api.service.SolrService;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: DefaultFileServiceImpl</b>
 * </p>
 * 
 * <p>
 * <b>Description: Para o funcionamento correto da solução é necessario configurar as 3 propriedades no arquivo property: ecm.dir.output : Diretorio base onde os arquivos seram gravados. ecm.dir.privateKeyFile : Caminho da privateKey ecm.dir.encryptedKeyFile : Caminho da encriptedKey File </b>
 * </p>
 * 
 * <p>
 * <b>Orientações de como gerar as chaves:</b>
 * 
 * To use the code, you need corresponding public and private RSA keys. RSA keys can be generated using the open source tool OpenSSL. However, you have to be careful to generate them in the format required by the Java encryption libraries. To generate a private key of length 2048 bits: 1. openssl
 * genrsa -out private.pem 2048
 * 
 * To get it into the required (PKCS#8, DER) format: 2. openssl pkcs8 -topk8 -in private.pem -outform DER -out private.der -nocrypt
 * 
 * To generate a public key from the private key: 3. openssl rsa -in private.pem -pubout -outform DER -out public.der
 * 
 * 
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 16/12/2015 - 10:30:24
 * 
 * @version 1.0.0
 * 
 * @author ally.barra
 * 
 */
@Service("solrService")
public class SolrServiceImpl implements SolrService {

	private static final String INFORMAÇÃO_RESTRITA = "Informação restrita";

	@Value("${ecm.solr.server}")
	public String solrServer;

	@Override
	public boolean addProcesso(Processo processo) throws SolrServerException, IOException {

		SimpleDateFormat out = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");

		SolrClient solrClient = new HttpSolrClient(solrServer);

		// Add some example docs
		SolrInputDocument processoDoc = new SolrInputDocument();
		processoDoc.setField("id", processo.getId());
		processoDoc.setField("nup", processo.getNup());
		processoDoc.setField("objeto", "Processo");
		processoDoc.setField("tipoProcesso", processo.getTipoProcesso().getNome());
		processoDoc.setField("tipoDocumento", "");
		processoDoc.setField("assunto", processo.getAssunto().getNome());
		processoDoc.setField("assuntoComplementar", processo.getAssuntoComplementar());
		processoDoc.setField("autor", processo.getAutor().getUsername());
		processoDoc.setField("destinatario", "");
		processoDoc.setField("atribuidoA", "");
		processoDoc.setField("unidadeGeradora", processo.getUnidade().getNome());
		processoDoc.setField("dataCriacao", out.format(processo.getDataCriacao().getTime()));
		processoDoc.setField("situacao", "");
		processoDoc.setField("nivelAcesso", processo.getNivelAcesso().getNivelAcesso().getDescricao());
		processoDoc.setField("formatoDocumento", "");
		processoDoc.setField("idProcessInstance", processo.getIdProcessInstance().toString());

		solrClient.add(processoDoc);

		// Make the docs we just added searchable using a "hard" commit
		solrClient.commit(true, true);

		return true;
	}

	@Override
	public boolean addDocumento(DocumentoGed documentoGed) throws SolrServerException, IOException {

		SimpleDateFormat out = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");

		// Get a connection to Solr
		SolrClient solrClient = new HttpSolrClient(solrServer);

		// Add some example docs
		SolrInputDocument documentoDoc = new SolrInputDocument();

		StringBuilder sb = new StringBuilder();

		sb.append("doc_").append(documentoGed.getIdRaiz() != null ? documentoGed.getIdRaiz() : documentoGed.getId());

		documentoDoc.setField("id", sb.toString());
		documentoDoc.setField("nup", documentoGed.getNumero());
		documentoDoc.setField("objeto", "Documento");
		documentoDoc.setField("tipoDocumento", documentoGed.getTipoDocumento().getNome());
		documentoDoc.setField("assunto", documentoGed.getPlanoClassificacao().getNome());
		documentoDoc.setField("assuntoComplementar", documentoGed.getAssuntoComplementar());
		documentoDoc.setField("autor", documentoGed.getAutor().getUsername());
		documentoDoc.setField("destinatario", UtilObjeto.isReferencia(documentoGed.getDestinatario()) ? documentoGed.getDestinatario().getNome() : "");
		documentoDoc.setField("atribuidoA", "");
		documentoDoc.setField("unidadeGeradora", documentoGed.getUnidade().getNome());
		documentoDoc.setField("dataCriacao", out.format(documentoGed.getDataCriacao().getTime()));
		documentoDoc.setField("situacao", "");
		documentoDoc.setField("nivelAcesso", documentoGed.getNivelAcesso().getNivelAcesso().getDescricao());
		documentoDoc.setField("formatoDocumento", documentoGed.getFormaCriacao().getDescricao());
		documentoDoc.setField("idProcessInstance", documentoGed.getProcesso().getIdProcessInstance().toString());
		documentoDoc.setField("idProcesso", documentoGed.getProcesso().getId().toString());
		// verifica se o nivel de acesso é restrito
		if (!documentoGed.getNivelAcesso().getNivelAcesso().getCodigo().equals(1L)) {

			documentoDoc.setField("conteudo", documentoGed.getFormaCriacao().getCodigo().equals(1L) ? Jsoup.parse(documentoGed.getConteudo()).text() : documentoGed.getConteudo());
		} else {

			documentoDoc.setField("conteudo", INFORMAÇÃO_RESTRITA);
		}

		solrClient.add(documentoDoc);

		// Make the docs we just added searchable using a "hard" commit
		solrClient.commit(true, true);

		return true;
	}
}
