package br.com.centralit.api.service.impl;

import java.io.IOException;
import java.security.Key;

import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;
import org.springframework.web.multipart.MultipartFile;

import br.com.centralit.api.dao.TarjaAssinaturaDao;
import br.com.centralit.api.model.Assinatura;
import br.com.centralit.api.model.ConteudoTarja;
import br.com.centralit.api.model.TarjaAssinatura;
import br.com.centralit.api.service.AnexoImagemService;
import br.com.centralit.api.service.AssinaturaUtilService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.OrganizacaoService;
import br.com.centralit.api.service.TarjaAssinaturaService;
import br.com.centralit.api.util.UtilBase64;
import br.com.centralit.framework.model.AnexoImagem;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Organizacao;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;

import com.ibm.icu.util.Calendar;
import com.lowagie.text.DocumentException;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governan&ccedil;a Corporativa -
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * Classe de servi&ccedil;os para a entidade TarjaAssinatura, a qual mant&eacute;m as regras de neg&oacute;cio.
 * 
 * @since 13 de abr de 2016 - 13:40:36
 * @version 1.0.0
 * @author ciro.junior (<a href="mailto:ciro.junior@centralit.com.br">ciro.junior@centralit.com.br</a>)
 * 
 */

@Service("tarjaAssinaturaService")
public class TarjaAssinaturaServiceImpl extends GenericServiceImpl<TarjaAssinatura, Long> implements TarjaAssinaturaService {

	private static final Logger LOG = Logger.getLogger(TarjaAssinaturaServiceImpl.class);

	@Autowired
	private OrganizacaoService organizacaoService;

	@Autowired
	private AnexoImagemService anexoImagemService;

	private TarjaAssinaturaDao tarjaAssinaturaDao;

	/** Atributo assinaturaUtilService. */
	@Autowired
	private AssinaturaUtilService assinaturaUtilService;

	@Autowired
	private DominioService dominioService;

	@Value("${ecm.doc.key.decrypted.document}")
	public String keyDocument;

	@Autowired
	public TarjaAssinaturaServiceImpl( TarjaAssinaturaDao tarjaAssinaturaDao, @Qualifier("tarjaAssinaturaValidator") Validator validator ) {

		this.dao = tarjaAssinaturaDao;
		this.tarjaAssinaturaDao = tarjaAssinaturaDao;
		this.validator = validator;
	}

	public TarjaAssinatura save(TarjaAssinatura tarjaAssinatura) {

		return this.merge(tarjaAssinatura);
	}

	@Override
	public TarjaAssinatura merge(TarjaAssinatura tarjaAssinatura) {

		this.validarEntidade(tarjaAssinatura, this.validator);
		return super.merge(tarjaAssinatura);
	}

	@Override
	public TarjaAssinatura removeImagem(TarjaAssinatura tarjaAssinatura) {

		if (tarjaAssinatura.getAnexoImagem() != null) {
			anexoImagemService.removeById(tarjaAssinatura.getAnexoImagem().getId());
			tarjaAssinatura.setAnexoImagem(null);
		}
		return this.merge(tarjaAssinatura);
	}

	/** {@inheritDoc} */
	@Override
	public void uploadAnexoImagemTarja(MultipartFile file, Long idOrganizacao, Long idTarjaAssinatura) {

		AnexoImagem anexoImagem = new AnexoImagem();
		try {
			Organizacao organizacao = (Organizacao) this.organizacaoService.find(idOrganizacao);
			anexoImagem.setAnexo(file.getBytes());
			anexoImagem.setDescricao(file.getOriginalFilename());
			anexoImagem.setOrganizacao(organizacao);

			AnexoImagem anexoImagemSaved = anexoImagemService.save(anexoImagem);
			TarjaAssinatura tarjaAssinatura = (TarjaAssinatura) this.tarjaAssinaturaDao.find(idTarjaAssinatura);
			tarjaAssinatura.setAnexoImagem(anexoImagemSaved);
			this.merge(tarjaAssinatura);
		} catch (IOException ioe) {
			LOG.error("N\u00e3o consegui anexar a imagem \u00e0 entidade TarjaAssinatura. Verifique a exce\u00e7\u00e3o para " + "identificar a causa: ", ioe.getCause());
		}
	}

	/**
	 * Método responsável por obter tarja assinatura interna
	 * 
	 * @author rogerio.cassimiro
	 * @return {@link TarjaAssinatura}
	 */
	@Override
	public TarjaAssinatura getTarjaAssinaturaInterna() {

		Dominio dominio = dominioService.findByChaveAndCodigo(Dominio.TIPO_ASSINATURA, Dominio.TIPO_ASSINATURA_INTERNA);
		return this.tarjaAssinaturaDao.getTarjaAssinaturaInterna(dominio);
	}

	/**
	 * Método responsável por gerar assinatura interna por usuário e por documento
	 * 
	 * @author rogerio.cassimiro
	 * @param assinatura
	 * @param formaCriacao
	 */
	@Override
	public void gerarAssinaturaInternaUsuario(Assinatura assinatura, Long formaCriacao) {

		TarjaAssinatura tarjaAssinatura = (TarjaAssinatura) tarjaAssinaturaDao.getReference(assinatura.getConteudoTarja().getIdTarjaAssinatura());
		String conteudoAssinatura = getConteudoTarja(assinatura, tarjaAssinatura.getConteudo(), tarjaAssinatura);

		gerarAssinaturaDocAnexo(assinatura, tarjaAssinatura.getAnexoImagem(), conteudoAssinatura, formaCriacao);
		if (formaCriacao == 1)
			gerarAssinaturaDocOnline(assinatura, tarjaAssinatura.getAnexoImagem(), conteudoAssinatura);
	}

	/**
	 * Método responsável por montar o conteúdo substituindo as chaves
	 * 
	 * @author rogerio.cassimiro
	 * @param assinatura
	 * @param conteudoAssinatura
	 * @return {@link String}
	 */
	private String getConteudoTarja(Assinatura assinatura, String conteudoAssinatura, TarjaAssinatura tarjaAssinatura) {

		conteudoAssinatura = conteudoAssinatura.replaceAll(TarjaAssinatura.NOME_ASSINANTE, assinatura.getPessoa().getNome()).replaceAll(TarjaAssinatura.FUNCAO_ASSINANTE, assinatura.getFuncao().getNome()).replaceAll(TarjaAssinatura.DATA_ASSINATURA, UtilDate.getDataSemHorasString(Calendar.getInstance().getTime())).replaceAll(TarjaAssinatura.HORA_ASSINATURA, UtilDate.getHoraAtualHHMMSS()).replaceAll(TarjaAssinatura.LOGO, getStringBase64(tarjaAssinatura));
		return conteudoAssinatura;
	}

	/**
	 * Método responsável por obter string em formato Base64 para apresentação no html
	 * 
	 * @author rogerio.cassimiro
	 * @param assinatura
	 * @return {@link String}
	 */
	private String getStringBase64(TarjaAssinatura tarjaAssinatura) {

		StringBuilder imgBase64 = new StringBuilder("");
		if (tarjaAssinatura.getConteudo().contains("@logo@") && UtilObjeto.isReferencia(tarjaAssinatura.getAnexoImagem()) && UtilObjeto.isReferencia(tarjaAssinatura.getAnexoImagem().getAnexo())) {
			imgBase64.append("<img src='data:image/png;base64, ");
			imgBase64.append(UtilBase64.byteToBase64(tarjaAssinatura.getAnexoImagem().getAnexo()));
			imgBase64.append("' height='60' width='60' />");
		}
		return imgBase64.toString();
	}

	/**
	 * Método responsável por gerar assinatura em documento anexo
	 * 
	 * @author rogerio.cassimiro
	 * @param assinatura
	 * @param tarjaAssinatura
	 * @param conteudoAssinatura
	 * @throws DocumentException
	 * @throws IOException
	 */

	private void gerarAssinaturaDocAnexo(Assinatura assinatura, AnexoImagem anexoImagem, String conteudoAssinatura, Long formaCriacao) {

		if(formaCriacao == 1){
			assinatura.getConteudoTarja().setConteudoCriptografado(this.encrypted(conteudoAssinatura));
			assinatura.getConteudoTarja().setAnexoImagem(anexoImagem);
			
		} else if(formaCriacao == 2) {
			try {
				conteudoAssinatura = conteudoAssinatura.replaceAll(TarjaAssinatura.CODIGO_ASSINATURA, "<br /><b>".concat(this.assinaturaUtilService.gerarHash(conteudoAssinatura)).concat("</b>"));
				assinatura.getConteudoTarja().setConteudoCriptografado(this.encrypted(conteudoAssinatura));
				assinatura.getConteudoTarja().setAnexoImagem(anexoImagem);
			} catch (Exception e) {
				e.printStackTrace();
			}

			StringBuilder conteudoOriginalDocAnexo = new StringBuilder();
			if (UtilObjeto.isReferencia(assinatura.getDocumentoGed().getConteudoTarja())) {
				conteudoOriginalDocAnexo.append(decrypted(assinatura.getDocumentoGed().getConteudoTarja().getConteudoCriptografado()));
			} else {
				assinatura.getDocumentoGed().setConteudoTarja(new ConteudoTarja());
			}
			conteudoOriginalDocAnexo.append(conteudoAssinatura);
			
			assinatura.getDocumentoGed().getConteudoTarja().setConteudoCriptografado(this.encrypted(conteudoOriginalDocAnexo.toString()));
		}
	}

	/**
	 * Método responsável por gerar assinatura em documento online
	 * 
	 * @author rogerio.cassimiro
	 * @param assinatura
	 * @param tarjaAssinatura
	 * @param conteudoAssinatura
	 */
	private void gerarAssinaturaDocOnline(Assinatura assinatura, AnexoImagem anexoImagem, String conteudoAssinatura) {

		StringBuilder conteudoOriginalDocOnLine = new StringBuilder();
		
		if (UtilObjeto.isReferencia(assinatura.getDocumentoGed().getConteudoTarja())) {
			conteudoOriginalDocOnLine.append(decrypted(assinatura.getDocumentoGed().getConteudoTarja().getConteudoCriptografado()));
		} else {
			assinatura.getDocumentoGed().setConteudoTarja(new ConteudoTarja());
			conteudoOriginalDocOnLine.append(decrypted(assinatura.getDocumentoGed().getConteudoCriptografado()));
		}
		conteudoOriginalDocOnLine.append("<br />").append(conteudoAssinatura);
		try {
			// Assinar documento
			assinatura.getDocumentoGed().setConteudoAssinado(this.assinaturaUtilService.geraAssinatura(conteudoOriginalDocOnLine.toString().getBytes()));

			String conteudo = conteudoOriginalDocOnLine.toString().replaceAll(TarjaAssinatura.CODIGO_ASSINATURA, "<br /><b>".concat(this.assinaturaUtilService.gerarHash(conteudoOriginalDocOnLine.toString()).concat("</b>")));

			assinatura.getDocumentoGed().getConteudoTarja().setConteudoCriptografado(this.encrypted(conteudo));

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Método responsável por criptografar o texto através da chave
	 * 
	 * @author rogerio.costa
	 * @param text
	 * @return {@link Byte}
	 */
	@Override
	public byte[] encrypted(String text) {

		try {
			byte[] encrypted = null;
			Key aesKey = new SecretKeySpec(this.keyDocument.getBytes(), "AES");
			Cipher cipher = Cipher.getInstance("AES");
			cipher.init(Cipher.ENCRYPT_MODE, aesKey);
			encrypted = cipher.doFinal(text.getBytes());

			return encrypted;

		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Método responsável por descriptografar texto
	 * 
	 * @author rogerio.cassimiro
	 * @param text
	 * @return {@link String}
	 */
	@Override
	public String decrypted(byte[] text) {

		try {
			Cipher cipher = Cipher.getInstance("AES");
			Key aesKey = new SecretKeySpec(this.keyDocument.getBytes(), "AES");

			cipher.init(Cipher.DECRYPT_MODE, aesKey);
			String decrypted = new String(cipher.doFinal(text));

			return decrypted;

		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
}
