package br.com.centralit.api.service.impl;

import java.util.Collection;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.TipoDocumentoDao;
import br.com.centralit.api.model.NivelAcessoTipoDocumento;
import br.com.centralit.api.model.SigiloTipoDocumento;
import br.com.centralit.api.model.TipoDocumento;
import br.com.centralit.api.model.TipoDocumentoPlanoClassificacao;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.NivelAcessoTipoDocumentoService;
import br.com.centralit.api.service.TipoDocumentoService;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilObjeto;

@Service("tipoDocumentoService")
public class TipoDocumentoServiceImpl extends GenericServiceImpl<TipoDocumento, Long> implements TipoDocumentoService {

	private static final String VALIDACAO_UNICIDADE_TIPO_DOCUMENTO = "ECM.VALIDACAO.UNICIDADE_TIPO_DOCUMENTO";

	private TipoDocumentoDao tipoDocumentoDao;

	@Autowired
	private NivelAcessoTipoDocumentoService nivelAcessoTipoDocumentoService;

	/** Atributo dominioService. */
	@Autowired
	private DominioService dominioService;

	@Autowired
	public TipoDocumentoServiceImpl( TipoDocumentoDao tipoDocumentoDao, @Qualifier("tipoDocumentoValidator") Validator validator ) {

		this.dao = tipoDocumentoDao;
		this.tipoDocumentoDao = tipoDocumentoDao;
		this.validator = validator;
	}

	@Override
	public TipoDocumento save(TipoDocumento tipoDocumento) {

		this.validarUnicidade(tipoDocumento);

		this.montaObjeto(tipoDocumento);

		return super.merge(tipoDocumento);
	}

	@Override
	public TipoDocumento merge(TipoDocumento entity) {

		this.validarUnicidade(entity);

		this.montaObjeto(entity);

		this.nivelAcessoTipoDocumentoService.removeNivelAcesso(entity);

		return super.merge(entity);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author maycon.silva
	 * 
	 * @param tipoDocumento
	 */
	private void montaObjeto(TipoDocumento tipoDocumento) {

		if (!UtilColecao.isVazio(tipoDocumento.getTipoDocumentoPlanoClassificacoes())) {
			for (TipoDocumentoPlanoClassificacao tipoDocumentoPlanoClassificacaoItem : tipoDocumento.getTipoDocumentoPlanoClassificacoes()) {
				tipoDocumentoPlanoClassificacaoItem.setTipoDocumento(tipoDocumento);
			}
		}

		if (!UtilColecao.isVazio(tipoDocumento.getAcessos())) {

			for (NivelAcessoTipoDocumento nivelAcessoTipoDocumento : tipoDocumento.getAcessos()) {

				nivelAcessoTipoDocumento.setTipoDocumento(tipoDocumento);

				if (!UtilColecao.isVazio(nivelAcessoTipoDocumento.getSigilos())) {

					for (SigiloTipoDocumento sigilo : nivelAcessoTipoDocumento.getSigilos()) {

						sigilo.setNivelAcessoTipoDocumento(nivelAcessoTipoDocumento);

					}

				}
			}

		}

	}

	@Override
	public boolean verificaDuplicidadeNomeDocumento(String nome) {

		Collection<TipoDocumento> listaTipoDocumento = tipoDocumentoDao.getTipoDocumentoByNome(nome);
		if (listaTipoDocumento != null && !listaTipoDocumento.isEmpty() && listaTipoDocumento.size() > 0) {
			return true;
		}
		return false;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por validar a unidade do documento
	 * 
	 * @author rogerio.costa
	 * 
	 * @param pessoaFisica
	 */
	public void validarUnicidade(TipoDocumento tipoDocumento) {

		TipoDocumento tipoDocumentoPersistido = this.tipoDocumentoDao.obterPorNome(tipoDocumento.getNome());

		if (UtilObjeto.isReferencia(tipoDocumentoPersistido) && ( !UtilObjeto.isReferencia(tipoDocumento.getId()) || UtilObjeto.isReferencia(tipoDocumento.getId()) && !tipoDocumento.getId().equals(tipoDocumentoPersistido.getId()) )) {

			throw new BusinessException(VALIDACAO_UNICIDADE_TIPO_DOCUMENTO, CodigoErro.REGRA_NEGOCIO.getValue());
		}
	}
}
