package br.com.centralit.api.service.impl;

import java.util.Collection;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.TipoProcessoDao;
import br.com.centralit.api.model.NivelAcessoTipoProcesso;
import br.com.centralit.api.model.SigiloTipoProcesso;
import br.com.centralit.api.model.TipoProcesso;
import br.com.centralit.api.model.TipoProcessoPlanoClassificacao;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.NivelAcessoTipoProcessoService;
import br.com.centralit.api.service.TipoProcessoService;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;

@Service("tipoProcessoService")
public class TipoProcessoServiceImpl extends GenericServiceImpl<TipoProcesso, Long> implements TipoProcessoService {

	private TipoProcessoDao tipoProcessoDao;

	/** Atributo nivelAcessoTipoProcessoService. */
	@Autowired
	private NivelAcessoTipoProcessoService nivelAcessoTipoProcessoService;

	/** Atributo dominioService. */
	@Autowired
	private DominioService dominioService;

	@Autowired
	public TipoProcessoServiceImpl( TipoProcessoDao tipoProcessoDao, @Qualifier("tipoProcessoValidator") Validator validator ) {

		this.dao = tipoProcessoDao;
		this.tipoProcessoDao = tipoProcessoDao;
		this.validator = validator;
	}

	@Override
	public TipoProcesso save(TipoProcesso tipoProcesso) {

		this.montaObjeto(tipoProcesso);
		this.validarEntidade(tipoProcesso, this.validator);

		return super.save(tipoProcesso);
	}

	@Override
	public TipoProcesso merge(TipoProcesso tipoProcesso) {

		this.montaObjeto(tipoProcesso);

		this.validarEntidade(tipoProcesso, this.validator);

		this.nivelAcessoTipoProcessoService.removeNivelAcesso(tipoProcesso);

		return super.merge(tipoProcesso);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author maycon.silva
	 * 
	 * @param tipoProcesso
	 */
	private void montaObjeto(TipoProcesso tipoProcesso) {

		if (!UtilColecao.isVazio(tipoProcesso.getTipoProcessoPlanoClassificacoes())) {
			for (TipoProcessoPlanoClassificacao tipoProcessoPlanoClassificacaoItem : tipoProcesso.getTipoProcessoPlanoClassificacoes()) {
				tipoProcessoPlanoClassificacaoItem.setTipoProcesso(tipoProcesso);

			}
		}

		if (!UtilColecao.isVazio(tipoProcesso.getAcessos())) {

			for (NivelAcessoTipoProcesso nivelAcessoTipoProcesso : tipoProcesso.getAcessos()) {

				nivelAcessoTipoProcesso.setTipoProcesso(tipoProcesso);

				if (!UtilColecao.isVazio(nivelAcessoTipoProcesso.getSigilos())) {

					for (SigiloTipoProcesso sigilo : nivelAcessoTipoProcesso.getSigilos()) {

						sigilo.setNivelAcessoTipoProcesso(nivelAcessoTipoProcesso);

					}

				}
			}

		}

	}

	public boolean verificaDuplicidadeNomeDocumento(String nome) {

		Collection<TipoProcesso> listaTipoProcesso = tipoProcessoDao.getTipoProcessoByNome(nome);
		if (listaTipoProcesso != null && !listaTipoProcesso.isEmpty() && listaTipoProcesso.size() > 0) {
			return true;
		}
		return false;

	}

}
