package br.com.centralit.api.service.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import br.com.centralit.api.dao.UnidadeProcessoDao;
import br.com.centralit.api.model.Processo;
import br.com.centralit.api.model.UnidadeProcesso;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.GrupoService;
import br.com.centralit.api.service.GrupoUsuarioService;
import br.com.centralit.api.service.HistoricoAlteracaoProcessoService;
import br.com.centralit.api.service.HistoricoAtribuicaoProcessoService;
import br.com.centralit.api.service.NotificacaoService;
import br.com.centralit.api.service.NotificacaoUsuarioService;
import br.com.centralit.api.service.ProcessoService;
import br.com.centralit.api.service.UnidadeProcessoService;
import br.com.centralit.api.service.UnidadeService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.api.viewHelper.EnviarProcessoVH;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Unidade;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilObjeto;

import com.googlecode.genericdao.search.ISearch;
import com.googlecode.genericdao.search.SearchResult;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 05/01/2016 - 14:55:51
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
@Service("unidadeProcessoService")
public class UnidadeProcessoServiceImpl extends GenericServiceImpl<UnidadeProcesso, Long> implements UnidadeProcessoService {

	/** Atributo unidadeProcessoDao. */
	private UnidadeProcessoDao unidadeProcessoDao;

	/** Atributo processoService. */
	@Autowired
	private ProcessoService processoService;

	/** Atributo grupoUsuarioService. */
	@Autowired
	private GrupoUsuarioService grupoUsuarioService;

	/** Atributo dominioService. */
	@Autowired
	private DominioService dominioService;

	/** Atributo notificacaoService. */
	@Autowired
	private NotificacaoService notificacaoService;

	/** Atributo notificacaoUsuarioService. */
	@Autowired
	private NotificacaoUsuarioService notificacaoUsuarioService;

	/** Atributo grupoService. */
	@Autowired
	private GrupoService grupoService;

	@Autowired
	/** Atributo unidadeService. */
	private UnidadeService unidadeService;

	/** Atributo usuarioService. */
	@Autowired
	private UsuarioService usuarioService;

	/** Atributo historicoAtribuicaoProcessoService. */
	@Autowired
	private HistoricoAtribuicaoProcessoService historicoAtribuicaoProcessoService;
	
	@Autowired
	private HistoricoAlteracaoProcessoService  historicoAlteracaoProcessoService;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * 
	 * @param unidadeProcessoDao
	 */
	@Autowired
	public UnidadeProcessoServiceImpl( UnidadeProcessoDao unidadeProcessoDao ) {

		this.dao = unidadeProcessoDao;

		this.unidadeProcessoDao = unidadeProcessoDao;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por enviar processo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param enviarProcessoVH
	 */
	@Override
	public EnviarProcessoVH save(EnviarProcessoVH enviarProcessoVH) {

		Processo processo = this.processoService.getReference(enviarProcessoVH.getIdProcesso());

		if (!UtilColecao.isVazio(enviarProcessoVH.getUnidadesProcessoList())) {

			List<String> groups = new ArrayList<String>();

			for (UnidadeProcesso unidadeProcesso : enviarProcessoVH.getUnidadesProcessoList()) {

				unidadeProcesso.setPrimeiroAcesso(Boolean.TRUE);
				unidadeProcesso.setProcesso(processo);
				// Adiciona os grupos para qual sera enviado.
				groups.add(this.grupoService.getReference(unidadeProcesso.getUnidade().getGrupo().getId()).getSigla());

			}

			// Verifica se a opção concluir na unidade foi selecionada
			if (UtilObjeto.isReferencia(enviarProcessoVH.getConcluirProcesso()) && enviarProcessoVH.getConcluirProcesso()) {

				this.executarConcluirProcessoNaUnidade(enviarProcessoVH.getIdTask(), processo);

			}

			enviarProcessoVH.setUnidadesProcessoList(this.saveList(enviarProcessoVH.getUnidadesProcessoList()));
			// Enviar o processo para os grupos padrão das unidades.
			this.delegateUserTask(enviarProcessoVH.getIdTask(), groups, null);
		}

		return enviarProcessoVH;
	}

	@Override
	public UnidadeProcesso merge(UnidadeProcesso entity) {

		if (UtilObjeto.isReferencia(entity.getUsuarioResponsavel())) {

			entity.setUsuarioResponsavel(this.usuarioService.getReference(entity.getUsuarioResponsavel().getId()));

			this.historicoAtribuicaoProcessoService.saveHistorico(entity);

		}
		return super.merge(entity);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por salvar unidadeProcesso
	 * 
	 * @author rogerio.costa
	 * 
	 * @param processo
	 */
	public void saveUnidadeProcesso(Processo processo) {

		UnidadeProcesso unidadeProcesso = new UnidadeProcesso();
		unidadeProcesso.setProcesso(processo);
		unidadeProcesso.setPrimeiroAcesso(Boolean.FALSE);
		unidadeProcesso.setUnidade(this.unidadeService.getReference(this.getUsuario().getUnidade().getId()));
		unidadeProcesso.setUsuarioResponsavel(this.getUsuario());

		unidadeProcesso = super.save(unidadeProcesso);
		
		historicoAlteracaoProcessoService.gerarHistoricoAtribuicao(unidadeProcesso);
		
		this.historicoAtribuicaoProcessoService.saveHistorico(unidadeProcesso);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por concluir o processo na unidade. A conclusão do processo na unidade é feita através de um registro na tabela <code>UnidadeProcesso</code>
	 * 
	 * 
	 * @author rogerio.costa
	 * 
	 * @param enviarProcessoVH
	 * @param processo
	 */
	public void executarConcluirProcessoNaUnidade(Long idTask, Processo processo) {

		UnidadeProcesso unidadeProcesso = this.unidadeProcessoDao.obterPorUnidadeEProcessoEmExecucao(processo.getId(), this.getUsuario().getUnidade().getId());

		if (UtilObjeto.isReferencia(unidadeProcesso)) {

			unidadeProcesso.setConcluido(Boolean.TRUE);

			List<String> grupos = new ArrayList<String>();

			List<String> users = new ArrayList<String>();

			users.add(processo.getAutor().getUsername());

			Unidade unidade = this.unidadeService.getReference(this.getUsuario().getUnidade().getId());

			grupos.add(unidade.getGrupo().getSigla());

			this.removeUserTaskDelegation(idTask, grupos, null);

			this.removeUserTaskDelegation(idTask, null, users);

			this.save(unidadeProcesso);
		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idProcesso
	 * @return
	 */
	@Override
	public Collection<UnidadeProcesso> findByIdProcesso(Long idProcesso) {

		return this.unidadeProcessoDao.findByIdProcesso(idProcesso);
	}
	
	/**
	 *
	 */
	@Override
	public Collection<UnidadeProcesso> findByPorIdProcessoEmExecucao(Long idProcesso) {

		return this.unidadeProcessoDao.findByPorIdProcessoEmExecucao(idProcesso);
	}
	
	

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar se o processo foi concluido em todas as unidades
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idProcesso
	 * 
	 * @return Boolean
	 */
	public Boolean isProcessoConcluidoEmTodasUnidades(Long idProcesso) {

		return this.unidadeProcessoDao.isProcessoConcluidoEmTodasUnidades(idProcesso);
	}

	/**
	 * 
	 */
	@Override
	public Collection<UnidadeProcesso> findProcesso(Processo processo) {

		return this.unidadeProcessoDao.findProcesso(processo);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter através da unidade e processo
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idProcesso
	 * @param idUnidade
	 * @return UnidadeProcesso
	 */
	public UnidadeProcesso obterPorUnidadeEProcesso(Long idProcesso, Long idUnidade) {

		return this.unidadeProcessoDao.obterPorUnidadeEProcesso(idProcesso, idUnidade);
	}
	
	/**
	 *
	 */
	public UnidadeProcesso obterPorUnidadeEProcessoEmExecucao(Long idProcesso, Long idUnidade){
		
		return this.unidadeProcessoDao.obterPorUnidadeEProcessoEmExecucao(idProcesso, idUnidade);
		
	}

	/**
	 * Método responsável por listar processos através da unidade do usuário
	 * 
	 * @author rogerio.cassimiro
	 * @param search
	 * @return <RT> SearchResult<RT>
	 */
	@Override
	public <RT> SearchResult<RT> searchAndCountPorUnidade(ISearch search) {

		Dominio dominioStatusProcesso = dominioService.findByChaveAndCodigo(Dominio.TIPO_STATUS_PROCESSO_ECM, Dominio.TIPO_STATUS_PROCESSO_ECM_CONCLUIDO);
		Usuario usuario = ( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() );
		return this.unidadeProcessoDao.searchAndCountPorUnidade(search, usuario, dominioStatusProcesso.getId());
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar se o processo foi enviado para outra unidade
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idProcesso
	 * 
	 * @return Boolean
	 */
	public Boolean isProcessoEnviado(Long idProcesso) {

		return this.unidadeProcessoDao.isProcessoEnviado(idProcesso);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar se o processo pertence a unidade
	 * 
	 * @author rogerio.costa
	 * 
	 * @param idProcesso
	 * @param idUnidade
	 * @return boolean
	 */
	public boolean isProcessoNaUnidadeUsuarioLogado(Long idProcesso) {

		if (UtilObjeto.isReferencia(this.getUsuario().getUnidade())) {

			return this.unidadeProcessoDao.isProcessoNaUnidadeUsuarioLogado(idProcesso, this.getUsuario().getUnidade().getId());
		}

		return Boolean.FALSE;

	}

}
