package br.com.centralit.controller;

import java.util.Collection;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import br.com.centralit.api.framework.json.ViewsEcm;
import br.com.centralit.api.model.HistoricoAlteracaoProcesso;
import br.com.centralit.api.service.HistoricoAlteracaoProcessoService;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.json.ResponseBodyWrapper;

@Controller
@RequestMapping("/rest/historicoAlteracaoProcesso")
public class HistoricoAlteracaoProcessoController extends GenericController<HistoricoAlteracaoProcesso> {

	/** Atributo attachedFileService. */
	private HistoricoAlteracaoProcessoService historicoAlteracaoProcessoService;

	public HistoricoAlteracaoProcessoController() {

		super();
	}

	@Autowired
	public HistoricoAlteracaoProcessoController( HistoricoAlteracaoProcessoService historicoAlteracaoProcessoService ) {

		super(historicoAlteracaoProcessoService);
		this.historicoAlteracaoProcessoService = historicoAlteracaoProcessoService;
	}

	@RequestMapping(value = "/saveParams", produces = "application/json", method = RequestMethod.POST)
	@ResponseBody
	public ResponseBodyWrapper saveWithParams(@RequestBody HistoricoAlteracaoProcesso historico) {

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(this.historicoAlteracaoProcessoService.saveWithParams(historico),
				ViewsEcm.HistoricoProcessoView.class);

		return responseBody;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por buscar historicos por processo e paginacao
	 * 
	 * @author halan.lima
	 * 
	 * @param idProcesso
	 * @param inicio
	 * @param max
	 * @return
	 */
	@RequestMapping(value = "/findByProcessoPaginado", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findByProcessoPaginado(@RequestParam(value = "idProcesso") String idProcesso,
			@RequestParam(value = "inicio") String inicio, @RequestParam(value = "max") String max) {

		final Collection<HistoricoAlteracaoProcesso> listaTipoProcessoClassificacao = this.historicoAlteracaoProcessoService
				.findByProcessoPaginado(Long.parseLong(idProcesso), Integer.valueOf(inicio), Integer.valueOf(max));

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(listaTipoProcessoClassificacao, ViewsEcm.HistoricoProcessoView.class);

		return responseBody;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por contar o numero de historicos alteracao por processo
	 * 
	 * @author halan.lima
	 * 
	 * @param idProcesso
	 * @return
	 */
	@RequestMapping(value = "/countByProcesso", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper countByProcesso(@RequestParam(value = "idProcesso") String idProcesso) {

		Integer toReturn = this.historicoAlteracaoProcessoService.countByProcesso(Long.valueOf(idProcesso));
		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(toReturn, ViewsEcm.HistoricoProcessoView.class);

		return responseBody;
	}

	@Override
	public Class<ViewsEcm.HistoricoProcessoView> getListView() {

		return ViewsEcm.HistoricoProcessoView.class;
	}
}
