package br.com.centralit.listener;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationListener;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.stereotype.Component;

import br.com.centralit.api.model.AcaoHistoricoAlteracaoProcesso;
import br.com.centralit.api.model.Sigilo;
import br.com.centralit.api.model.SigiloPrivilegio;
import br.com.centralit.api.service.AcaoHistoricoAlteracaoProcessoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.ModuloService;
import br.com.centralit.api.service.PrivilegioService;
import br.com.centralit.api.service.SigiloPrivilegioService;
import br.com.centralit.api.service.SigiloService;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Internacionalizacao;
import br.com.centralit.framework.model.Modulo;
import br.com.centralit.framework.model.Privilegio;
import br.com.centralit.framework.model.UtilStartup;

@Component
public class StartupListenerEcm extends UtilStartup implements ApplicationListener<ContextRefreshedEvent> {

	@Autowired
	private AcaoHistoricoAlteracaoProcessoService acaoHistoricoAlteracaoProcessoService;

	/** Atributo dominioService. */
	@Autowired
	private DominioService dominioService;

	/** Atributo sigiloService. */
	@Autowired
	private SigiloService sigiloService;

	/** Atributo privilegioService. */
	@Autowired
	private PrivilegioService privilegioService;

	/** Atributo sigiloPrivilegioService. */
	@Autowired
	private SigiloPrivilegioService sigiloPrivilegioService;

	@Autowired
	InternacionalizacaoService internacionalizacaoService;

	@Autowired
	ModuloService moduloService;

	@Override
	public void onApplicationEvent(ContextRefreshedEvent event) {

		this.gerarInternacionalizacao();
	}

	@PostConstruct
	public void init() {

		this.popularAcoesAlteracaoProcesso();

		this.criarDominios();

		this.criarRegraDeAcesso();

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por criar as regras de acesso
	 * 
	 * @author rogerio.costa
	 * 
	 */
	private void criarRegraDeAcesso() {

		this.privilegioService.saveIfNotExist(new Privilegio("PROTOCOLO", "Acessa processo finalizados"));
		
		Privilegio privilegioReservado = this.privilegioService.saveIfNotExist(new Privilegio("RESERVADO", "Acessa o sigilo reservado"));
		Privilegio privilegioSecreto = this.privilegioService.saveIfNotExist(new Privilegio("SECRETO", "Acessa o sigilo secreto"));
		Privilegio privilegioUltrassecreto = this.privilegioService.saveIfNotExist(new Privilegio("ULTRASSECRETO", "Acessa o sigilo Ultrassecreto"));
		
		Dominio dominioReservado = this.dominioService.findByChaveAndCodigo("tipoSigilo", 0L);
		Dominio dominioSecreto = this.dominioService.findByChaveAndCodigo("tipoSigilo", 1L);
		Dominio dominioUltrassecreto = this.dominioService.findByChaveAndCodigo("tipoSigilo", 2L);

		Sigilo sigiloReservado = this.sigiloService.saveIfNotExist(new Sigilo(dominioReservado));
		Sigilo sigiloSecreto = this.sigiloService.saveIfNotExist(new Sigilo(dominioSecreto));
		Sigilo sigiloUltrassecreto = this.sigiloService.saveIfNotExist(new Sigilo(dominioUltrassecreto));

		this.sigiloPrivilegioService.saveIfNotExist(new SigiloPrivilegio(privilegioReservado, sigiloReservado));
		this.sigiloPrivilegioService.saveIfNotExist(new SigiloPrivilegio(privilegioSecreto, sigiloSecreto));
		this.sigiloPrivilegioService.saveIfNotExist(new SigiloPrivilegio(privilegioUltrassecreto, sigiloUltrassecreto));
	}

	/**
	 * Método responsável por criar domínios específicos ECM
	 * 
	 * @author halan.lima
	 * 
	 */
	protected void popularAcoesAlteracaoProcesso() {

		List<AcaoHistoricoAlteracaoProcesso> list = new ArrayList<AcaoHistoricoAlteracaoProcesso>();

		list.add(new AcaoHistoricoAlteracaoProcesso("INCLUSAO_PROCESSO", "Inclusão do processo", "Processo <<protocolo>> criado."));
		list.add(new AcaoHistoricoAlteracaoProcesso("INCLUSAO_DOCUMENTO", "Inclusão de documento no processo", "Documento <<tipo_documento>> <<numero_documento>> incluído no processo."));
		list.add(new AcaoHistoricoAlteracaoProcesso("REMOCAO_DOCUMENTO", "Remoção de documento no processo", "Documento <<tipo_documento>> <<numero_documento>> removido do processo."));
		list.add(new AcaoHistoricoAlteracaoProcesso("ALTERACAO_METADADO_PROCESSO", "Alteração dos metadados do processo",
		// TODO valor temporario para alteracao de metadados
				"Processo alterado"));
		// TODO valor correto para alteracao de metadados:
		// "Processo alterado: <<nome_atributo>> foi modificado de <<valor_antigo>> para <<valor_novo>>"
		list.add(new AcaoHistoricoAlteracaoProcesso("RELACIONAR_PROCESSO", "Relacionar Processo", "O relacionamento com o processo <<protocolo>> foi adicionado."));
		list.add(new AcaoHistoricoAlteracaoProcesso("REMOVER_RELACIONAMENTO_PROCESSO", "Remover relacionamento Processo", "O relacionamento com o processo <<protocolo>> foi removido."));
		list.add(new AcaoHistoricoAlteracaoProcesso("ANEXAR_PROCESSO", "Anexar Processo", "O processo <<protocolo>> foi anexado a este processo, junto a todos os seus documentos."));
		list.add(new AcaoHistoricoAlteracaoProcesso("REMOVER_PROCESSO_ANEXADO", "Remover Anexo Processo", "O Anexo com o processo <<protocolo>> foi removido."));
		list.add(new AcaoHistoricoAlteracaoProcesso("ANEXAR_PROCESSO_PRINCIPAL", "Anexar Processo principal", "Processo <<protocolo>> anexado a este processo."));
		list.add(new AcaoHistoricoAlteracaoProcesso("ANEXAR_PROCESSO_SECUNDARIO", "Anexar Processo secundário", "Processo anexado ao processo <<protocolo>>."));
		list.add(new AcaoHistoricoAlteracaoProcesso("ATUALIZAR_ANDAMENTO_PROCESSO", "Atualização de Andamento do Processo", "Situação do processo alterada para <<situacao>>."));
		list.add(new AcaoHistoricoAlteracaoProcesso("CRIACAO_NOVA_VERSAO_DOCUMENTO", "Criação de uma nova versão do documento", "Documento <<tipo_documento>> <<numero_documento>> alterado para a versão <<numero_versao>>."));
		list.add(new AcaoHistoricoAlteracaoProcesso("RESTAURACAO_VERSAO_DOCUMENTO", "Restauração de uma versão do documento", "Documento <<tipo_documento>> <<numero_documento>> restaurado para a versão <<numero_versao>>."));
		list.add(new AcaoHistoricoAlteracaoProcesso("CONCLUSAO_PROCESSO_UNIDADE", "Conclusão do Processo na Unidade", "Processo concluído na unidade <<unidade>>."));
		list.add(new AcaoHistoricoAlteracaoProcesso("REABRIR_PROCESSO", "Reabrir Processo", "Processo reaberto na unidade <<unidade>>."));
		list.add(new AcaoHistoricoAlteracaoProcesso("CANCELAR_DOCUMENTO", "Cancelar Documentos", "Documento <<tipo_documento>> <<numero_documento>> cancelado."));
		list.add(new AcaoHistoricoAlteracaoProcesso("ENVIAR_PROCESSO", "Enviar Processo", "O processo foi enviado para a <<unidade>>."));
		list.add(new AcaoHistoricoAlteracaoProcesso("JUSTIFICATIVA_ACAO_PROCESSO", "Justificativa", "<<acao>> , protocolo <<protocolo>>. Justificativa: <<justificativa>>."));
		list.add(new AcaoHistoricoAlteracaoProcesso("ACAO_PROCESSO", "Ação", "<<acao>> , protocolo <<protocolo>>."));
		list.add(new AcaoHistoricoAlteracaoProcesso("VALIDAR_PROCESSO", "Validar Processo", "O processo <<protocolo>> e seus documentos foram validados de acordo com seu nível de acesso."));
		list.add(new AcaoHistoricoAlteracaoProcesso("ATRIBUICAO_PROCESSO", "Atribuição de processo", "O processo <<protocolo>> foi atribuido para <<usuario>>."));
		list.add(new AcaoHistoricoAlteracaoProcesso("ASSINAR_DOCUMENTO", "Assinatura de documento", "O documento <<tipo_documento>> <<numero_documento>> foi assinado por <<usuario>>."));
		list.add(new AcaoHistoricoAlteracaoProcesso("ATUALIZAR_ANDAMENTO_PROCESSO", "Atualização de Andamento do Processo", "Situação do processo alterada para <<situacao>>."));		
		list.add(new AcaoHistoricoAlteracaoProcesso("INICIAR_ATRIBUICAO_PROCESSO", "Atribuição do processo", "Processo atribuído a <<usuario>>."));		
		list.add(new AcaoHistoricoAlteracaoProcesso("ALTERAR_ATRIBUICAO_PROCESSO", "Atribuição do processo alterada", "Processo atribuído a <<usuario>>."));
		list.add(new AcaoHistoricoAlteracaoProcesso("REABERTURA_DE_PROCESSO", "Reabrir Processo", "Processo reaberto na unidade <<unidade>>."));
		
		try {
			this.acaoHistoricoAlteracaoProcessoService.saveListIfNotExist(list);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por incluir dominios
	 * 
	 * @author rogerio.costa
	 * 
	 */
	private void criarDominios() {

		List<Dominio> list = new ArrayList<Dominio>();

		list.add(new Dominio("estadoDocumentoGed", "Minuta", "MINUTA", 1L));
		list.add(new Dominio("estadoDocumentoGed", "Original", "ORIGINAL", 2L));
		list.add(new Dominio("estadoDocumentoGed", "Cópia", "COPIA", 3L));

		list.add(new Dominio("tipoDestinacao", "Guardar Permanente", "GUARDAR_PERMANENTE", 1L));
		list.add(new Dominio("tipoDestinacao", "Eliminação", "ELIMINAR", 2L));

		list.add(new Dominio("tipoTemporalidade", "Dia", "DIA", 1L));
		list.add(new Dominio("tipoTemporalidade", "Mês", "MES", 2L));
		list.add(new Dominio("tipoTemporalidade", "Ano", "ANO", 3L));

		list.add(new Dominio("tipoPlanoClassificacao", "Classe", "CLASSE", 1L));
		list.add(new Dominio("tipoPlanoClassificacao", "SubClasse", "SUB_CLASSE", 2L));
		list.add(new Dominio("tipoPlanoClassificacao", "Grupo", "GRUPO", 3L));
		list.add(new Dominio("tipoPlanoClassificacao", "SubGrupo", "SUB_GRUPO", 4L));

		list.add(new Dominio("tipoNivelAcesso", "Sigiloso", "SIGILOSO", 0L));
		list.add(new Dominio("tipoNivelAcesso", "Restrito", "RESTRITO", 1L));
		list.add(new Dominio("tipoNivelAcesso", "Público", "PUBLICO", 2L));

		list.add(new Dominio("tipoSigilo", "Reservado", "RESERVADO", 0L));
		list.add(new Dominio("tipoSigilo", "Secreto", "SECRETO", 1L));
		list.add(new Dominio("tipoSigilo", "Ultrassecreto", "ULTRASSECRETO", 2L));

		list.add(new Dominio("tipoFormaCriacao", "Online", "ONLINE", 1L));
		list.add(new Dominio("tipoFormaCriacao", "Anexo", "ANEXO", 2L));
		list.add(new Dominio("tipoFormaCriacao", "Ambos", "AMBOS", 3L));

		list.add(new Dominio("tipoProtocoloNup", "Automático", "EXTERNO", 1L, Boolean.FALSE));
		list.add(new Dominio("tipoProtocoloNup", "Informado", "NUP_NOVO", 2L, Boolean.FALSE));
		
		list.add(new Dominio("formaNumeracao", "Sequencial por ano", "SEQUENCIAL_ANO", 1L));
		list.add(new Dominio("formaNumeracao", "Sequencial único", "SEQUENCIAL_UNICO", 2L));
		
		list.add(new Dominio("statusProcesso", "Concluído", "CONCLUIDO", 1L));
		list.add(new Dominio("statusProcesso", "Em andamento", "EM_ANDAMENTO", 2L));
		list.add(new Dominio("statusProcesso", "Aguardando Validação", "AGUARDANDO_VALIDACAO", 3L));
		list.add(new Dominio("statusProcesso", "Validado", "VALIDADO", 4L));
		list.add(new Dominio("statusProcesso", "Anexado", "ANEXADO", 5L));

		list.add(new Dominio("statusDocumentoGed", "Aguardando Validação", "AGUARDANDO_VALIDACAO", 0L));
		list.add(new Dominio("statusDocumentoGed", "Validado", "VALIDADO", 1L));
		
		list.add(new Dominio("filtroPesquisa", "Ambos", "AMBOS", 0L));
		list.add(new Dominio("filtroPesquisa", "Processo", "PROCESSO", 1L));
		list.add(new Dominio("filtroPesquisa", "Documento", "DOCUMENTO", 2L));
		
		list.add(new Dominio("filtroData", "Em qualquer data", "QUAL_QUER_DATA", 0L));
		list.add(new Dominio("filtroData", "Na última hora", "ULTIMA_HORA", 1L));
		list.add(new Dominio("filtroData", "Nas últimas 24 horas", "ULTIMAS_24_HORA", 2L));
		list.add(new Dominio("filtroData", "Na última semana", "ULTIMA_SEMANA", 3L));
		list.add(new Dominio("filtroData", "No último mês", "ULTIMO_MES", 4L));
		list.add(new Dominio("filtroData", "No último ano", "ULTIMO_ANO", 5L));
		list.add(new Dominio("filtroData", "Intervalo personalizado", "INTERVALO_PERSONALIZADO", 6L));

		list.add(new Dominio("tipoAssinatura", "Assinatura interna", "ASSINATURA_INTERNA", 1L));
		list.add(new Dominio("tipoAssinatura", "Assinatura por certificado digital", "ASSINATURA_CERTIFICADO_DIGITAL", 2L));
		
		list.add(new Dominio("statusSolicitaoProcesso", "Em Aberto", "EM_ABERTO", 1L, Boolean.FALSE));
		list.add(new Dominio("statusSolicitaoProcesso", "Aprovada", "APROVADA",  2L, Boolean.FALSE));
		list.add(new Dominio("statusSolicitaoProcesso", "Rejeitada", "REJEITADA",  3L, Boolean.FALSE));

		this.dominioService.saveListIfNotExist(list);

	}

	/**
	 * 
	 * Método responsável por gerar internacionalização do módulo ECM
	 * 
	 * @author rogerio.cassimiro
	 * 
	 */
	private void gerarInternacionalizacao() {

		this.dominioPT_BR = this.dominioService.findByChaveAndCodigo(Dominio.TIPO_IDIOMA, Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
		this.moduloSelecionado = this.moduloService.getModuloPorBaseUrl(UtilStartup.BASE_URL_ECM, null);
		List<Internacionalizacao> internacionalizacaoList = new ArrayList<Internacionalizacao>();

		this.gerarLabel(this.dominioPT_BR, this.moduloSelecionado, internacionalizacaoList);
		this.gerarValidacao(this.dominioPT_BR, this.moduloSelecionado, internacionalizacaoList);
		this.gerarMensagem(this.dominioPT_BR, this.moduloSelecionado, internacionalizacaoList);

		this.internacionalizacaoService.saveListIfNotExist(internacionalizacaoList);
		Logger.getLogger(StartupListenerEcm.class).info("Internacionalização executada - StartupListenerEcm!");
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por criar labels
	 * 
	 * @author Rogério Gomes
	 * 
	 * @param dominio
	 * @param modulo
	 * @param internacionalizacaoList
	 */
	private void gerarLabel(Dominio dominio, Modulo modulo, List<Internacionalizacao> internacionalizacaoList) {

		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.MINIMIZAR", "Minimizar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.VER_MAIS", "Ver mais", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ACAODEPROCESSO", " Ações de Processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ASSINARDOCUMENTO", " Assinar Documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ATRIBUIDO", " Atribuído a", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.CANCELARDOCUMENTO", " Cancelar Documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.CONFIGURACAO_TEMPORALIDADE", "Configuração temporalidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DADOS_HIPOTESELEGAL", "Dados da tabela de hipótese legais", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DADOS_PLANOCLASSIFICACAO", "Dados de plano classificação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DADOS_PROCESSO", "Dados do Processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DADOS_TIPOPROCESSO", "Dados do tipo processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DEFINICOES_ESTRUTURA_ECM", "Definições das estruturas para o ECM", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DESTINATARIO", "Destinatário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DUPLICARPROCESSO", "Duplicar Processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ENVIAR", "Enviar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ENVIAR_EMAIL", "Enviar e-mail", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ENVIARDOCUMENTOPOREMAIL", " Enviar Documento por E-mail", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.EXCLUIR", "Excluir", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.HISTORICO_ALTERACAO_PROCESSO", "Histórico de alteração do processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.HISTORICO_DE_ENVIO", "Histórico de envio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.IMPRIMIRDOCUMENTO", " Imprimir Documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.INCLUIRDOCUMENTO", " Incluir Documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.NOVA_VERSAO", "Nova Versão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.NOVOPROCESSO", " Novo Processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.TIPOPROTOCOLO", "Tipo de Protocolo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.TOTAL_ITENS_HISTORICO", "Total de históricos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.VERSAO_ATUAL", "Versão atual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.VERSAO_DOCUMENTO", "Versões do documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.VERSAO_INTERMEDIARIA", "Versão Intermediária", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.VERSOESDODOCUMENTO", "Versões do Documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ACOMPANHAMENTOESPECIAL", " Acompanhamento Especial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ANEXARPROCESSO", " Anexar Processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ANEXO", "Anexo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ASSUNTO", "Assunto", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ASSUNTOCOMPLEMENTAR", "Assunto Complementar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.AUTOR", "Autor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.CIENCIA", " Ciência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DADOS_DOCUMENTO", "Dados do Documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DADOS_TIPODOCUMENTO", "Dados do Tipo Documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DATACRIACAO", "Criada em", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DATAEDICAO", " Alterado em:", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DOCUMENTOS_PROCESSO", "Documentos do Processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ENVIAR_PROCESSO", "Enviar Processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ENVIARCORRESPONDENCIA", " Enviar Correspondência Eletrônica", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.GERARARQUIVOPROCESSO", " Gerar arquivo do processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.GERENCIARDISPONIBILIZACAOACESSOEXTERNO", "", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.HIPOTESELEGAL", "Hipótese Legal", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.INFORMACAODOPROCESSO", " Informações do Processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.INTERESSADOS", "Interessados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.NIVEL_ACESSO_PERMITIDO", "Níveis de Acesso Permitidos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.NIVELACESSO", "Nivel Acesso", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.OBSERVACAO", "Observação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.PLANOCLASSIFICACAO", "Plano classificação, temporalidade e destinação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.FASE_CORRENTE_ANO", "Fase corrente (Anos)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.FASE_INTERMEDIARIA_ANO", "Fase intermediaria (Anos)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.PERMITIR_USO_CLASSIFICACAO", "Permitir utilizar como classificação?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.PROCESSO", "Processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.PROCESSORELACIONADOS", " Processos Relacionados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.PROTOCOLO", "Protocolo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.SUGESTAODEASSUNTO", "Sugestão de Assuntos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.TIPOPROCESSO", "Tipo de processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.CODIGO_SIORG", "Código SIORG", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.CONCLUIR_UNIDADE", "Concluír na unidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DATA_DOCUMENTO", "Data do documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DESTINACAO", "Destinação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.FORMADECRIACAO", "Forma de Criação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.FUNDAMENTACAO_LEGAL", "Fundamentação legal", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.NIVEL_ACESSO", "Nível Acesso", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.NIVEL_SIGILO", "Nível de sigilo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.NIVEL_SIGILO_ORIGINAL", "Nível de sigilo original", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.PLANO_CLASSIFICACAO_SUPERIOR", "Elemento superior", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.SUGESTOES_ASSUNTO", "Sugestões de assunto", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.TIPODOCUMENTO", "Tipo Documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DADOS_MODELO_DOCUMENTO", "Dados do modelo de documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.CABECALHO", "Cabeçalho", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.RODAPE", "Rodapé", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.CONTEUDO", "Conteúdo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DADOS_RODAPE", "Dados do rodapé", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DADOS_CABECALHO", "Dados do cabeçalho", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.OBSERVACAOGERAL", "Observação geral", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.INTERESSADO", "Interessado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.SOLR_SEARCH_INTERFACE", "Pesquisa Documentos/Processos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.JUSTIFICATIVA", "Justificativa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.SUA_PESQUISA", "Sua pesquisa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.NENHUM_DOCUMENTO_CORRESPONDENTE", "não encontrou nenhum documento correspondente.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.SUGESTOES", "Sugestões:", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.CERTIFIQUE_PALAVRAS_ESCRITAS_CORRETAMENTE", "Certifique-se de que todas as palavras estejam escritas corretamente.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.TENTE_PALAVRAS_CHAVES_DIFERENTE", "Certifique-se de que todas as palavras estejam escritas corretamente.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.TENTE_PALAVRAS_CHAVES_MAIS_GENERICAS", "Tente palavras-chave mais genéricas.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.TENTE_MENOS_PALAVRAS_CHAVES", "Tente usar menos palavras-chave.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DATA_ENVIO", "Data de envio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.SOBRESTAR", "Sobrestar.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.CREDENCIAR_USUARIO_GRUPO", "Credenciar Usuários/Grupos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.PODE_CREDENCIAR", "Pode credenciar outros usuários/grupos?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.GRUPO_USUARIO", "Grupo/Usuário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.GERENCIAR_CREDENCIAIS_ACESSO", "Gerenciar credenciais de acesso", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.USUARIO_GRUPO", "Usuário/Grupo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.HERANCA_SISTEMA", "Herança do Sistema", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.CREDENCIADOR", "Credenciador", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.CREDENCIADO_EM", "Credenciado em", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DATA_DESCREDENCIAMENTO", "Data descredenciamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DESCREDENCIADOR", "Descredenciador", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.HISTORICO_CREDENCIAL", "Histórico de credencial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.OBJETO", "Objeto", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ATRIBUIDOA", "Atribuído a", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.UNIDADE_GERADORA", "Unidade gestora", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.PRAZO_TEMPORALIDADE", "Prazo temporalidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.CLASSIFICACAO_ARQUIVISTICA", "Classificação Arquivística (Assunto)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.CRIADO_EM", "Criado em", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DATA_CONCLUSAO", "Data conclusão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.SIGILO", "Sigilo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.TEXTO", "Texto", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.PADRAO", "Padrão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ATUALIZA_BPE", "Atualiza ESI", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.PROTOCOLO", "Protocolo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.NIVEL_FINAL", "Nível final", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.PRAZO_SIGILO", "Prazo sigilo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ANEXAR_ESTE_PROCESSO", "Anexar a este processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.TEMPLATE", "Template", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.MANTIDO", "Mantido", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DATA_REFERENCIA", "Data de referência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.CONCLUIR", "Concluir", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DOCUMENTOS", "Documentos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.STATUS", "Status", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.SOLICITANTE", "Solicitante", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.PROCESSOS ", "Processos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.REMOVERRELACIONAMENTO", "Remover Relacionamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.PROCESSO_NEGOCIO", "Processo de negócio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.PROPOSTA", "Proposta", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.VALIDAR_NIVEL_ACESSO", "Validar nível de acesso", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.NIVEL_ACESSO_ORIGINAL", "Nível Original", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.NIVEL_PROPOSTO", "Nível Proposto", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.NIVEL_REVISADO", "Nível Revisado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.NUM_DOCUMENTO", "Número Documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.RENUNCIAR_CREDENCIAL", "Renunciar credencial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.APROVAR", "Aprovar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.AVANCAR_FLUXO", "Avançar no Fluxo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.JUSTIFICATIVA_CANCELAMENTO", "Justificativa Cancelado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ATRIBUIR_PROCESSO", "Atribuir processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.CAPTURAR", "Capturar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.HISTORICO_DE_ATRIBUICAO", "Histórico de atribuição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DATA_ATRIBUICAO", "Data da atribuição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ATRIBUIDO_NOME", "Atribuído a", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ATRIBUIDO_POR", "Atribuído por", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.REMOVIDO_POR", "Removido por", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DATA_REMOCAO", "Data remoção", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.FORMA_NUMERACAO", "Forma de numeração", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ADICIONAR_PARAMETRO_NUMERO", "Adicionar parâmetro número", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DADOS_SUPORTE_DOCUMENTO", "Dados do suporte de documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.TIPO_SUPORTE", "Tipo de suporte", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.OUTROS", "Outros", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.NOME_NOVO_SUPORTE ", "Nome do suporte", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.NOVO_TIPO_SUPORTE ", "Novo tipo suporte de documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.TIPO_SUPORTE_DOCUMENTO ", "Tipo suporte de documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.TARJAS_ASSINATURA", "Tarjas de assinatura", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.LOGO_TARJA_ASSINATURA", "Logotipo da tarja de assinatura", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DADOS_TARJA_ASSINATURA", "Dados da tarja de assinatura", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.TIPO_ASSINATURA", "Tipo de assinatura", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.CONTEUDO_HTML", "Conteúdo HTML", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.REMOVER_IMAGEM", "Remover imagem", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.CAMPOS_DINAMICOS", "Campos Dinâmicos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.SOBRESTAR_SEM", "Sobrestar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.SOBRESTADA", "Sobrestada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.SITUACAO_INSTANCIA_TAREFA", "Situação instância tarefa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.NIVEL_DE_ACESSO", "Nível de acesso", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ASSUNTO_COMPLEMENTAR", "Assunto complementar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ASSINATURAS_DOCUMENTO", "Assinaturas do Documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ASSINANTE", "Assinante", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.CARGO_FUNCAO", "Cargo/Função", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.OU", "ou", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.CERTIFICADO_DIGITAL", "Certificado Digital", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ASSINATURA_DOCUMENTO", "Assinatura de Documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ASSINAR", "Assinar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.UNIDADE_GESTORA ", "Unidade gestora", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DATA_CRIACAO ", "Data criação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.PLACEHOLDER_PESQUISA ", "Digite o Nup, tipo de processo, tipo de documento, conteúdo do documento, autor, assunto, destinatário...", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ATE", "Até", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.VALIDADO", "Validado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ESTADO", "Estado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.VALIDAR", "Validar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.NAO_CONTEM_DOCUMENTOS", "Não contém documentos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DATA_DA_ASSINATURA", "Data da assinatura", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.ASSINADO_SUCESSO", "Documento assinado com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.ASSINATURA_DO_DOCUMENTO", "Assinatura do Documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DOCUMENTO_SEM_ASSINATURA", "Documento sem assinaturas!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DADOS_PROCESSO_ANEXADO", "Dados do Processo Anexado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.VISUALIZAR_ASSINATURAS", "Visualizar Assinaturas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.JUSTIFICATIVA_CANCELAMENTO", "Justificativa Cancelado", dominio, modulo));		
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DADOS_REABERTURA", "Dados da Reabertura", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.VISUALIZAR_PROCESSO", "Visualizar Processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.REABRIR", "Executar Solicitacao", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.SOLICITAR_REABERTURA", "Solicitar Reabertura", dominio, modulo));		
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.JUSTIFICATIVAREJEICAO", "Rejeitar Solicitação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.LABEL.DADOS_SOLICITACAO", "Dados da Solicitação", dominio, modulo));
		
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author Rogério Gomes
	 * 
	 * @param dominio
	 * @param modulo
	 * @param internacionalizacaoList
	 */
	private void gerarValidacao(Dominio dominio, Modulo modulo, List<Internacionalizacao> internacionalizacaoList) {

		internacionalizacaoList.add(new Internacionalizacao("ECM.VALIDACAO.VALIDAR_TEMPO_CRIACAO_ANEXO", "Só é possível anexar um processo mais novo a um mais antigo. O(s) seguinte(s) processo(s) não foi(ram) anexado(s):", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.VALIDACAO.UNICIDADE_DOCUMENTO", "Já existe um documento cadastrado com o mesmo número e tipo de documento para este processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.VALIDACAO.REMOCAO_PLANO_CLASSIFICACA_VINCULO", "Não é possível remover pois o mesmo encontra-se classificado em Documento(s) ou Processo(s)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.VALIDACAO.VALIDAR_EXISTENCIA_ANEXO", "Não é possível anexar processos que já estão anexados a outros processos.  O(s) seguinte(s) processo(s) não foi(ram) anexado(s):", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.VALIDACAO.PLANO_CLASSIFICACAO_ULTRAPASSA_NIVEL_PERMITIDO", "Não é permitido cadastrar mais itens deste nível", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.VALIDACAO.ERRO_VINCULO_FLUXO_BPE_ESI", "O processo deverá estar vinculado a um fluxo no Workflow/ESI!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.VALIDACAO.ERRO_UNIDADE_ECM_VIGENTE_USUARIO", "O usuário vigente não possui unidade organizacional ECM vinculada!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.VALIDACAO.ERRO_CONFIGURACAO_GRUPO_PADRAO_USUARIO", "O grupo padrão não foi configurado para unidade vigente", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.VALIDACAO.REMOCAO_COM_VINCULO", "Não é possível excluir pois o mesmo encontra-se em uso por outro cadastro no sistema!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.VALIDACAO.PERMISSAO_OPERACAO", "Você não tem permissão para executar a operação.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.VALIDACAO.GRUPO_PADRAO_EM_OUTRA_UNIDADE", "O grupo selecionado como padrão está sendo usado como grupo padrão da unidade ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.VALIDACAO.VALIDAR_ALTERACAO_NIVEL_ACESSO_OUTRAS_UNIDADES", "Não é possível alterar o nível de acesso para sigiloso, pois o processo foi enviado para outra unidade.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.VALIDACAO.VALIDAR_NIVEL_ACESSO", "Não é possível incluir documento com o nível de acesso menor que o do processo.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.VALIDACAO.SENHA_ASSINATURA", "Informe sua senha para assinar!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.VALIDACAO.SENHA_ASSINATURA_INVALIDA", "Senha do usuário inválida!", dominio, modulo));
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author Rogério Gomes
	 * 
	 * @param dominio
	 * @param modulo
	 * @param internacionalizacaoList
	 */
	private void gerarMensagem(Dominio dominio, Modulo modulo, List<Internacionalizacao> internacionalizacaoList) {

		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.CONFIRMA_CONCLUIR_PROCESSO", "Deseja concluir o processo?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.ITEM_EXCLUIDO", "Item(ns) excluído(s) com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.REMOCAO_DOCUMENTO", "Remoção de documento no processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.ATRIBUICAO", "Atribuição de processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.EDICAO_HIPOTESE_LEGAL", "Não é possível editar pois o mesmo encontra-se em uso por outro cadastro no sistema!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.NOME_TIPO_DOCUMENTO_EM_USO", "Tipo de documento já existe!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.NOME_TIPO_PROCESSO_EM_USO", "Tipo de processo já existe!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.ALTERACAO_METADADO_PROCESSO", "Alteração dos metadados do processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.CRIACAO_NOVA_VERSAO_DOCUMENTO", "Criação de uma nova versão do documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.INCLUSAO_DOCUMENTO", "Inclusão de documento no processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.INCLUSAO_PROCESSO", "Inclusão do processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.RELACIONAR_PROCESSO", "Relacionar Processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.REMOVER_RELACIONAMENTO_PROCESSO", "Remover relacionamento Processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.REMOVER_ANEXO_PROCESSO", "Remover Processo Anexado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.ANEXAR_PROCESSO", "Anexar Processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.ANEXAR_PROCESSO_PRINCIPAL", "Anexar Processo principal", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.ANEXAR_PROCESSO_SECUNDARIO", "Anexar Processo secundário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.ATUALIZAR_ANDAMENTO_PROCESSO", "Atualização de Andamento do Processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.ALTERAR_ATRIBUICAO_PROCESSO", "Atribuição do processo alterada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.RESTAURACAO_VERSAO_DOCUMENTO", "Restauração de uma versão do documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.CONCLUSAO_PROCESSO_UNIDADE", "Conclusão do Processo na Unidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.CANCELAR_DOCUMENTO", "Cancelar Documentos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.ENVIO_PROCESSO", "Envio de processo para outra unidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.JUSTIFICATIVA_ACAO_PROCESSO", "Justificativa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.SUBSTITUIR_VERSAO_DOCUMENTO", "Confirma substituir a versão do documento?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.PROCESSO_SEM_DOCUMENTO", "É necessário incluir ao menos um documento no processo para executar a ação:", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.SUCESSO_CONCLUIR_PROCESSO", "Processo concluído com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.CONFIRMA_EXECUCAO_FLUXO", "Após a confirmação o fluxo será encaminhado para esta ação: ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.PROCESSO_ENVIADO_SUCESSO", "Processo enviado com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.CONFIRMA_CONCLUIR_UNIDADE", "Concluir processo nesta unidade?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.EXISTE_VINCULO_PROCESSO", "Não é possível excluir este cadastro. Existe vínculo com um ou mais processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.CONFIRMA_CANCELAR_DOCUMENTO", "Confirma cancelar o documento?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.DOCUMENTO_CANCELADO_SUCESSO", "Documento cancelado com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.CONFIRMA_RENUNCIAR_CREDENCIAL", "Deseja renunciar a credencial?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.CREDENCIAL_REMOVIDA_SUCESSO", "Credencial renunciada com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.SELECIONE_GRUPO_USUARIO", "Selecione um grupo/usuario", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.PROCESSO_SEM_DOCUMENTO_ENVIO_UNIDADE", "Para enviar o processo é necessário incluir documento ao mesmo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.UNIDADE_SEM_GRUPO_PADRAO", "Não é possível enviar o processo para unidade selecionada. Motivo: Grupo padrão não contém usuário(s)!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.CONFIRMA_CONCLUIR_UNIDADE", "Concluir processo nesta unidade?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.VALIDAR_PROCESSO", "Validar Processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.EXISTE_PROCESSO_ABERTO", "Existe um processo aberto, deseja fechá-lo?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.ATRIBUICAO_PRIMEIRO_ACESSO", "Esté processo foi atribuido para você", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.DESEJA_CAPTURAR", "Desejá capturar o processo?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.CAPTURADO_SUCESSO", "Processo capturado com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.LEGENDA_CAMPO_DINAMICO_NUMERO_DOCUMENTO", "Use: @numeroDocumento@ para adicionar o número do documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.PROCESSO_CONCLUIDO_SUCESSO", "Processo concluído com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.O_PROCESSO", "O processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.SALVO_SUCESSO", "Foi salvo com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.INICIAR_ATRIBUICAO_PROCESSO", "Atribuição do processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.NAO_EXISTE_TAREFA_EXECUTAR", "Não existem tarefas a serem executadas!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.CONFIRMA_SOBRESTAMENTO_TAREFA", "Confirma o sobrestamento da tarefa?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.TAREFA_SOBRESTADA_SUCESSO", "Tarefa sobrestada com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.CONFIRMA_REATIVACAO_TAREFA", "Confirma reativação da tarefa?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.TAREFA_REATIVADA", "Tarefa reativada com sucesso", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.VALIDADO", "Validado com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.SELECIONE_UM_ITEM_PARA_VALIDACAO", "Selecione um item para validar.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.ERRO_PERMISSAO_ASSINAR", "Erro! Para assinar documentos é necessário possuir o  papel “Servidor” ou “Terceirizado” com “Função” preenchida no Cadastro de Pessoas!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.USUARIO_SEM_PESSOA", "Usuário não está presente no Cadastro de Pessoa!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.DOCUMENTO_SEM_ASSINATURAS", "Esta versão do Documento não possui assinaturas!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.DOCUMENTO_JA_ASSINADO", "Documento já foi assinado por este usuário!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.INFORMACAO_RESTRITA", "Informação restrita", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG.CONFIRMAR_ASSINATURA", "Deseja realmente assinar este documento? Após esta ação, não será possível cancelar!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ECM.MSG_HISTORICO_ECM.REABERTURA_DE_PROCESSO", "Reabertura de processo", dominio, modulo));

	}
}
