'use strict';


citApp.controller('AssinarDocumentoController', [ '$scope', '$translate', 'GerenciarProcessoRepository', 'DocumentoGedRepository', 'HistoricoAlteracaoProcessoRepository', '$q', 'AssinaturaRepository', 'TarjaAssinaturaRepository', '$timeout',
                function AssinarDocumentoController($scope, $translate, GerenciarProcessoRepository, DocumentoGedRepository, HistoricoAlteracaoProcessoRepository, $q, AssinaturaRepository, TarjaAssinaturaRepository, $timeout) {

	/*
	 * Abre panel assinar documento
	 */
	$scope.abrirAssinarDocumento = function(documento) {
		$scope.documentoSelecionado = null;
		$scope.exibirDocJaAssinado = false;
		$scope.autenticacao = null;
		DocumentoGedRepository.get(documento.id).then(function(result) {
			$scope.documentoSelecionado = result.originalElement;
			if(documento.formaCriacao.codigo === 1){
				__abrirAssinarDocOnline(documento);
			} else if(documento.formaCriacao.codigo === 2){
				__abrirAssinarDocAnexo(documento);
			}
		});	
	};
	
	function __abrirAssinarDocOnline(documento){
		_obterAssinaturasDocumento(documento).then(function(documentosSelecionados){
			$scope.documentosSelecionados = documentosSelecionados;
			_exibirNenhumaAssinatura();
			_validarAssinaturaPorUsuario(documento.id).then(function(exibirIncluirAssinatura){
				$scope.exibirIncluirAssinatura = !(exibirIncluirAssinatura == true);
				$scope.exibirDocJaAssinado =  !$scope.exibirIncluirAssinatura;
				_getTarjaAssinaturaInterna();
			});
		});
	}
	
	function __abrirAssinarDocAnexo(documento){
		_obterAssinaturasDocumento(documento).then(function(documentosSelecionados){
			$scope.documentosSelecionados = documentosSelecionados;
			_exibirNenhumaAssinatura();
			_validarAssinaturaPorUsuario(documento.id).then(function(exibirIncluirAssinatura){
				$scope.exibirIncluirAssinatura = !(exibirIncluirAssinatura == true);
				$scope.exibirDocJaAssinado =  !$scope.exibirIncluirAssinatura;
				_getTarjaAssinaturaInterna();
			});
		});
	};
	
	function _validarAssinaturaPorUsuario(idDocumento){
		var deferred = $q.defer();
		AssinaturaRepository.validarAssinaturaPorUsuario(idDocumento).then(function(result){
			deferred.resolve(result);
		});
		return deferred.promise;
	}
	
	function _obterAssinaturasDocumento(documento) {
		var deferred = $q.defer();
		var idRaiz = documento.idRaiz;
		if ( !idRaiz )
			idRaiz = documento.id;
		DocumentoGedRepository.findByIdDocumentoTodasVersoes(idRaiz).then(function(result) {
			var results = [];
			if(result.length>0){
				result.forEach(function(item){
					results.push(item.originalElement);
				});
			}
			deferred.resolve(results);
		});
		return deferred.promise;
	}
	
	$scope.assinar = function() {
		if(!$scope.autenticacao){
			$scope.showAlert("error", $translate.instant('ECM.VALIDACAO.SENHA_ASSINATURA'));
			return;
		}
		
		$scope.$openModalConfirm({
			message: $translate.instant("ECM.MSG.CONFIRMAR_ASSINATURA"),
			callback: function () {
				$scope.setLoadingGet(true);
				AssinaturaRepository.saveAssinaturaInterna(_getAssinatura()).then(function(result){
					_atualizarDocumentoSelecionadoAposAssinar(result.originalElement);
					_resetAssinar();
					_atualizarDocDoProcessoAposAssinar();
					$scope.$modalConfirmInstance.dismiss('cancel');
					$scope.setLoading(false);
					$scope.showAlert("success", $translate.instant('ECM.ASSINADO_SUCESSO'));
				});
			}
		});

	};
	
	function _getAssinatura(){
		return {
				documentoGed : { id : $scope.documentoSelecionado.id },	
				funcao : {id : _getFuncaoChecked()}, 
				idTask : $scope.$parent.task.id, 
				password : $scope.autenticacao, 
				pessoa : { id : $scope.pessoa.id},
				conteudoTarja : { idTarjaAssinatura : $scope.tarjaAssinatura.id}};
	};
	
	function _atualizarDocumentoSelecionadoAposAssinar(assinatura){
		if($scope.documentoSelecionado.assinaturas)
			$scope.documentoSelecionado.assinaturas.push(assinatura);
		$scope.documentosSelecionados.forEach(function(doc){
			if(doc.id == $scope.documentoSelecionado.id)
				doc.assinaturas = $scope.documentoSelecionado.assinaturas;
		});
	};
	
	function _resetAssinar() {
		$scope.autenticacao = null;
		$scope.exibirIncluirAssinatura = false;
		$scope.nenhumaAssinatura = false;
	};
	
	function _atualizarDocDoProcessoAposAssinar() {
		DocumentoGedRepository.get($scope.documentoSelecionado.id).then(function(result){
			$scope.processo.documentos.forEach(function(doc){
				if(doc.id == result.originalElement.id) {
					doc.assinaturas = result.originalElement.assinaturas;
					doc.tresAssinaturas = result.originalElement.tresAssinaturas;
					doc.estado = result.originalElement.estado;
				}
			});
		});
	};
	
	function _getFuncaoChecked(){
		for(var pos = 0; pos < $scope.funcoes.length; pos++){
			if($scope.funcoes[pos].isServidor)
				return $scope.funcoes[pos].id;
		}
	};
	
	/*
	 * Abre panel visualizar assinaturas
	 */
	$scope.visualizarAssinaturasDocumento = function(documento){
		$scope.documentoSelecionado = documento;
		$scope.exibirIncluirAssinatura = false;
		$scope.exibirDocJaAssinado = false;
		$scope.setLoadingGet(true);
		_obterAssinaturasDocumento(documento).then(function(documentosSelecionados){
			$scope.documentosSelecionados = documentosSelecionados;
			$scope.documentosSelecionados.forEach(function(doc){
				if(doc.id == $scope.documentoSelecionado.id)
					$scope.documentoSelecionado = doc;
			});
			_exibirNenhumaAssinatura();
			$scope.setLoading(false);
			if($scope.documentoSelecionado.assinaturas && $scope.documentoSelecionado.assinaturas.length  == 0)
				$scope.showAlert("warning", $translate.instant('ECM.MSG.DOCUMENTO_SEM_ASSINATURAS'));
		});
	};
	
	function _exibirNenhumaAssinatura(){
		$scope.nenhumaAssinatura = $scope.documentosSelecionados.some(function(doc){
			return doc.assinaturas.length == 0;
		});
	};
	
	$scope.visualizarAssinatura = function(docSelecionado, assinatura){
		var valorOriginal = assinatura.isVisualizarAssinatura;
		var idAssinaturaSelecionada = null;
		if(valorOriginal)
			assinatura.isVisualizarAssinatura = false;
		else
			$scope.setLoadingGet(true);
			DocumentoGedRepository.get(docSelecionado.id).then(function(result){
				$scope.doc = result.originalElement;
				$scope.documentosSelecionados.forEach(function(doc){
					doc.assinaturas.forEach(function(ass){
						ass.isVisualizarAssinatura = false;
					});
					if(doc.id == docSelecionado.id)
						doc.assinaturas.forEach(function(ass){
							if(ass.id != assinatura.id)
								ass.isVisualizarAssinatura = false;
							else {
								ass.isVisualizarAssinatura = valorOriginal != true;
								idAssinaturaSelecionada = angular.copy(ass.id);	
							}
						});
					
					$scope.setLoading(false);
				});
				
				$scope.visualizarAssinaturaOnLine = $scope.doc.formaCriacao.codigo == 1;
				_getTarjaDaAssinatura(idAssinaturaSelecionada);
			});
	};
	
	function _getTarjaAssinaturaInterna(){
		$scope.tarjaAssinatura = null;
		TarjaAssinaturaRepository.getTarjaAssinaturaInterna().then(function(result){
			$timeout(function(){
				$scope.tarjaAssinatura = result.originalElement;
			},200);
		});
	};
	
	function _getTarjaDaAssinatura(idAssinaturaSelecionada){
		AssinaturaRepository.get(idAssinaturaSelecionada).then(function(result){
			$scope.assinaturaSelecionada = result.originalElement;
		});
	}
	
}] );	