'use strict';

citApp.controller('CredencialProcessoController', ['$scope', 'CredencialProcessoRepository', '$translate', '$timeout', '$filter', 'HistoricoCredencialProcessoRepository', function CredencialProcessoController($scope, CredencialProcessoRepository, $translate, $timeout, $filter, HistoricoCredencialProcessoRepository) {
	
	$scope.credencialProcesso = [];
	
	$scope.permiteRemoverCredencial = false;
	
	
	//Iniciar credencial de acesso
    $scope.iniciarCredencial = function(processo){
        this.credenciarProcessoForm = $scope.credencialProcessoController.credenciarProcessoForm;
        $scope.credenciais = [];
        $scope.processo = processo;
        $scope.herdadas = [];
        $scope.credencialProcesso.grupoUsuario = null;
        CredencialProcessoRepository.findByIdProcesso($scope.processo.id).then(function(result) {
	      	result.forEach(function (credencialProcesso) {
	    		
	    		$scope.credenciais.push(credencialProcesso.originalElement);
	    	});
	      	
	      	CredencialProcessoRepository.findCredenciaisHerdadas($scope.processo.id).then(function(result) {
	      		$scope.herdadas = result;
	      	});
	      	
	      	 $scope.findHistorico();

        });
      	
        $timeout(function(){
            $scope.credenciarProcessoForm.$submitted = false;
            $scope.credenciarProcessoForm.$setPristine();
        });
    };
    
    //Listar o historico de credenciais
    $scope.findHistorico = function(){
    	
    	$scope.historicos = [];
    	
      	HistoricoCredencialProcessoRepository.findByIdProcessoAndIdProcessoRemocao($scope.processo.id).then(function(result) {
	      	result.forEach(function (historico) {
	    		
	    		$scope.historicos.push(historico.originalElement);
	    	});
      	});
    };
    
    //Salvar credenciais
	 $scope.saveOrUpdate = function(){
		 
	        $scope.setLoadingSalva(true);
	        
	        $scope.credenciarProcessoForm.$submitted = true;
			
			 //verifica se o formulario está valido para salvar
	        if($scope.credenciais.length > 0){
	        
		        CredencialProcessoRepository.saveCredencial($scope.credenciais).then(function(result) {
		        	$scope.credenciais = [];
		        	result.forEach(function (credencialProcesso) {
		        		
		        		$scope.credenciais.push(credencialProcesso.originalElement);
		        	});
		        	$scope.findHistorico();
		            $scope.credenciarProcessoForm.$submitted = false;
		            $scope.showAlert("success", $translate.instant('MSG.REGISTRO_SALVO'));
		            $scope.setLoading(false);           
		        });
		        
	        }else{
	        	//Mensagem de erro de campos obrigatorios não preenchidos
	            $scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
	        }
		};
    
    //listar os grupos e usuarios
    $scope.findAutoCompleteGrupoUsuario =  function(value){
		   return CredencialProcessoRepository.findGrupoUsuarioByName(value).then(function(result) {
			   
			   var listSemGrupoAdicionado = $filter('idNotEqualObj')(result, $scope.credenciais, 'grupo');
			   
			   var credenciadosFiltrados =  $filter('idNotEqualObj')(listSemGrupoAdicionado, $scope.credenciais, 'usuario');
			   
			   return  $filter('idNotEqual')(credenciadosFiltrados, $scope.herdadas);
		   });
		};
		
	//Adicionar credencial
	$scope.addCredencial = function(){
		
		if($scope.credencialProcesso.grupoUsuario){
			//verifica se a credencial já foi persistida. Verdadeiro atualiza a propriedade que está sendo editada. Falso: cria um novo objeto
			if($scope.credencialProcesso.edit){
				//Atribui a propriedade que está sendo editada
				$.grep($scope.credenciais, function(e){ return e.$$hashKey == $scope.credencialProcesso.$$hashKey; })[0].permiteCredenciar = $scope.credencialProcesso.permiteCredenciar;
				
			}else{
				
				var credencialProcesso = {};
				credencialProcesso.id = $scope.credencialProcesso.id;
				credencialProcesso.permiteCredenciar = $scope.credencialProcesso.permiteCredenciar;
				credencialProcesso.processo = $scope.processo;
				credencialProcesso.idTask = $scope.processo.idTask;
				
				//Verifica se o grupUsuario é um grupo, codigo 1 é grupo e 2 usuário
				if($scope.credencialProcesso.grupoUsuario.codigo == 1){
					
					credencialProcesso.grupo ={
							
						id   :	$scope.credencialProcesso.grupoUsuario.id,
					    nome :  $scope.credencialProcesso.grupoUsuario.nome,
					};
					credencialProcesso.icon = "fa fa-users";
					
				}else{
					
					credencialProcesso.icon = "fa fa-user";
					credencialProcesso.usuario = {
						
						id : $scope.credencialProcesso.grupoUsuario.id,
						username : $scope.credencialProcesso.grupoUsuario.nome
						
					};
					
					//Verifica se o usuário tem unidade padrão
					if($scope.credencialProcesso.grupoUsuario.nomeUnidade){
						
						credencialProcesso.usuario.unidade = {};
						credencialProcesso.usuario.unidade.nome = $scope.credencialProcesso.grupoUsuario.nomeUnidade;
						
					}
				}
				
				$scope.credenciais.push(credencialProcesso);
			}
			$timeout(function(){
				$scope.credencialProcesso = null;
			});
		}else{
			 $scope.showAlert('warning', $translate.instant('ECM.MSG.SELECIONE_GRUPO_USUARIO'));
		};

	};
	
	//Editar credencial
	$scope.editCredencial = function(credencialProcesso){
		//prepara objeto edição
		$scope.credencialProcesso = {
				edit : true,
				$$hashKey : credencialProcesso.$$hashKey,
				permiteCredenciar  :  credencialProcesso.permiteCredenciar,
				grupoUsuario : {
					nome : credencialProcesso.grupo ?  credencialProcesso.grupo.nome :  credencialProcesso.usuario.username,
					id : credencialProcesso.grupo ? credencialProcesso.grupo.id : credencialProcesso.usuario.id
				}
		};
		
	};
	
    //Exibir o dialog de confirmação de exclusão
	$scope.exibirDialogRemocao = function(credencialProcesso){
			
		$scope.$openModalConfirm({message: $translate.instant('LABEL.CONFIRMA_EXCLUSAO'), callback: $scope.remove, item : credencialProcesso});
	};
	
	//Remover credencial
	$scope.remove = function(){
		
		//Obter o index do elemento
		 var index =  $scope.credenciais.indexOf(this.item);
		 
		//Verifica se a credencial selecionada já foi persistida
		if(this.item.id){
			
			CredencialProcessoRepository.remove(this.item).then(function() {
				$scope.showAlert("success", $translate.instant('MSG.EXCLUSAO_SUCESSO'));
				//Remove a credencial da lista de credenciais
				$scope.credenciais.splice(index , 1);
				$scope.credencialProcessoChecked = null;
				$scope.findHistorico();
				
			});
					
		}else{
			$scope.showAlert("success", $translate.instant('MSG.EXCLUSAO_SUCESSO'));
			//Remove a credencial da lista de credenciais
			$scope.credenciais.splice(index , 1);
			
			$scope.credencialProcessoChecked = null;
			
		}
		$scope.$modalConfirmInstance.dismiss('cancel');
  };
  
	  //Verifica se o usuario logado tem permissão de remover a credencial
	  $scope.verificarPermissaoRemover = function(){
	  
	  $scope.permiteRemoverCredencial = $scope.usuarioLogado.id === $scope.credencialProcessoChecked.autor.id;
  };
    
    
}]);