'use strict';

citApp.controller('PlanoClassificacaoListController',['$scope', 'PlanoClassificacaoRepository', '$translate', '$timeout', 'DominioRepository', function PlanoClassificacaoListController($scope, PlanoClassificacaoRepository, $translate, $timeout, DominioRepository) {
    $scope.$showAdvancedFilters = false;
    
    $scope.planoClassificacoes = [];
    
    $scope.mostraArvore = true;
    
    $scope.buscaPlanoClassificacao = null;

    $scope.resetForm = function(planoClassificacaoPai) {
        angular.element("#editPlanoClassificacao").scope().resetForm(planoClassificacaoPai);
    };

    $scope.headers = [ {title : $translate.instant('LABEL.CODIGO'), value : 'codigo', tamanho : 10} , {title : $translate.instant('LABEL.ASSUNTO'), value : 'nome', tamanho : 35  }, {title : $translate.instant('ECM.LABEL.FASE_CORRENTE_ANO'), value : 'faseCorrente', tamanho : 5 },{title : $translate.instant('ECM.LABEL.FASE_INTERMEDIARIA_ANO'), value : 'faseIntermediaria', tamanho : 5 } , {title : $translate.instant('ECM.LABEL.DESTINACAO'), value : 'tipoDestinacao.descricao', tamanho : 15 }, {title : $translate.instant('ECM.LABEL.OBSERVACAO'), value : 'observacao', tamanho : 30 }];

    $scope.filterCriteria = {
        start : 1,
        dir : 'asc',
        sort : 'id',
        limit : 10,
        fields: ['id', 'codigo', 'nome', 'observacao', 'faseCorrente', 'faseIntermediaria', 'tipoDestinacao.descricao'],
        filters : [ {type : 'string', field : 'codigo' } , {type : 'string', field : 'nome' } , {type : 'string', field : 'observacao' }, {type : 'numeric', field : 'faseCorrente' }, {type : 'numeric', field : 'faseIntermediaria' }, {type : 'string', field : 'tipoDestinacao.descricao', listaDominio : [] }]
    };
    
    
    DominioRepository.findAllDominio('tipoDestinacao').then(function(result) {
        $scope.filterCriteria.filters[5].listaDominio = result;
    });
            
    // ABRI PlanoClassificacao SELECIONADA
    $scope.abrirVisualizar = function(edit){
        var planoClassificacao = $scope.planoClassificacaoChecked;

        if(!planoClassificacao) {
            $scope.showAlert('warning', !edit ? $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_VISUALIZACAO') : $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO'));
            return;
        }
        angular.element('#editPlanoClassificacao').scope().getPlanoClassificacao(planoClassificacao, edit);
        $scope.$showPageEditWorkspace($scope.workspace);
    };
    
    //abir a tela de edição
    $scope.abrirPlanoClassificacao = function(planoClassificacao, edit){
    	
    	  angular.element('#editPlanoClassificacao').scope().getPlanoClassificacao(edit, planoClassificacao);
          $scope.$showPageEditWorkspace($scope.workspace);
    };
    
	//Define se a arvore sera apresentada
	$scope.mostrarArvore = function() {
		$scope.mostraArvore = !$scope.mostraArvore;
		
		if ($scope.mostraArvore) {
			$scope.inicializarArvore();
			$scope.buscaPlanoClassificacao = null;
		} else {
			$scope.planoClassificacoes = [];
			$scope.fetchResult();
		}
	};
	
	//Voltar para tela de listagem
	$scope.voltar = function(){
		
		if ($scope.mostraArvore) {
			$scope.inicializarArvore();
			$scope.buscaPlanoClassificacao = null;
		} else {
			$scope.planoClassificacoes = [];
			$scope.fetchResult();
		}
	};
	
	//Responsável por iniciar a arvore
	$scope.inicializarArvore = function(){
		
		PlanoClassificacaoRepository.findParent($scope.usuarioLogado.organizacao.id).then(function(result) {
			
			angular.forEach(result, function(item) {
				
				item.sinalPositivo = item.contemFilhos;
				
				
			});
			
			 $scope.planoClassificacoes = result;
			
		});
	};
	
	//Busca os filhos do planoClassificacao selecionado
	$scope.mostrarFilhos = function(planoClassificacaoSelecionado){
		
		if(planoClassificacaoSelecionado.sinalPositivo) {

			planoClassificacaoSelecionado.sinalPositivo = !planoClassificacaoSelecionado.sinalPositivo;
	
			$scope.setLoadingPesquisa(true);
	
			planoClassificacaoSelecionado.planoClassificacaoFilhas = [];
	
			// busca os filhos 
			PlanoClassificacaoRepository.findChildrens(planoClassificacaoSelecionado.originalElement.id).then(function(result) {
	
				angular.forEach(result, function(subPlanoClassificacao){
					subPlanoClassificacao.sinalPositivo = subPlanoClassificacao.contemFilhos;
					planoClassificacaoSelecionado.planoClassificacaoFilhas.push(subPlanoClassificacao);
					subPlanoClassificacao.planoClassificacaoFilhas = [];
				});
	
				$scope.setLoading(false);
	
			});
			
		}else {

			planoClassificacaoSelecionado.sinalPositivo = !planoClassificacaoSelecionado.sinalPositivo && (planoClassificacaoSelecionado.planoClassificacaoFilhas && planoClassificacaoSelecionado.planoClassificacaoFilhas.length > 0);

			planoClassificacaoSelecionado.planoClassificacaoFilhas = [];
		}

	};
	
	//Listar recursivo
	$scope.buscarTree = function(nome){

		// Valida se foi digitado mais que 4 caracteres
		if(nome.length > 2) {

			$scope.setLoadingPesquisa(true);

			$scope.planoClassificacoes = [];

			return PlanoClassificacaoRepository.findTree(nome).then(function(result) {

				$scope.planoClassificacoes = [];
			
				angular.forEach(result, function(item) {
					
					item.sinalPositivo = !item.contemFilhos;
					
				});
				
				$scope.planoClassificacoes = result;
				
				$scope.setLoading(false);

			});

			// Se limpar o campo de pesquisa, inicializa
		} else{
			$scope.inicializarArvore();
		}
	};
	
	$scope.inicializarArvore();
	
}]);
