/**
 * This file is subject to the terms and conditions defined in the
 * 'LICENSE.txt' file, which is part of this source code package.
 */

'use strict';

/*---------------------------------------------------------------------------*/
/* DocumentSearchResultsController              
 * 
 * @autor: Ally Junio
 * @autor: Rogerio Costa
 * @autor: Maycon Fernandes
 *                               
 **/

/**
 * Presents search results for a named query.
 * @param $scope
 * @param $attrs
 * @param $location
 * @param $route
 * @param $routeParams
 * @param $window
 * @param SolrSearchService
 * @param Utils
 */

citApp.controller('SolrSearchController', ['$scope','$attrs','$location','$route','$routeParams','$window','$timeout','SolrSearchService','Utils', "$compile", 'DominioRepository', '$filter', "workflowFilterCriteriaService", 
                                           "BusinessProcessRepository", "appService", "$rootScope", "$translate", 'UnidadeProcessoRepository', 'CredencialProcessoRepository',
          function SolrSearchController($scope, $attrs, $location, $route, $routeParams, $window, $timeout, SolrSearchService, Utils, $compile, DominioRepository, $filter, workflowFilterCriteriaService, BusinessProcessRepository, 
        		  appService, $rootScope, $translate, UnidadeProcessoRepository, CredencialProcessoRepository) {
	
	$scope.searchHintsField = 'tipoProcesso,tipoDocumento,objeto,assunto,autor,atribuidoA,destinatario,unidadeGeradora,situacao,nivelAcesso,dataCriacao';
	
    // instructional message to aid the user in constructing a query
    $scope.messages = [
        "Enter one or more search terms. Press Enter to search.",
        "Key up or down to navigate hints. Press Enter to search.",
        "Press Enter to search."
    ];
    $scope.message = $scope.messages[0];
    
    //Propriedade responsavel por apresentar a mensagem informando que a pesquisa não obteve resultado
    $scope.apresentarMsgSemResultado = false;
    
    // the maximum number of hints to display at any moment
    $scope.maxHints = 10;
    
    // find near matches to the user query
    $scope.nearMatch = false;
    
    // the minimum number characters that the user should enter before the list
    // of search hints is displayed
    $scope.minSearchLength = 3;
    
	// document search results
    $scope.documents = [];

    // the number of search results to display per page
    $scope.documentsPerPage = 10;

    // flag for when the controller has submitted a query and is waiting on a
    // response
    $scope.loading = false;

    // the current search result page
    $scope.page = 0;

    // list of pages in the current navigation set
    $scope.pages = [];

    // the number of pages in a navigation set
    $scope.pagesPerSet = 10;

    // the query name
    $scope.queryName = SolrSearchService.defaultQueryName;

    // url to solr core
    $scope.source = undefined;

    // zero based document index for first record in the page
    $scope.start = 0;

    // count of the total number of result pages
    $scope.totalPages = 1;

    // count of the total number of search results
    $scope.totalResults = 0;

    // count of the number of search result sets
    $scope.totalSets = 1;

    // update the browser location on query change
    $scope.updateLocationOnChange = true;

    // user query
    $scope.userquery = '';
    
    //parametro para pesquisa
    $scope.parameterPesquisa  = {};
    
    //Parameters
    $scope.queryParameters = [];

    //Date parameter
    $scope.dateMathFilter = "";
    

    /**
     * A page in a pagination list
     * @param Name Page name
     * @param Num Page number
     */
    function Page(Name,Num) {
        this.name = Name;
        this.number = Num;
        this.isCurrent = false;
    }

    /**
     * Set the results page number.
     * @param Start Index of starting document
     */
    $scope.handleSetPage = function() {
    	
    	$scope.timeout = $timeout(function () {
            var query = SolrSearchService.createQuery("https://localhost:8983/solr/jcg");
            query.setOption("rows", "10");
            query.setOption("facet", "true");
            query.setOption("facet.limit", $scope.maxHints);
            query.setOption('start',  $scope.start * $scope.documentsPerPage - 10);
            
            //Verifica se foi selecionado filtro de data
            if($scope.dateMathFilter != ""){
            	
            	query.setOption("fq", $scope.dateMathFilter);
            }
            
            var hints = $scope.searchHintsField.split(',');
            for(var i = 0; i < hints.length; i++){
            	query.setOption("facet.field#" + i, hints[i]);
            }
            
            query.setNearMatch($scope.nearMatch);
            query.setUserQuery($scope.userQuery);
            query.setQueryParameters($scope.queryParameters);
            SolrSearchService.setQuery($scope.queryName, query);
            SolrSearchService.updateQuery($scope.queryName);
        }, 350);    	
    };
    
    $scope.setPage = function (pageNo) {
    	$scope.start = pageNo;
    };
      
    /**
     * Update the controller state.
     */
    $scope.handleUpdate = function() {
        // clear current results
        $scope.documents = [];
        $scope.loading = false;
        // get new results
        var results = SolrSearchService.getResponse($scope.queryName);
        if (results && results.docs) {
        	
            $scope.totalResults = results.numFound;
            // calculate the total number of pages and sets
            $scope.totalPages = Math.ceil($scope.totalResults / $scope.documentsPerPage);
            $scope.totalSets = Math.ceil($scope.totalPages / $scope.pagesPerSet);
            // add new results
            for (var i=0;i<results.docs.length && i<$scope.documentsPerPage;i++) {
                // clean up document fields
                results.docs[i].fromDate = Utils.formatDate(results.docs[i].fromDate);
                results.docs[i].toDate = Utils.formatDate(results.docs[i].toDate);
                
              	if(results.docs[i].objeto == 'Documento' && SolrSearchService.queries.defaultQuery.highlighting){

             	    var object = results.docs[i].id;

             	    if(SolrSearchService.queries.defaultQuery.highlighting[object].conteudo){

             	        var highlightings = SolrSearchService.queries.defaultQuery.highlighting[object];

             	        results.docs[i].conteudo = "";
             	        
                        angular.forEach(highlightings, function (item) {
                            results.docs[i].conteudo = results.docs[i].conteudo +  item; 
                        });
                        
             	    }else{
                  		results.docs[i].conteudo = "";
             	    }
              	}
                
                // add to result list
                $scope.documents.push(results.docs[i]);
            }
        } else {
            $scope.documents = [];
            $scope.totalResults = 0;
            $scope.totalPages = 1;
            $scope.totalSets = 1;
        }
        
        // GET FACETS
        $scope.hints = [];
        if(results.numFound > 0){
        	$scope.apresentarMsgSemResultado = false;
        	$scope.message = $scope.messages[1];
            var query = SolrSearchService.getQuery($scope.queryName);
            var results = query.getFacetCounts();
            if (results && results.hasOwnProperty('facet_fields')) {
                
                $scope.hintFields = $scope.searchHintsField.split(',');
                for(var j = 0; j < $scope.hintFields.length; j++){
                	var result = results.facet_fields[$scope.hintFields[j]];
                	
                	if (result) {
                    	var myArray = new Array();
                        for (var i = 0; i < result.length; i += 2) {
                        	//Verifica se o filtro está vazio
                            if(result[i] != "" && result[i + 1] > 0){
                                var hint = {
                            	name: Object.keys(results.facet_fields)[j],	
                                title: result[i],
                                quantidade: result[i + 1],
                                selected: false
                             };
                               myArray.push(hint);  
                            }                                   
                        }
                        
                        $scope.hints[$scope.hintFields[j]] = myArray;
                    }
                }
            }
            
        }else{
        	
        	$scope.apresentarMsgSemResultado = true;
        }
		 $timeout(function(){
        	angular.forEach($scope.documents, function (item) {
        		
        		if(item.objeto == 'Documento'){
	        		
        			document.getElementById(item.id).innerHTML = item.conteudo ;
        		}
        		
        	});
		 });
        
        // update the page index
        $scope.updatePageIndex();
    };

    /**
     * Initialize the controller.
     */
    $scope.init = function() {
    	// apply configured attributes
        for (var key in $attrs) {
            if ($scope.hasOwnProperty(key)) {
                if (key == 'documentsPerPage' || key == 'pagesPerSet') {
                    $scope[key] = parseInt($attrs[key]);
                } else if ($attrs[key] == 'true' || $attrs[key] == 'false') {
                    $scope[key] = $attrs[key] == "true";
                } else {
                    $scope[key] = $attrs[key];
                }
            }
        }
        
        // handle location change event, update query results
        $scope.$on("$routeChangeSuccess", function() {
            // if there is a query in the current location
            $scope.query = ($routeParams.query || "");
            if ($scope.query) {
                // reset state
                $scope.loading = false;
                // get the current query
                var query = SolrSearchService.getQueryFromHash($scope.query, $scope.source);
                // if there is a data source specified, override the default
                if ($scope.source) {
                    query.solr = $scope.source;
                }
                query.setOption("rows",$scope.documentsPerPage);
                // set the display values to match those in the query
                $scope.userquery = query.getUserQuery();
                // update query results
                SolrSearchService.setQuery($scope.queryName, query);
                $scope.loading = true;
                SolrSearchService.updateQuery($scope.queryName);
            }
        });
        // handle update events from the search service
        $scope.$on($scope.queryName, function () {
            $scope.handleUpdate();
        });
    };
    
    /**
     * Handle search box input blur event. Deselect any selected
     * hints then update the user interface.
     */
    $scope.onblur = function () {
        //$scope.showHints = false;
        // $scope.resetHintSelection();
        // $scope.$apply();
    };

    /**
     * Handle search box input focus event. When the search
     * box has focus then size and display the search hints
     * DIV.
     */
    $scope.onfocus = function () {
        // set the width of the div to be equal to the input box
        // subtract the width of the #hints border
        // var width = $scope.query_input.offsetWidth - 2;
        // $scope.hints_div.setAttribute('style', "width:" + width + "px");
        $scope.showHints = true;
        //$scope.$apply();
    };

    /**
     * Handle user data entry on input field.
     * @param event
     */
    $scope.onkeyup = function (event) {
    	
      if ($scope.userQuery && $scope.userQuery.length >= $scope.minSearchLength) {
            if ($scope.timeout) $timeout.cancel($scope.timeout);
	            $scope.timeout = $timeout(function () {
	                var query = SolrSearchService.createQuery("https://localhost:8983/solr/jcg");
	                query.setOption("rows", "10");
	                query.setOption("facet", "true");
	                query.setOption("facet.limit", $scope.maxHints);
	                
	                //Verifica se foi selecionado filtro de data
	                if($scope.dateMathFilter != ""){
	                	
	                	query.setOption("fq", $scope.dateMathFilter);
	                }
	                
	                var hints = $scope.searchHintsField.split(',');
	                for(var i = 0; i < hints.length; i++){
	                	query.setOption("facet.field#" + i, hints[i]);
	                }
	                
	                query.setNearMatch($scope.nearMatch);
	                query.setUserQuery($scope.userQuery);
	                query.setQueryParameters($scope.queryParameters);
	                
	                SolrSearchService.setQuery($scope.queryName, query);
	                SolrSearchService.updateQuery($scope.queryName);
            }, 350);
        }else{
        	
        	$scope.apresentarMsgSemResultado = false;
        	
        	 $scope.documents = [];
        }
    };
    
    /**
     * Update page index for navigation of search results. Pages are presented
     * to the user and are one-based, rather than zero-based as the start
     * value is.
     */
    $scope.updatePageIndex = function() {
        var query = SolrSearchService.getQuery($scope.queryName);
        $scope.documentsPerPage = (query.getOption('rows') || $scope.documentsPerPage);
        $scope.page = (Math.ceil(query.getOption('start') / $scope.documentsPerPage) || 0);
        // the default page navigation set
        $scope.pages = [];
        // determine the current zero based page set
        var currentSet = Math.floor($scope.page / $scope.pagesPerSet);
        // determine the first and last page in the set
        var firstPageInSet = (currentSet * $scope.pagesPerSet) + 1;
        var lastPageInSet = firstPageInSet + $scope.pagesPerSet - 1;
        if (lastPageInSet > $scope.totalPages) {
            lastPageInSet = $scope.totalPages;
        }
        // link to previous set
        if ($scope.totalSets > 1 && currentSet != 0) {
            var previousSet = firstPageInSet - $scope.pagesPerSet - 1;
            var prevPage = new Page("teste;", previousSet);
            $scope.pages.push(prevPage);
        }
        // page links
        for (var i=firstPageInSet; i<=lastPageInSet; i++) {
            var page = new Page(i, i-1);
            if (page.number == $scope.page) {
                page.isCurrent = true;
            }
            $scope.pages.push(page);
        }
        // link to next set
        if ($scope.totalSets>1 && currentSet<$scope.totalSets-1) {
            var nextSet = lastPageInSet;
            var nextPage = new Page("77", nextSet);
            $scope.pages.push(nextPage);
        }
    };
    
    $scope.handleSubmit = function () {
        // clean up the user query
        var trimmed = Utils.trim($scope.userQuery);
        if (trimmed === '') {
            $scope.userQuery = "*:*";
        }
        $scope.timeout = $timeout(function () {
            var query = SolrSearchService.createQuery("https://localhost:8983/solr/jcg");
            query.setOption("rows", "10");
            query.setOption("facet", "true");
            query.setOption("facet.limit", $scope.maxHints);
            
            var hints = $scope.searchHintsField.split(',');
            for(var i = 0; i < hints.length; i++){
            	query.setOption("facet.field#" + i, hints[i]);
            }
            
            query.setNearMatch($scope.nearMatch);
            query.setUserQuery($scope.userQuery);
            SolrSearchService.setQuery($scope.queryName, query);
            SolrSearchService.updateQuery($scope.queryName);
        }, 350);
        
    };
    
    $scope.selectHint = function (nameIndex, index) {
        var hint = $scope.hints[nameIndex][index];
        var userQuery = hint.title;
        $scope.aplicarFilter(userQuery);
    };
    
    $scope.aplicarFilter = function(valueFilter){
    	 // clean up the user query
        var trimmed = Utils.trim(valueFilter);
        if (trimmed === '') {
        	valueFilter = "*:*";
        }
        $scope.timeout = $timeout(function () {
            var query = SolrSearchService.createQuery("https://localhost:8983/solr/jcg");
            query.setOption("rows", "10");
            query.setOption("facet", "true");
            query.setOption("facet.limit", $scope.maxHints);
            
            var hints = $scope.searchHintsField.split(',');
            for(var i = 0; i < hints.length; i++){
            	query.setOption("facet.field#" + i, hints[i]);
            }
            
            query.setNearMatch($scope.nearMatch);
            query.setUserQuery(valueFilter);
            SolrSearchService.setQuery($scope.queryName, query);
            SolrSearchService.updateQuery($scope.queryName);
        }, 350);
    };
    
    //limpar os filtros da tela
    $scope.cleanFilters = function(){
    	
    	$scope.documents = [];
    	$scope.userQuery = "";
    	$scope.hints = [];
    	$scope.parameterDatePesquisa =  $.grep($scope.filtroDateList, function(e){ return e.codigo == 0; })[0].originalElement;
    };
    
    //Adicionar os parametro processo
    $scope.addParameter = function(){
    	
    	$timeout(function () {
			//verifica se o parametro é ambos
			if($scope.parameterPesquisa.codigo == 0){
				
				$scope.removeQueryParameter("AND objeto: Processo ");
				
				$scope.removeQueryParameter("AND objeto: Documento ");
				$scope.onkeyup();
				
				//verifica se o parametro é processo
			}else if($scope.parameterPesquisa.codigo == 1){
				
				$scope.addQueryParameter("AND objeto: Processo ");
				$scope.removeQueryParameter("AND objeto: Documento ");
				$scope.onkeyup();
				//verifica se o parametro é documento
			}else if($scope.parameterPesquisa.codigo == 2){
				
				$scope.addQueryParameter("AND objeto: Documento ");
				$scope.removeQueryParameter("AND objeto: Processo ");
				$scope.onkeyup();
			}
    	});
    	
    };
    
    //adicionar o parametro de data
    $scope.addParameterDate = function(){
    	
    	$timeout(function () {
    		
    		if($scope.parameterDatePesquisa.codigo == 0){
    			
    			$scope.dateMathFilter = "";
    			
    			$scope.onkeyup();
    			
    		  	//Última Hora
    		}else if($scope.parameterDatePesquisa.codigo == 1){
	    		
	    		var date = new Date();
	    		date.setHours(date.getHours() - 1);
	    		
	    		var dataUltimaHora = $filter('date')(date, "yyyy-MM-dd'T'HH:mm:ss'Z'");
	    		
	    		$scope.dateMathFilter = "dataCriacao:["+dataUltimaHora+" TO NOW]";
	    		$scope.onkeyup();
	    		//Útimas 24 horas
	    	}else if($scope.parameterDatePesquisa.codigo == 2){
	    		
	    		$scope.dateMathFilter = "dataCriacao:[NOW-1DAY TO NOW]";
	    		$scope.onkeyup();
	    		//Última semana
	    	}else if($scope.parameterDatePesquisa.codigo == 3){
	    		
	    		$scope.dateMathFilter = "dataCriacao:[NOW-7DAY TO NOW]";
	    		$scope.onkeyup();
	    		//Último mês
	    	}else if($scope.parameterDatePesquisa.codigo == 4){
	    		
	    		$scope.dateMathFilter = "dataCriacao:[NOW-31DAY TO NOW]";
	    		$scope.onkeyup();
	    		//Útimo ano
	    	}else if($scope.parameterDatePesquisa.codigo == 5){
	    		
	    		$scope.dateMathFilter = "dataCriacao:[NOW-365DAY TO NOW]";
	    		$scope.onkeyup();
	    	}
    	});
    	
    };
    
    //adiciona o filtro entre datas
    $scope.addParameterDatePersonalisada = function(){
    	
    	var dataInicioFormat = $filter('date')($scope.dataInicioFiltro, "yyyy-MM-dd'T'HH:mm:ss'Z'");
    	
    	var dataFimFormat = $filter('date')($scope.dataFimFiltro, "yyyy-MM-dd'T'HH:mm:ss'Z'");
    	
    	if($scope.dataInicioFiltro && $scope.dataFimFiltro){
    		
    		$scope.dateMathFilter = "dataCriacao:["+dataInicioFormat+" TO "+dataFimFormat+ "]";
    		
    		$scope.onkeyup();
    	}else{
    		
    		$scope.dateMathFilter = "";
    	}
    	
    	
    };
    
    /**
     * Add a query parameter. The parameter setting is added only if it is
     * unique.
     * @param Parameter
     */
    $scope.addQueryParameter = function(Parameter) {
        for (var i=0; i<$scope.queryParameters.length;i++) {
            if ($scope.queryParameters[i] == Parameter) {
                return;
            }
        }
        $scope.queryParameters.push(Parameter);
    };
    
    /**
     * remove a query parameter
     * unique.
     * @param Parameter
     */
    $scope.removeQueryParameter = function(Parameter) {
        for (var i=0;i<$scope.queryParameters.length;i++) {
            if ($scope.queryParameters[i] == Parameter) {
            	$scope.queryParameters.splice(i, 1);
            }
        }
    };
    
    DominioRepository.findAllDominio('filtroPesquisa').then(function(result) {
        $scope.filtroPesquisaList = result;
        
        $scope.parameterPesquisa =  $.grep($scope.filtroPesquisaList, function(e){ return e.codigo == 0; })[0].originalElement;
    });
    
    DominioRepository.findAllDominio('filtroData').then(function(result) {
        $scope.filtroDateList = $filter('orderBy')(result, "codigo");
        
        $scope.parameterDatePesquisa =  $.grep($scope.filtroDateList, function(e){ return e.codigo == 0; })[0].originalElement;
        
    });
	
	$scope.obterTransclude = function(doc) {
		
		doc.$show = !doc.$show;
		$scope.documents.forEach(function(item){
			if(item.id != doc.id){
				item.$expandido = false;
				item.$show = false;
			}
		});
		
		 $scope.docSelecionado = doc;
		 if(doc.$show){
			 _getFilterCriteriaTarefa();
			 _fetchBusinessProcess(doc);
		 }else{
		 	doc.$expandido = false;
		 	doc.$show = false;
		 };
	};
	
	function _getFilterCriteriaTarefa(){
		var filterService = workflowFilterCriteriaService.getDefaultFilterCriteria();
		$scope.filterCriteriaBusinessProcess = {
				start : 1,
				sort : 'workItem.processInstance.id',
				limit : 10,
				deadline : 1,
				processDeadline : 1,
				timeManagementStatus : "",
				fields : [ 'workItem.id', 'workItem.processInstance.id'],
				filters : [{type: 'numeric', field: 'workItem.processInstance.id', value : $scope.docSelecionado.idProcessInstance}
						 , {type: 'string', field: 'workItem.flowElement.name'}
						 , {type: 'string', field: 'workItem.processInstance.businessProcess.description'}
						 , {type: 'string', field: 'workItem.processInstance.flowStatus.name'}]
		};
		$scope.filterCriteriaBusinessProcess.username = filterService.username;
		$scope.filterCriteriaBusinessProcess.groups = filterService.groups;
	}
	
	function _fetchBusinessProcess(doc) {
		$scope.assignments = [];
		$scope.btnAction = false;
		$scope.workItem = null;
		$scope.setLoadingGet(true);
		BusinessProcessRepository.tasksByParam($scope.filterCriteriaBusinessProcess).then(function(result) {
			$scope.assignments = result.originalElement.objects;

			if($scope.docSelecionado.idProcessInstance){
				CredencialProcessoRepository.validarAcessoPesquisa($scope.docSelecionado.idProcessInstance).then(function(result) {
					
					$scope.urlInclude= '/cit-ecm-web/html/pesquisa/tarefasProcesso.jsp?i=' + $scope.docSelecionado.id;
					doc.$expandido = !doc.$expandido;
				});
				
			}
			if($scope.assignments.length > 0){
				_recuperarPermissoes($scope.assignments[0].id, doc);
			};
			$scope.setLoadingGet(false);
		});
	};
	
	function _recuperarPermissoes(idAssignment, doc){
		BusinessProcessRepository.getPermissions({id: idAssignment}).then(function(result) {
			$scope.workItem = result.originalElement;
			vincularWorkItemAssignments();
			$scope.btnAction = !(!$scope.workItem.execute && !$scope.workItem.suspend && !$scope.workItem.delegate && !$scope.workItem.restart && !$scope.workItem.visualize);
			$scope.docSelecionado.workItem = $scope.workItem;
			$scope.setLoadingGet(false);
		});
	};
	
	function vincularWorkItemAssignments(){
		$scope.assignments.forEach(function(assignment){
			assignment.workItem = $scope.workItem;
			assignment.checked = false;
		});
		if($scope.assignments.length > 0){
			$scope.assignments[0].checked = true;
		}
	};
	
	$scope.executarTarefa = function(){
		_getAssignmentSelected();
		if($scope.workItem){
			var pagina = '/cit-esi-web/assets/js/angular/custom/directive/html/userTask.html';
			if ($scope.workItem.flowElement.userInterface && $scope.workItem.flowElement.userInterface.executeCustomPage) {
				pagina = $scope.workItem.flowElement.userInterface.URLCustomPage;
				_abrirPaginaTarefa(pagina);
			}else if ($scope.workItem.flowElement.userInterface && $scope.workItem.flowElement.userInterface.resource && $scope.workItem.flowElement.userInterface.resourceName) {
				FormBuilderRepository.getByName($scope.workItem.flowElement.userInterface.resourceName).then(function(result) {
					pagina = '/cit-esi-web/forms/'+result.originalElement.path+'/'+result.originalElement.resource.name+"_task.html";
					_abrirPaginaTarefa(pagina);
				});
			}else{
				_abrirPaginaTarefa(pagina);
			}
		}
	};
	
	function _abrirPaginaTarefa(pagina){
		if (appService.existsWorkspace(pagina)) {
			$scope.showAlert('warning', $translate.instant('ESI.MSG.JA_EXISTE_TELA_TAREFA'));
			return ;
		}
		$scope.processInstance = $scope.docSelecionado.workItem.processInstance;
		$scope.processInstance.collapsed = true;
		$scope.processInstance.collapsedError = true;
		$rootScope.task = $scope.workItem;
		$rootScope.processInstance = $scope.processInstance;
		$rootScope.controllerScope = $scope;

		var nome = $scope.task.flowElement.description;
		if (!nome || nome == '')
			nome = $scope.workItem.flowElement.name;

		$scope.addNewWorkspace($translate.instant('ESI.PERMISSAO.EXECUTAR')+" "+$translate.instant('ESI.TAREFA').toLowerCase()+" '"+nome+"'", pagina, true, 'mod-red-dark', $scope.workItem);
	};
	
	function _getAssignmentSelected() {
		for(var pos = 0; pos < $scope.assignments.length; pos++){
			if($scope.assignments[pos].checked){
				$scope.docSelecionado.workItem = $scope.assignments[pos];
				break;
			}
		}
	};
	
	//Abre a tela de visualização de processo e documentos
	$scope.abreVisualizar = function(){
		
		var pagina = '/cit-ecm-web/html/processoVisualizacao/processoView.html';
    		
	    var paginaGerenciamento = '/cit-ecm-web/html/gerenciarProcesso/gerenciarProcesso.html';
	    	
	    	if (appService.existsWorkspace(paginaGerenciamento)) {
				$scope.$openModalConfirm({message:$translate.instant('ECM.MSG.EXISTE_PROCESSO_ABERTO'), callback: function(){
					$scope.$modalConfirmInstance.dismiss('cancel');
					$scope.setLoading(true);
					$scope.openWorkspaceIfNotOpen($translate.instant('ECM.LABEL.PROCESSO'), pagina, 'mod-orange');
					$timeout(function() { 
						angular.element('#visualizarGerenciarProcesso').scope().getProcessoVH($scope.docSelecionado.objeto === 'Processo'? $scope.docSelecionado.id : $scope.docSelecionado.idProcesso);
							
				   }, 100);
					$scope.setLoading(false);
				}});
			}else{
				$scope.openWorkspaceIfNotOpen($translate.instant('ECM.LABEL.PROCESSO'), pagina, 'mod-orange');
				$timeout(function() { 
					angular.element('#visualizarGerenciarProcesso').scope().getProcessoVH($scope.docSelecionado.objeto === 'Processo'? $scope.docSelecionado.id : $scope.docSelecionado.idProcesso);
						
			   }, 100);
			};
    	};
	
    $scope.init();    
}]);

