'use strict';

citApp.controller('TipoProcessoController',['$scope', 'TipoProcessoRepository', '$translate', '$timeout', 'DominioRepository', 'PlanoClassificacaoRepository', '$filter', 'BusinessProcessRepository', 'ProcessoRepository', 'SigiloRepository', function TipoProcessoController($scope, TipoProcessoRepository, $translate, $timeout, DominioRepository, PlanoClassificacaoRepository, $filter, BusinessProcessRepository, ProcessoRepository, SigiloRepository) {

    $scope.tipoProcesso = {
    		tipoProcessoPlanoClassificacoes : []
    };
    
    $scope.sigilosSelecionados = [];
    
    // Limpa formulário para novo cadastro
    $scope.resetForm = function() {
        $scope.limparTipoProcesso();
        $scope.edit = true;
        $scope.findDominioNivelAcesso(true);
        $scope.findSigiloList();
        $scope.sigilosSelecionados = [];
        $scope.isSigilo = false;
        $timeout(function(){
            $scope.tipoProcessoForm.$submitted = false;
            $scope.tipoProcessoForm.$setPristine();
        });
    };

    // Atualiza pagina de pesquisa
    $scope.atualizaPaginaPesquisa = function () {
        angular.element('#searchTipoProcesso').scope().fetchResult();
    };

    // MODAL QUE CONFIRMA REMOVER DO TIPO PROCESSO
    $scope.remove = function(tipoProcesso){
    	
    	ProcessoRepository.existeVinculo({"joinClass": 'tipoProcesso.id', "id": tipoProcesso.id}).then(function(result) {
			
    		if(result){
    			$scope.showAlert('error', $translate.instant('ECM.MSG.EXISTE_VINCULO_PROCESSO'));
    			
    		}else{
    	        $scope.tipoProcesso = tipoProcesso;
    	        $scope.$openModalConfirm({
    	            message: $translate.instant('MSG.CONFIRMA_EXCLUSAO'),
    	            callback: function () {
    	                TipoProcessoRepository.remove($scope.tipoProcesso).then(function() {

    	                    $scope.$modalConfirmInstance.dismiss('cancel');
    	                    $scope.showAlert("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
    	                    angular.element('#searchTipoProcesso').scope().fetchResult();

    	                    $scope.resetForm();
    	                });
    	            }
    	        });
    		}
		});
    };

    // SALVA O TipoProcesso
    $scope.saveOrUpdate = function(){
        $scope.tipoProcessoForm.$submitted = true;

        //verifica se o formulario está valido para salvar
        if($scope.tipoProcessoForm.$valid && (!$scope.isSigilo || $scope.isTipoSigiloSelecionado())){

            $scope.setLoadingSalva(true);
            
            TipoProcessoRepository.save($scope.tipoProcesso).then(function(result) {
                $scope.tipoProcesso = result.originalElement;
                $scope.showAlert("success", $translate.instant('MSG.REGISTRO_SALVO'));
                $scope.tipoProcessoForm.$submitted = false;
                $scope.resetForm();
            });
            $scope.setLoading(false);
        }else{
            //Mensagem de erro de campos obrigatorios não preenchidos
            $scope.showAlert('error', $translate.instant('MSG.MN001'), " ", false);
        };
    };
    
    // Limpa o formulario preenchido
    $scope.limparTipoProcesso = function(){
        $scope.tipoProcesso = {	
        		tipoProcessoPlanoClassificacoes : [],
        		acessos : []
        		
        };
        $scope.processoNegocioSelecionado = null;
    };

    // Consulta entidade e mostra no formulario
    $scope.getTipoProcesso = function(tipoProcesso, edit){
    	$scope.processoNegocioSelecionado = {};
        $scope.setLoadingGet(true);
        $scope.findSigiloList();
        TipoProcessoRepository.get(tipoProcesso.id).then(function(result) {
            $scope.tipoProcesso = result.originalElement;
            $scope.edit = edit;
           //Solução temporaria
            if($scope.tipoProcesso.processoNegocio != null){
            	$scope.processoNegocioSelecionado = {           		
            			id : 1
            	};            
            	$scope.processoNegocioSelecionado.name = $scope.tipoProcesso.processoNegocio;           	
            }else{
            	$scope.processoNegocioSelecionado = null;
            }
            
            $scope.montarNivelAcesso();
            
            $scope.verificarSigiloSelecionado();
            
            $scope.setLoading(false);
            
            $timeout(function(){
                $scope.tipoProcessoForm.$submitted = false;
                $scope.tipoProcessoForm.$setPristine();
            });
        });
    };
    
    //Adiciona assunto a lista de assuntos #INICIO
    
    $scope.findAutoCompleteSugestao = function(value){
        return PlanoClassificacaoRepository.findAssuntoAutoComplete(value).then(function(result) {
        	return $filter('idNotEqualObj')(result, $scope.tipoProcesso.tipoProcessoPlanoClassificacoes, 'planoClassificacao');
        	});
    	};
    
	$scope.setAssunto = function(value){
		$scope.TipoProcessoPlanoClassificacao = {
				planoClassificacao : value
		};
		
		$scope.tipoProcesso.tipoProcessoPlanoClassificacoes.push($scope.TipoProcessoPlanoClassificacao);

		$timeout(function(){
			$scope.assunto = null;
		});
	};
	
	//MONTA E EXCLUIR LISTA DE ASSUNTOS SELECIOANDO #INICIO
	
	$scope.montaListaDelete = function(){
		var excluir = false;

		for (var i = $scope.tipoProcesso.tipoProcessoPlanoClassificacoes.length - 1; i >= 0; i--){
			if($scope.tipoProcesso.tipoProcessoPlanoClassificacoes[i].$selected){
				excluir = true;
				break;
			}
		}

		if(excluir){
			$scope.$openModalConfirm({
				message: $translate.instant("MSG.DESEJA_EXCLUIR_ITENS"), callback: function () {
					for (var i = $scope.tipoProcesso.tipoProcessoPlanoClassificacoes.length - 1; i >= 0; i--){

						if($scope.tipoProcesso.tipoProcessoPlanoClassificacoes[i].$selected){
							if($scope.tipoProcesso.tipoProcessoPlanoClassificacoes[i].planoClassificacao.id !== undefined){								
								$scope.tipoProcesso.tipoProcessoPlanoClassificacoes.splice(i, 1).planoClassificacao;
							}
						}
					}
					
					$timeout(function(){
						$scope.tipoProcessoForm.$submitted = false;
						$scope.tipoProcessoForm.$setPristine();
					});

					if($scope.selectedAll){
						$scope.selectedAll = false;
					}
					
					$scope.$modalConfirmInstance.dismiss('cancel');
					$scope.showAlert("warning", $translate.instant("ECM.MSG.ITEM_EXCLUIDO"));
				}
			});

		}else{
			$scope.showAlert("warning", $translate.instant("MSG.SELECIONE_ITEM_EXCLUIR"));
		}
	};
	//MONTA E EXCLUIR LISTA DE ASSUNTOS SELECIOANDO #FIM
	
	//SELECIONA TODOS ITENS DA LISTA DE ASSUNTO	 #INICIO
	$scope.checkAll = function(value) {
		angular.forEach($scope.tipoProcesso.tipoProcessoPlanoClassificacoes, function (item) {
			item.$selected = value;
        });
	};
	
	$scope.verificaSelectedAll = function(PlanoClassificacao) {
		var verificaSelected = true;

		if(!PlanoClassificacao.$selected){
			$scope.selectedAll = false;
		}else{
			angular.forEach($scope.tipoProcesso.tipoProcessoPlanoClassificacoes, function (item) {
				if(!item.$selected){
					verificaSelected = false;
				}
	        });

			if(verificaSelected){
				$scope.selectedAll = true;
			}
		}
	};
	//SELECIONA TODOS ITENS DA LISTA DE ASSUNTO	 #FIM
	
	//AUTOCOMPLETE FLUXO #INICIO
	
	$scope.findProcessoNegocio = function(value) {
		return BusinessProcessRepository.findAutoComplete('name',value).then(function(result) {
			return result;
		});
	};

	$scope.setProcessoNegocio = function (item) {
		 $scope.tipoProcesso.processoNegocio =  item.name;
	};

	
	$scope.limparProcessoNegocio = function() {
		$scope.processoNegocioSelecionado = null;
		 $scope.tipoProcesso.processoNegocio = null;
	};
	
	//AUTOCOMPLETE FLUXO #FIM
	
	//Inicia o nivel acesso publico como true.
	$scope.iniciarNivelAcessoPublico = function(){
		 // checked true no tipoNivelAcesso público.
		 $.grep($scope.tipoNivelAcessoList, function(e){ return e.codigo === 2; })[0].checked = true;
		 var nivelAcessoTipoProcesso = {};
		 nivelAcessoTipoProcesso.nivelAcesso = $.grep($scope.tipoNivelAcessoList, function(e){ return e.codigo === 2; })[0].originalElement;
		 //add o nivel acesso público
		 $scope.tipoProcesso.acessos.push(nivelAcessoTipoProcesso);
	};
	
	//Adiciona assunto a lista de assuntos #FIM
	//Setar nivel acesso.
	$scope.setNivelAcesso = function(nivelAcesso){
		
		 $timeout(function(){
			 //verifica se o nivelAcesso foi checado
			if(nivelAcesso.checked){
				
				var nivelAcessoTipoProcesso = {};
				nivelAcessoTipoProcesso.nivelAcesso = nivelAcesso.originalElement;
				if(nivelAcesso.codigo === 0){
					nivelAcessoTipoProcesso.sigilos = [];
					 $scope.isSigilo = true;
					 $scope.setNivelRestrito(nivelAcesso);
					 $scope.setSigilosoAutomatico();
				}
				
				$scope.tipoProcesso.acessos.push(nivelAcessoTipoProcesso);
				
			}else{
				//verifica se o nivel acesso removido é sigiloso
				 if(nivelAcesso.codigo == 0){
					 
					 $scope.isSigilo = false;
					 //remove os sigilos do nivelAcesso
					 $.grep($scope.tipoProcesso.acessos, function(e){ return e.nivelAcesso.codigo === 0; })[0].sigilos = [];
					 
					//Remover a seleção dos sigilos selecionados
					removerSelecaoSigilo();
				 }else if(nivelAcesso.codigo == 1){
					 $scope.desabilitaNivelSigiloso(nivelAcesso);
				 }
				 //Remove nivelAcesso
				 $scope.removeNivelAcesso(nivelAcesso);
			}
			
		 });
	
	};
	

	//Verifica se foi selecionado algum tipo de segilo
	$scope.isTipoSigiloSelecionado = function() {
	    return $scope.isSigilo && $.grep($scope.tipoProcesso.acessos, function(e){ return e.nivelAcesso.codigo === 0; })[0].sigilos.length > 0;
	};
	
	//Remove a seleção dos sigilos selecionados
	var removerSelecaoSigilo = function(){
		 
		 angular.forEach($scope.sigiloList,  function (item, index) {
			 
			 item.checked = false;
		 });
	};
	
	/*Desabilita o nivel de acesso  SIGILOSO quando Nivel acesso RESTRITO for desabilitado * */
	$scope.desabilitaNivelSigiloso = function(nivelAcessoAutomatico){
		angular.forEach($scope.tipoNivelAcessoList, function (item, index) {
			
			if(!$scope.isSigilo && item.codigo == 0 ){
				item.checked = false;
				$scope.setNivelAcesso(item);
				return;
			} else if(nivelAcessoAutomatico && nivelAcessoAutomatico.codigo == 1 && (item.codigo == 0 && item.checked)){
				item.checked = false;
				$scope.setNivelAcesso(item);
				return;
			}
		
		});
		
	};
	
	/*checked no nivel de acesso RESTRITO quando Nivel acesso SIGILOSO for checked
	 * Público  = 2
	 * Restrito = 1
	 * Sigiloso = 0
	 * */
	$scope.setNivelRestrito = function(nivelAcessoAutomatico){
		angular.forEach($scope.tipoNivelAcessoList, function (item, index) {
			if( nivelAcessoAutomatico.codigo == 0 && (item.codigo == 1 && !item.checked)){
				item.checked = true;
				$scope.setNivelAcesso(item);
			}
	    });
	};

	
	//seta o tipoSigilo selecionado 
	
	/*
	 * Reservado     = 0
	 * Secreto       = 1
	 * Ultrassecreto = 2
	 * */
	$scope.setSigilo = function(sigilo){ 

		if(sigilo.checked){
			
			var sigilotipoProcesso = {
					
					sigilo : sigilo 		
							
					
			};
			
			$scope.setSigilosoAutomatico(sigilo);
			
			$.grep($scope.tipoProcesso.acessos, function(e){ return e.nivelAcesso.codigo === 0; })[0].sigilos.push(sigilotipoProcesso);
			
			
		}else{
			//Remove o tipoSigilo da lista de selecionados
			angular.forEach($.grep($scope.tipoProcesso.acessos, function(e){ return e.nivelAcesso.codigo === 0; })[0].sigilos,  function (item, index) {
				
				if(sigilo.tipoSigilo.codigo === item.sigilo.tipoSigilo.codigo){
					
					$.grep($scope.tipoProcesso.acessos, function(e){ return e.nivelAcesso.codigo === 0; })[0].sigilos.splice(index, 1);
				}
			});
			
			$scope.desabilitaNivelSigilosoAutomatico(sigilo);
	
		}
	};
	/*Seta hierarquia de sigilo caso Sigilo
	 * Reservado     = 0
	 * Secreto       = 1
	 * Ultrassecreto = 2
	 * */
	$scope.setSigilosoAutomatico = function(sigiloAutomatico){
		angular.forEach($scope.sigiloList, function (item, index) {
			//Identifica sigilo RESERVADO caso nivel acesso SIGILOSO seja selecionado
			if ($scope.isSigilo && (item.tipoSigilo.codigo == 0 && !item.checked)){
				$timeout(function(){
					item.checked = true;
				    $scope.setSigilo(item);
				});
			    
			}else if( sigiloAutomatico){
				//Identifica sigilo SECRETO caso ULTRASSECRETO seja selecionado
				if( sigiloAutomatico.tipoSigilo.codigo == 2 && (item.tipoSigilo.codigo == 1 && !item.checked)){
					item.checked = true;
					$scope.setSigilo(item);
					return;	
				}
			}
			
	    });
	};
	
	/*
	 * Desabilita nivel acesso SIGILOSO
	 * Codigos
	 * Reservado  = 0
	 * Secreto = 1
	 * Ultrassecreto = 2
	 * */
	$scope.desabilitaNivelSigilosoAutomatico = function(sigilo){
		
	  angular.forEach($scope.sigiloList, function (item, index) {
		 //Chama função para desabilitar SIGILOSO caso sigilo RESERVADO seja selecionado
		  if(sigilo.tipoSigilo.codigo == 0 && (item.tipoSigilo.codigo == 0 && !item.checked)){
			    $scope.isSigilo = false;
			    $scope.desabilitaNivelSigiloso();
			    return;
			    
	     //Identifica sigilo ULTRASSECRETO caso SECRETO seja selecionado
			}else if (sigilo.tipoSigilo.codigo == 1 && (item.tipoSigilo.codigo == 2 && item.checked)){
				item.checked = false;
			    $scope.setSigilo(item);
			    return;
			}
			
	    });
		
	};
	
	//Remove o nivel acesso
	$scope.removeNivelAcesso = function(nivelRemocao){
		
		angular.forEach($scope.tipoProcesso.acessos, function (item, index) {
			
			if(nivelRemocao.id == item.nivelAcesso.id){
				
				$scope.tipoProcesso.acessos.splice(index, 1);  
			}
        });
		
	};
	
	//Verificar se o siligio foi selecionado.
	$scope.verificarSigiloSelecionado = function(){
		$scope.isSigilo = $scope.tipoNivelAcessoList.some(function(item){
			return item.codigo == 0 && item.checked;
			
		});
	};
	
    //monta o nivel de acesso persistido
    $scope.montarNivelAcesso = function(){
    	
    	if( $scope.tipoProcesso.acessos){
    		
    		angular.forEach($scope.tipoNivelAcessoList, function (item) {
    			
    			item.checked = $scope.contemNivelAcesso(item);
    				
    		});
    		
    		angular.forEach($scope.sigiloList, function (item) {
    			
    			item.checked = $scope.contemSigilo(item);
    				
    		});
    		
    	}
    	
    };
    //Verifica se a lista acessos do tipo processo contem nivelAcesso passado por parametro
	$scope.contemNivelAcesso = function(nivelAcesso){
		return $scope.tipoProcesso.acessos.some(function(item){
			if(item.nivelAcesso.codigo == 0){
				$scope.tipoSigilo = item.tipoSigilo;
			}
			return item.nivelAcesso.id == nivelAcesso.id;
			
		});
	};
	
	$scope.contemSigilo = function(sigilo){
		
		var contemSigilo = false;
		
		angular.forEach($scope.tipoProcesso.acessos, function (nivelAcessoTipoProcesso) {
			
			if(nivelAcessoTipoProcesso.nivelAcesso.codigo === 0){
				
				contemSigilo = nivelAcessoTipoProcesso.sigilos.some(function(sigiloTipoDocumento){
					return sigiloTipoDocumento.sigilo.id === sigilo.id;
					
				});
			}
			
		});
		
		return contemSigilo;
	};
	
	//## Iniciar Dominios ##
	$scope.findDominioNivelAcesso = function(iniciarNivelAcessoPublico){
		
		DominioRepository.findAllDominio('tipoNivelAcesso').then(function(result) {
			$scope.tipoNivelAcessoList = result;
			
			if(iniciarNivelAcessoPublico){
				//Selecionar nivel acesso publico por padrão
				$scope.iniciarNivelAcessoPublico();
			}
		});
		
	};
	
	$scope.findDominioNivelAcesso(false);
	
	 //Lista todos os Sigilos do sistema
	$scope.findSigiloList = function(){
		
		$scope.sigiloList = [];
		
		SigiloRepository.getList().then(function(result) {
			
			angular.forEach(result,  function (item) {
				
				$scope.sigiloList.push(item.originalElement);
			});
			
		});
	};
    
	// ##Metodo responsavel por verificar a duplicidade do nome do documento. ##//
	$scope.validaDuplicidadeNome = function(){
		if($scope.tipoProcesso.nome != undefined && $scope.tipoProcesso.nome != ''){
			return TipoProcessoRepository.verificaDuplicidadeNome($scope.tipoProcesso.nome).then(function(result) {
				if(result && result == true){
					$scope.showAlert('error', $translate.instant('ECM.MSG.NOME_TIPO_PROCESSO_EM_USO'), " ", false);
				}
			});
		}
	};
	

}]);
