'use strict';

citApp.controller('UnidadeProcessoController', ['$scope', 'UnidadeProcessoRepository', '$translate', '$timeout', '$filter', 'ProcessoRepository', 'UnidadeRepository', 'HistoricoAlteracaoProcessoRepository', function UnidadeProcessoController($scope, UnidadeProcessoRepository, $translate, $timeout, $filter, ProcessoRepository, UnidadeRepository, HistoricoAlteracaoProcessoRepository) {
	
	$scope.unidade = null;
	
	$scope.historicos = [];
	
	$scope.enviarProcessoVH = {
			
			unidadesProcessoList : [],
	
	       idProcesso : {},
	       
	       concluirProcesso : false
	};    
    
	
    $scope.iniciarEnvio = function(idProcesso, idTask){
    	$scope.setLoadingSalva(true);
        this.enviarProcessoForm = $scope.unidadeProcessoController.enviarProcessoForm;
        $scope.enviarProcessoVH.idProcesso = idProcesso;
        $scope.enviarProcessoVH.idTask = idTask;
        $scope.enviarProcessoVH.unidadesProcessoList = [];
        $scope.unidade = null;
        UnidadeProcessoRepository.findByIdProcesso(idProcesso).then(function(result) {
        	
        	$scope.historicos = result;
        });
        
        $timeout(function(){
            $scope.enviarProcessoForm.$submitted = false;
            $scope.enviarProcessoForm.$setPristine();
        });
        $scope.setLoading(false);
        
    };
    
    //Buscar unidade através do nome
    $scope.findAutoCompleteUnidade = function(value){
        return UnidadeRepository.findAutoComplete('nome', value).then(function(result) {
        	
        	//Filtrar através do historico
        	var filtrolistHistorico = $filter('idNotEqualObj')(result, $scope.historicos, 'unidade');
        	
        	var filtroListUnidadeLogada = null;
        	//Verifica se o usuario contem uma unidade padrão
        	if($scope.usuarioLogado.unidade){
        		//Filtro da unidadeParão
        		filtroListUnidadeLogada = $filter('idNotObject')(filtrolistHistorico, $scope.usuarioLogado.unidade);
        	}
        	 //Filtro das unidade adicionadas 
        	return $filter('idNotEqualObj')(filtroListUnidadeLogada ? filtroListUnidadeLogada : filtrolistHistorico, $scope.enviarProcessoVH.unidadesProcessoList, 'unidade');
        });
    };
    
    //Setar o unidade selecionado
	$scope.addUnidadeProcesso = function(){
		//Verifica se a unidade selecionada contem um grupo padrão
		if($scope.unidadeProcesso.unidade.grupo){
			
			$scope.enviarProcessoVH.unidadesProcessoList.push($scope.unidadeProcesso);
			
			$timeout(function(){
				$scope.unidadeProcesso = null;
			});
		}else{
			 $scope.showAlert('warning', $translate.instant('ECM.MSG.UNIDADE_SEM_GRUPO_PADRAO'));
		}
		

	};
	
	//Enviar processo
	$scope.enviar = function(){
		
		$scope.enviarProcessoForm.$submitted = true;
		
		 //verifica se o formulario está valido para salvar
        if($scope.enviarProcessoVH.unidadesProcessoList.length > 0){
        	
        	if($scope.enviarProcessoVH.concluirProcesso){
        		
        		$scope.$openModalConfirm({message: $translate.instant('ECM.MSG.CONFIRMA_CONCLUIR_UNIDADE'), callback: $scope.saveOrUpdate});
        		
        	}else{
        		$scope.saveOrUpdate();
        	}
        }else{
            //Mensagem de erro de campos obrigatorios não preenchidos
            $scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
        };
	};
	
	//Salvar
	 $scope.saveOrUpdate = function(){
		 
        $scope.setLoadingSalva(true);
        
        UnidadeProcessoRepository.save($scope.enviarProcessoVH).then(function(result) {
        	
        	result.originalElement.unidadesProcessoList.forEach(function (unidadeProcesso, index) {
        		
        		$scope.historicos.push(unidadeProcesso);
        	});
        	//Salvar historico do processo
        	$scope.saveHistoricoProcesso(result.originalElement.unidadesProcessoList);
        	$scope.enviarProcessoVH.unidadesProcessoList = [];
            $scope.enviarProcessoForm.$submitted = false;
            $scope.showAlert("success", $translate.instant('ECM.MSG.PROCESSO_ENVIADO_SUCESSO'));
        	
            //Voltar para tela de gerenciamento de tarefa
            $scope.voltarGerenciamentoTarefa();
            $scope.setLoading(false);           
        });
	        
	   };
	   
	   //Salvar Historico processo
	   $scope.saveHistoricoProcesso = function(unidadesProcessoList){
		   
		   var processo = {
				   
				   id : $scope.enviarProcessoVH.idProcesso
		   };
		   //Gerar historico para cada unidade enviada
		   unidadesProcessoList.forEach(function (unidadeProcesso){
			   var historicoAlteracaoProcesso = {
					   descricaoAcao :  $translate.instant('ECM.MSG_HISTORICO_ECM.ENVIO_PROCESSO'),
					   processo : processo,
					   autor : $scope.usuarioLogado,
					   acaoAlteracao : "ENVIAR_PROCESSO",
					   mapaAtributos : {"unidade" : unidadeProcesso.unidade.nome}
			   };
			   
			   HistoricoAlteracaoProcessoRepository.saveParams(historicoAlteracaoProcesso).then(function(result){
				   
			   }); 
			   
		   });
		   
		   //Verifica se o processo foi concluído na unidade, caso sejá verdadeiro gera o histórico de conclusão
		 	if($scope.enviarProcessoVH.concluirProcesso){
		   
			   var historicoAlteracaoProcesso = {
					   descricaoAcao :  $translate.instant('ECM.MSG_HISTORICO_ECM.CONCLUSAO_PROCESSO_UNIDADE'),
					   processo : processo,
					   autor : $scope.usuarioLogado,
					   acaoAlteracao : "CONCLUSAO_PROCESSO_UNIDADE"
			   };
			   
			   HistoricoAlteracaoProcessoRepository.saveParams(historicoAlteracaoProcesso).then(function(result){
				   
			   }); 
			   
		 	}
	   
	   };

	   //Voltar para tela de gerenciamento de tarefa
	   $scope.voltarGerenciamentoTarefa = function(){
		   
       	//Verifica se o processo está sendo concluido na unidade
       	if($scope.enviarProcessoVH.concluirProcesso){
       		
       		var workspace = angular.element('#editProcesso').scope().workspace;
       		if (workspace) {
       			$scope.$modalConfirmInstance.dismiss('cancel');
       			angular.element("#citapp-controller").scope().removeWorkspace(workspace.id);
       			
       			angular.element("#citapp-controller").scope().showAlert("success", $translate.instant('MSG.PROCESSO_ENVIADO_SUCESSO'));
       		}
       		
       		 $timeout(function(){
       			 angular.element('#idGerenciamentoTarefas').scope().fetchResult();
       		 });
       		
       	}else{
       		
            $timeout(function(){
            	$scope.$parent.$parent.closeWidget();
            	$scope.$parent.$parent.atualizarAtribuicaoProcesso($scope.enviarProcessoVH.idProcesso);
            },200);
            
       	}
	   };
	    
	$scope.exibirDialogRemocao = function(){
		
		if(!$scope.unidadeProcessoChecked){
			
			 $scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));
			
		}else{
			
			$scope.$openModalConfirm({message: $translate.instant('LABEL.CONFIRMA_EXCLUSAO'), callback: $scope.remove});
		}

	};
	
	//Remove unidadeProcesso
	$scope.remove = function(){
		
		 var index =  $scope.enviarProcessoVH.unidadesProcessoList.indexOf($scope.unidadeProcessoChecked);
			
		$scope.showAlert("success", $translate.instant('MSG.EXCLUSAO_SUCESSO'));
		
		$scope.enviarProcessoVH.unidadesProcessoList.splice(index , 1);
		
		$scope.$modalConfirmInstance.dismiss('cancel');
   };
		
    //Fechar widget
	$scope.closeWidget = function() {
    	angular.element("#searchGerenciarProcesso").scope().closeWidget();
    };
	
	
}]);
